#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.DxExtensions.MemoEdit
{
    using System;
    using System.Collections;
    using System.Windows.Forms;
    using DevExpress.XtraEditors;

    /// <summary>
    /// Extension methods to make some DevExpress tasks easier.
    /// </summary>
    public static class DxExtensionsMemoEdit
    {

        /// <summary>
        /// Appends the specified text to the memo. Adds text to the end of the
        /// last line of the memo. No new line characters are added.  Passing null 
        /// as the text will not cause an exception but will also not add a new 
        /// line (functionally a NOP).
        /// </summary>
        /// <param name="text">The text.</param>
        public static void Append(this MemoEdit memo, string text)
        {
            GuardAgainst.ArgumentIsNull(memo, "memo");
            TextBox textBox = GetMemoEditInnerTextBox(memo);
            if (textBox == null)
                throw new InvalidOperationException("Inner TextBox not found.");
            else if (text != null)
                textBox.AppendText(text);
        }

        /// <summary>
        /// Appends a new line (no text) to the MemoEdit. If the memo is empty the
        /// text is placed on the first line and no new line character is added.
        /// If there is already text in the memo, the new line is added to the
        /// front of the text. This behavior keeps it in line with the
        /// AppendLineNoScroll method. The MemoEdit is scrolled so that the newly
        /// appended text is visible in the MemoEdit.
        /// </summary>
        public static void AppendLine(this MemoEdit memo)
        {
            memo.AppendLine(String.Empty);
        }

        /// <summary>
        /// Appends a new line of text to the MemoEdit. If the memo is empty the
        /// text is placed on the first line and no new line character is added.
        /// If there is already text in the memo, the new line is added to the
        /// front of the text. This behavior keeps it in line with the
        /// <see cref"AppendLineNoScrool">AppendLineNoScroll</see> method. The 
        /// MemoEdit is scrolled so that the newly appended text is visible in
        /// the MemoEdit.
        /// </summary>
        /// <param name="text">The text.</param>
        public static void AppendLine(this MemoEdit memo, string text)
        {
            if (memo.Lines.Length > 0)
            {
                if (text == null)
                    text = Environment.NewLine;
                else
                    text = Environment.NewLine + text;
            }
            memo.Append(text);
        }

        /// <summary>
        /// Appends a new line without additional text to the MemoEdit. This is
        /// done without scrolling the MemoEdit.
        /// </summary>
        public static void AppendLineNoScroll(this MemoEdit memo)
        {
            memo.AppendLineNoScroll(String.Empty);
        }

        /// <summary>
        /// Appends the text onto a new line in the MemoEdit. This is done without
        /// scrolling the MemoEdit. Passing null as the text will not cause an
        /// exception but will also not add a new line (functionally a NOP).
        /// </summary>
        /// <param name="text">The text.</param>
        public static void AppendLineNoScroll(this MemoEdit memo, string text)
        {
            GuardAgainst.ArgumentIsNull(memo, "memo");
            ArrayList array = new ArrayList(memo.Lines);
            array.Add(text);
            memo.Lines = array.ToArray(typeof(string)) as string[];
        }

        private static TextBox GetMemoEditInnerTextBox(TextEdit editor)
        {
            if (editor != null)
                foreach (Control control in editor.Controls)
                    if (control is TextBox)
                        return (TextBox)control;
            return null;
        }
    }
}
