#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Interface
{
    using System;

    /// <summary>
    /// Object to hold arguments for the ExceptionHandler.
    /// </summary>
    public class ExceptionHandlerArgs
    {

        /// <summary>
        /// When the ExceptionHandler has to terminate the application the
        /// DefaultExceptionExitCode is passed back to the operating system.
        /// </summary>
        public static int DefaultExceptionExitCode = 999;


        /// <summary>
        /// Initializes a new instance of the <see cref="ExceptionHandlerArgs"/> class.
        /// </summary>
        /// <param name="caption">The dialog box caption.</param>
        /// <param name="isFatal">if set to <c>true</c> the exception is fatal and the application will terminate.</param>
        /// <param name="exitCode">The exit code to pass back to the operating system.</param>
        /// <param name="whatHappenedText">The what happened text.</param>
        /// <param name="howAffectedText">The how am I affected text.</param>
        /// <param name="whatToDoText">The what to do text.</param>
        /// <param name="emailInformationText">The email information text.</param>
        /// <param name="whatHappenedCaption">The what happened caption.</param>
        /// <param name="howAffectedCaption">The how affected caption.</param>
        /// <param name="whatToDoCaption">The what to do caption.</param>
        /// <param name="emailInformationCaption">The email information caption.</param>
        /// <param name="emailSubjectText">The text of the email subject.</param>
        public ExceptionHandlerArgs(string caption, bool isFatal, int exitCode,
            string whatHappenedText, string howAffectedText, string whatToDoText,
            string emailInformationText, string whatHappenedCaption, string howAffectedCaption,
            string whatToDoCaption, string emailInformationCaption, string emailSubjectText)
        {
            Caption = caption;
            IsFatal = isFatal;
            ExitCode = exitCode;
            WhatHappenedText = whatHappenedText;
            HowAffectedText = howAffectedText;
            WhatToDoText = whatToDoText;
            EmailInformationText = emailInformationText;
            WhatHappenedCaption = whatHappenedCaption;
            HowAffectedCaption = howAffectedCaption;
            WhatToDoCaption = whatToDoCaption;
            EmailInformationCaption = emailInformationCaption;
            EmailSubjectText = emailSubjectText;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExceptionHandlerArgs"/> class.
        /// </summary>
        /// <param name="caption">The caption.</param>
        /// <param name="isFatal">if set to <c>true</c> the exception is fatal and the application will terminate.</param>
        /// <param name="exitCode">The exit code to pass back to the operating system.</param>
        /// <param name="whatHappenedText">The what happened text.</param>
        /// <param name="howAffectedText">The how affected text.</param>
        /// <param name="whatToDoText">The what to do text.</param>
        /// <param name="emailInformationText">The email information text.</param>
        public ExceptionHandlerArgs(string caption, bool isFatal, int exitCode, string whatHappenedText,
            string howAffectedText, string whatToDoText, string emailInformationText)
            : this(caption, isFatal, exitCode, whatHappenedText, howAffectedText,
            whatToDoText, emailInformationText, String.Empty, String.Empty, String.Empty, String.Empty, String.Empty) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExceptionHandlerArgs"/> class.
        /// </summary>
        /// <param name="caption">The caption.</param>
        /// <param name="isFatal">if set to <c>true</c> the exception is fatal and the application will terminate.</param>
        /// <param name="exitCode">The exit code to pass back to the operating system.</param>
        public ExceptionHandlerArgs(string caption, bool isFatal, int exitCode)
            : this(caption, isFatal, exitCode, String.Empty, String.Empty, String.Empty, String.Empty) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExceptionHandlerArgs"/> class.
        /// </summary>
        /// <param name="caption">The dialog box caption.</param>
        public ExceptionHandlerArgs(string caption)
            : this(caption, false, DefaultExceptionExitCode) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExceptionHandlerArgs"/> class.
        /// </summary>
        public ExceptionHandlerArgs()
            : this(String.Empty) { }


        /// <summary>
        /// Gets or sets the dialog box caption.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The caption.</value>
        public string Caption { get; set; }

        /// <summary>
        /// Gets or sets the email information caption.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The email information caption.</value>
        public string EmailInformationCaption { get; set; }

        /// <summary>
        /// Gets or sets the email information text.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The email information text.</value>
        public string EmailInformationText { get; set; }

        /// <summary>
        /// Gets or sets the email subject text. EmailSubjectText ultimately calls
        /// String.Format with two format items:
        /// <list type="bullet">
        /// <item>{0}: Exception type as a string</item>
        /// <item>{1}: The product name</item>
        /// </list>
        /// </summary>
        /// <value>The email subject text.</value>
        public string EmailSubjectText { get; set; }
       
        /// <summary>
        /// Gets or sets the exit code to pass back to the operating system.
        /// </summary>
        /// <value>The exit code.</value>
        public int ExitCode { get; set; }

        /// <summary>
        /// Gets or sets the how affected caption.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The how affected caption.</value>
        public string HowAffectedCaption { get; set; }

        /// <summary>
        /// Gets or sets the how am I affected text.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The how am I affected text.</value>
        public string HowAffectedText { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this exception is fatal.
        /// </summary>
        /// <value><c>true</c> if this exception is fatal; otherwise, <c>false</c>.</value>
        public bool IsFatal { get; set; }

        /// <summary>
        /// Gets or sets the what happened caption.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The what happened caption.</value>
        public string WhatHappenedCaption { get; set; }

        /// <summary>
        /// Gets or sets the what happened text.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The what happened text.</value>
        public string WhatHappenedText { get; set; }

        /// <summary>
        /// Gets or sets the what to do caption.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The what to do caption.</value>
        public string WhatToDoCaption { get; set; }

        /// <summary>
        /// Gets or sets the what to do text.
        /// <para>Ultimately String.Format is called with a single format option of product name.</para>
        /// </summary>
        /// <value>The what to do text.</value>
        public string WhatToDoText { get; set; }
    }
}
