#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Interface
{
    using System;
    using System.Windows.Forms;

    /// <summary>
    /// IExceptionHandler defines how a class implements global exception handling.
    /// </summary>
    public interface IExecptionHandler
    {

        /// <summary>
        /// Gets the default exit code if the program is forced to close on a fatal exception.
        /// </summary>
        /// <value>The default exit code.</value>
        int DefaultExitCode { get; }

        /// <summary>
        /// Gets or sets the name of the product. This is displayed in some of the explanatory text.
        /// </summary>
        /// <value>The name of the product.</value>
        string ProductName { get; set; }

        /// <summary>
        /// Gets or sets the send exception email handler.
        /// </summary>
        /// <value>The send exception email handler.</value>
        ISendExceptionEmail SendExceptionEmail { get; set; }

        /// <summary>
        /// Gets or sets the services helper for logging and localization.
        /// </summary>
        /// <value>The services helper.</value>
        IServicesHelper ServicesHelper { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether the dialog box should show email controls.
        /// </summary>
        /// <value><c>true</c> if showing email controls; otherwise, <c>false</c>.</value>
        bool ShowEmailControls { get; set; }

        /// <summary>
        /// Gets a value indicating whether this handler supports emailing the exception back to your company.
        /// </summary>
        /// <value><c>true</c> if email is supported; otherwise, <c>false</c>.</value>
        bool SupportsEmail { get; }

        /// <summary>
        /// Gets or sets the user defined additional information. If this is non-null then the associated
        /// UserControl will be injected into the global exception handler dialog.
        /// </summary>
        /// <value>The user defined additional information.</value>
        IExceptionHandlerUserDefined UserDefined { get; set; }

        /// <summary>
        /// Attaches the event handlers.
        /// </summary>
        void AttachEventHandlers(bool useExceptionHandlerDuringDebugging);

        /// <summary>
        /// Gets and formats exception details. This will be a multi-line string
        /// with information about the exception and all of the loaded assemblies
        /// within the AppDomain.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <returns>Exception detail string.</returns>
        string ExceptionDetails(Exception exception);

        /// <summary>
        /// Gets the exception display text. This allows you to obtain the display text of the dialog.
        /// </summary>
        /// <param name="defaultText">The default text to obtain.</param>
        /// <returns>The display text associated with the supplied ExceptionHandlerDisplayText.</returns>
        string GetExceptionDisplayText(ExceptionHandlerDisplayText defaultText);

        /// <summary>
        /// Sets the exception display text. This allows you to customize the display text of the dialog.
        /// </summary>
        /// <param name="defaultText">The default text to set.</param>
        /// <param name="value">The value of the display text associated with the supplied ExceptionHandlerDisplayText.</param>
        void SetExceptionDisplayText(ExceptionHandlerDisplayText defaultText, string value);

        /// <summary>
        /// Shows the specified exception.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <returns>
        /// DialogResult of the dialog showing the exception details.
        /// </returns>
        DialogResult Show(Exception exception);

        /// <summary>
        /// Shows the specified exception.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <param name="args">Information about the exception.</param>
        /// <returns>
        /// DialogResult of the dialog showing the exception details.
        /// </returns>
        DialogResult Show(Exception exception, ExceptionHandlerArgs args);
    }
}
