#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Interface
{
    using System;
    using System.Drawing;
    using System.Reflection;

    /// <summary>
    /// Value is associated with the current user or the local machine.
    /// </summary>
    public enum UserMachine
    {
        /// <summary>
        /// Value is associated with the current user.
        /// </summary>
        CurrentUser,
        /// <summary>
        /// Value is associated with the local machine.
        /// </summary>
        LocalMachine
    }

    /// <summary>
    /// Whether this value is associated with the current application or all applications from this company.
    /// </summary>
    public enum AppCommon
    {
        /// <summary>
        /// Value is associated with the current application.
        /// </summary>
        ThisApplication,
        /// <summary>
        /// Value is associated with all applications from this company.
        /// </summary>
        AllApplications
    }

    /// <summary>
    /// <para>DevExpressContrib cannot work in complete isolation. Many application
    /// frameworks will have configuration, logging as well as localization tools.
    /// To facilitate the various configuration, logging and localization services 
    /// different implementations of IServicesHelper can be passed in to customize
    /// these needs.</para>
    /// 
    /// <para>If you don't care about these services a default implementation
    /// will be created as needed.</para>
    /// </summary>
    public interface IServicesHelper
    {

        /// <summary>
        /// Gets the localized string.
        /// </summary>
        /// <param name="localizerString">The localizer key.</param>
        /// <returns>The localized string.</returns>
        string GetString(string localizerString);

        /// <summary>
        /// Gets the localized string.
        /// </summary>
        /// <param name="localizerString">The localizer key.</param>
        /// <param name="callingAssembly">The calling assembly.</param>
        /// <returns>The localized string.</returns>
        string GetString(string localizerString, Assembly callingAssembly);

        /// <summary>
        /// Logs an error.
        /// </summary>
        /// <param name="message">The message.</param>
        void LogError(string message);

        /// <summary>
        /// Logs the exception.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        void LogException(string message, Exception exception);

        /// <summary>
        /// Logs the exception.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="details">The exception details.</param>
        void LogException(string message, Exception exception, string details);

        /// <summary>
        /// Logs the exception.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="details">The exception details.</param>
        /// <param name="screenshot">Screenshot of the exception.</param>
        void LogException(string message, Exception exception, string details, Image screenshot);

        /// <summary>
        /// Logs a fatal error.
        /// </summary>
        /// <param name="message">The message.</param>
        void LogFatal(string message);

        /// <summary>
        /// Logs an informational message.
        /// </summary>
        /// <param name="message">The message.</param>
        void LogInformation(string message);

        /// <summary>
        /// Logs a warning.
        /// </summary>
        /// <param name="message">The message.</param>
        void LogWarning(string message);

        /// <summary>
        /// Reads a string value from a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>Value associated with the key.</returns>
        string ReadConfigString(string key);

        /// <summary>
        /// Reads a string value from a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <returns>Value associated with the key.</returns>
        string ReadConfigString(string key, UserMachine userMachine);

        /// <summary>
        /// Reads a string value from a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <param name="appCommon">Whether this value is associated with the current application or all applications from this company.</param>
        /// <returns>Value associated with the key.</returns>
        string ReadConfigString(string key, UserMachine userMachine, AppCommon appCommon);

        /// <summary>
        /// Writes a string value to a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        void WriteConfigString(string key, string value);

        /// <summary>
        /// Writes a string value to a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        void WriteConfigString(string key, string value, UserMachine userMachine);

        /// <summary>
        /// Writes a string value to a configuration system.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <param name="appCommon">Whether this value is associated with the current application or all applications from this company.</param>
        void WriteConfigString(string key, string value, UserMachine userMachine, AppCommon appCommon);
    }
}
