#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Ux
{
    using System.Drawing;
    using System.Media;
    using System.Windows.Forms;
    using DevExpress.XtraEditors;
    using DevExpress.XtraLayout;
    using DevExpress.XtraLayout.Utils;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// The dialog box showing the unhandled exception information.
    /// </summary>
    public partial class DxThreadExceptionDialog : XtraForm
    {

        private DxThreadExceptionUserControl dxThreadExceptionUserControl;
        private LayoutControlItem dxThreadExceptionUserControlLayoutItem;


        /// <summary>
        /// Initializes a new instance of the <see
        /// cref="T:DevExpressContrib.Core.Ux.DxThreadExceptionDialog">DxThreadExceptionDialog</see>
        /// class.
        /// </summary>
        /// <param name="servicesHelper">The services helper.</param>
        public DxThreadExceptionDialog(IServicesHelper servicesHelper)
        {
            InitializeComponent();
            dxThreadExceptionUserControl = new DxThreadExceptionUserControl(servicesHelper);
            dxThreadExceptionUserControlLayoutItem = new LayoutControlItem { TextVisible = false, Control = dxThreadExceptionUserControl };
            userControlLayoutGroup.Add(dxThreadExceptionUserControlLayoutItem);
        }


        /// <summary>
        /// Gets or sets the user defined additional information.
        /// </summary>
        /// <value>The user defined additional information.</value>
        public IExceptionHandlerUserDefined UserDefined
        {
            get { return dxThreadExceptionUserControl.UserDefined; }
            set { dxThreadExceptionUserControl.UserDefined = value; }
        }

        /// <summary>
        /// Gets or sets the exception detail text displayed in the Details tab.
        /// </summary>
        /// <value>The detail text.</value>
        public string DetailText
        {
            get { return dxThreadExceptionUserControl.DetailText; }
            set { dxThreadExceptionUserControl.DetailText = value; }
        }

        /// <summary>
        /// Gets or sets the email information group caption.
        /// </summary>
        /// <value>The email information group caption.</value>
        public string EmailInformationCaption
        {
            get { return dxThreadExceptionUserControl.EmailInformationCaption; }
            set { dxThreadExceptionUserControl.EmailInformationCaption = value; }
        }

        /// <summary>
        /// Gets or sets the email information text.
        /// </summary>
        /// <value>The email information text.</value>
        public string EmailInformationText
        {
            get { return dxThreadExceptionUserControl.EmailInformationText; }
            set { dxThreadExceptionUserControl.EmailInformationText = value; }
        }

        /// <summary>
        /// Gets or sets from email.
        /// </summary>
        /// <value>From email.</value>
        public string FromEmail
        {
            get { return dxThreadExceptionUserControl.FromEmail; }
            set { dxThreadExceptionUserControl.FromEmail = value; }
        }

        /// <summary>
        /// Gets or sets the how affected group caption.
        /// </summary>
        /// <value>The how affected group caption.</value>
        public string HowAffectedCaption
        {
            get { return dxThreadExceptionUserControl.HowAffectedCaption; }
            set { dxThreadExceptionUserControl.HowAffectedCaption = value; }
        }

        /// <summary>
        /// Gets or sets the how is the user affected text.
        /// </summary>
        /// <value>The how is the user affected text.</value>
        public string HowAffectedText
        {
            get { return dxThreadExceptionUserControl.HowAffectedText; }
            set { dxThreadExceptionUserControl.HowAffectedText = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the user wants the screenshot 
        /// included in the email.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the screenshot can be included in the email; otherwise, <c>false</c>.
        /// </value>
        public bool IncludeScreenshotInEmail
        {
            get { return dxThreadExceptionUserControl.IncludeScreenshotInEmail; }
            set { dxThreadExceptionUserControl.IncludeScreenshotInEmail = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this exception is fatal. If it is
        /// fatal then the 'Continue' button will not be shown.
        /// </summary>
        /// <value><c>true</c> if this instance is fatal; otherwise, <c>false</c>.</value>
        public bool IsFatal
        {
            get { return continueLayoutItem.Visibility == LayoutVisibility.Never; }
            set
            {
                if (value)
                    continueLayoutItem.Visibility = LayoutVisibility.Never;
                else
                    continueLayoutItem.Visibility = LayoutVisibility.Always;
            }
        }

        /// <summary>
        /// Gets or sets the screenshot.
        /// </summary>
        /// <value>The screenshot.</value>
        public Image Screenshot
        {
            get { return dxThreadExceptionUserControl.Screenshot; }
            set { dxThreadExceptionUserControl.Screenshot = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the user wants to send the email.
        /// </summary>
        /// <value><c>true</c> if the user wants to send the email; otherwise, <c>false</c>.</value>
        public bool SendEmail
        {
            get { return dxThreadExceptionUserControl.SendEmail; }
            set { dxThreadExceptionUserControl.SendEmail = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the email controls should be shown on the control.
        /// </summary>
        /// <value><c>true</c> if the email controls should be shown; otherwise, <c>false</c>.</value>
        public bool ShowEmailControls
        {
            get { return dxThreadExceptionUserControl.ShowEmailControls; }
            set { dxThreadExceptionUserControl.ShowEmailControls = value; }
        }

        /// <summary>
        /// Gets or sets the what happened group caption.
        /// </summary>
        /// <value>The what happened group caption.</value>
        public string WhatHappenedCaption
        {
            get { return dxThreadExceptionUserControl.WhatHappenedCaption; }
            set { dxThreadExceptionUserControl.WhatHappenedCaption = value; }
        }

        /// <summary>
        /// Gets or sets the what happened text.
        /// </summary>
        /// <value>The what happened text.</value>
        public string WhatHappenedText
        {
            get { return dxThreadExceptionUserControl.WhatHappenedText; }
            set { dxThreadExceptionUserControl.WhatHappenedText = value; }
        }

        /// <summary>
        /// Gets or sets the what to do group caption.
        /// </summary>
        /// <value>The what to do group caption.</value>
        public string WhatToDoCaption
        {
            get { return dxThreadExceptionUserControl.WhatToDoCaption; }
            set { dxThreadExceptionUserControl.WhatToDoCaption = value; }
        }

        /// <summary>
        /// Gets or sets the what to do text.
        /// </summary>
        /// <value>The what to do text.</value>
        public string WhatToDoText
        {
            get { return dxThreadExceptionUserControl.WhatToDoText; }
            set { dxThreadExceptionUserControl.WhatToDoText = value; }
        }


        /// <summary>
        /// Releases all resources used by the <see cref="T:System.ComponentModel.Component"/>.
        /// </summary>
        /// 
        public new void Dispose()
        {
            if (Screenshot != null)
                Screenshot.Dispose();
            base.Dispose();
        }

        private void DxThreadExceptionDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (dxThreadExceptionUserControl.HasUxErrors)
            {
                SystemSounds.Exclamation.Play();
                e.Cancel = true;
            }
            else if (UserDefined != null)
            {
                UserDefined.FinalizeUI();
                // disconnect the user defined user control from the layout
                dxThreadExceptionUserControl.Parent = null;
                userControlLayoutGroup.Remove(dxThreadExceptionUserControlLayoutItem);
            }
        }
    }
}