#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Ux
{
    using System.Drawing;
    using DevExpress.XtraEditors;
    using DevExpress.XtraLayout;
    using DevExpress.XtraLayout.Utils;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// The user control containing functionality to show user friendly exceptions.
    /// </summary>
    public partial class DxThreadExceptionUserControl : XtraUserControl
    {

        private IExceptionHandlerUserDefined userDefined;
        private LayoutControlItem userDefinedLayoutItem;
        private bool userDefinedInitialized = false;


        /// <summary>
        /// Initializes a new instance of the <see cref="DxThreadExceptionUserControl"/> class.
        /// </summary>
        public DxThreadExceptionUserControl(IServicesHelper servicesHelper)
        {
            InitializeComponent();
            tabbedGroup.SelectedTabPageIndex = 0;
            dxValidationProvider.SetValidationRule(fromEmailTextEdit, new EmailValidationRule(servicesHelper));
        }


        /// <summary>
        /// Gets or sets the user defined additional information.
        /// </summary>
        /// <value>The user defined additional information.</value>
        public IExceptionHandlerUserDefined UserDefined
        {
            get { return userDefined; }
            set
            {
                if (userDefined != value)
                {
                    if (!userDefinedInitialized)
                    {
                        userDefined = value;
                        InitializeUserDefined();
                        userDefinedInitialized = true;
                    }
                    UserDefined.InitializeUI();
                }
            }
        }

        /// <summary>
        /// Gets or sets the exception detail text displayed in the Details tab.
        /// </summary>
        /// <value>The detail text.</value>
        public string DetailText
        {
            get { return detailsMemo.Text; }
            set { detailsMemo.Text = value; }
        }

        /// <summary>
        /// Gets or sets the email information group caption.
        /// </summary>
        /// <value>The email information group caption.</value>
        public string EmailInformationCaption
        {
            get { return emailLayoutControlGroup.Text; }
            set { emailLayoutControlGroup.Text = value; }
        }

        /// <summary>
        /// Gets or sets the email information text.
        /// </summary>
        /// <value>The email information text.</value>
        public string EmailInformationText
        {
            get { return emailInformationLabel.Text; }
            set { emailInformationLabel.Text = value; }
        }

        /// <summary>
        /// Gets or sets from email.
        /// </summary>
        /// <value>From email.</value>
        public string FromEmail
        {
            get { return fromEmailTextEdit.Text; }
            set { fromEmailTextEdit.Text = value; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance has input errors.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance has input errors; otherwise, <c>false</c>.
        /// </value>
        public bool HasUxErrors
        {
            get { return dxValidationProvider.GetInvalidControls().Count > 0; }
        }

        /// <summary>
        /// Gets or sets the how affected group caption.
        /// </summary>
        /// <value>The how affected group caption.</value>
        public string HowAffectedCaption
        {
            get { return howAffectedLayoutControlGroup.Text; }
            set { howAffectedLayoutControlGroup.Text = value; }
        }

        /// <summary>
        /// Gets or sets the how is the user affected text.
        /// </summary>
        /// <value>The how is the user affected text.</value>
        public string HowAffectedText
        {
            get { return howAffectedLabel.Text; }
            set { howAffectedLabel.Text = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the user wants the screenshot 
        /// included in the email.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the screenshot can be included in the email; otherwise, <c>false</c>.
        /// </value>
        public bool IncludeScreenshotInEmail
        {
            get { return includeScreenshotInEmailCheckBox.Checked; }
            set { includeScreenshotInEmailCheckBox.Checked = value; }
        }

        /// <summary>
        /// Gets or sets the screenshot.
        /// </summary>
        /// <value>The screenshot.</value>
        public Image Screenshot
        {
            get { return screenshotPicture.Image; }
            set { screenshotPicture.Image = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the user wants to send the email.
        /// </summary>
        /// <value><c>true</c> if the user wants to send the email; otherwise, <c>false</c>.</value>
        public bool SendEmail
        {
            get { return sendEmailCheckBox.Checked; }
            set { sendEmailCheckBox.Checked = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the email controls should be shown on the control.
        /// </summary>
        /// <value><c>true</c> if the email controls should be shown; otherwise, <c>false</c>.</value>
        public bool ShowEmailControls
        {
            get { return emailLayoutControlGroup.Visible; }
            set
            {
                if (value)
                    emailLayoutControlGroup.Visibility = LayoutVisibility.Always;
                else
                    emailLayoutControlGroup.Visibility = LayoutVisibility.Never;
            }
        }

        /// <summary>
        /// Gets or sets the what happened group caption.
        /// </summary>
        /// <value>The what happened group caption.</value>
        public string WhatHappenedCaption
        {
            get { return whatToDoLayoutControlGroup.Text; }
            set { whatToDoLayoutControlGroup.Text = value; }
        }

        /// <summary>
        /// Gets or sets the what happened text.
        /// </summary>
        /// <value>The what happened text.</value>
        public string WhatHappenedText
        {
            get { return whatHappenedLabel.Text; }
            set { whatHappenedLabel.Text = value; }
        }

        /// <summary>
        /// Gets or sets the what to do group caption.
        /// </summary>
        /// <value>The what to do group caption.</value>
        public string WhatToDoCaption
        {
            get { return whatToDoLayoutControlGroup.Text; }
            set { whatToDoLayoutControlGroup.Text = value; }
        }

        /// <summary>
        /// Gets or sets the what to do text.
        /// </summary>
        /// <value>The what to do text.</value>
        public string WhatToDoText
        {
            get { return whatToDoLabel.Text; }
            set { whatToDoLabel.Text = value; }
        }


        private void InitializeUserDefined()
        {
            if (UserDefined == null)
                return;
            userDefinedLayoutControlGroup.Visibility = LayoutVisibility.Always;
            userDefinedLayoutControlGroup.Text = UserDefined.TabText;
            userDefinedLayoutItem = new LayoutControlItem();
            userDefinedLayoutItem.TextVisible = false;
            userDefinedLayoutItem.Control = UserDefined.UserInterface;
            userDefinedLayoutControlGroup.AddItem(userDefinedLayoutItem);
        }
    }
}
