#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core.Ux
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using DevExpress.XtraEditors;
    using DevExpress.XtraEditors.Controls;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// <para>WatermarkInputPromptManager manages watermark input prompts for 
    /// DevExpress edit controls.</para>
    /// <para>Input Prompt (also known as a Watermark) is one of the interface
    /// design patterns mentioned in "Designing Interfaces" by Jenifer Tidwell.
    /// The idea is to 'Prefill a text field or drop-down with a prompt that tells
    /// the user what to do or type.' (Tidwell pg 225)</para>
    /// <para>One common way to accomplish this is when the edit box is blank 
    /// some explanatory text is placed in the edit box. This text is generally
    /// shown in a muted color.</para>
    /// </summary>
    [ObsoleteAttribute("WatermarkInputPromptManager is no longer necessary. Use Properties.NullValuePrompt instead.", false)]
    public class WatermarkInputPromptManager : IDisposable
    {

        private Color defaultDefaultTextColor = SystemColors.ControlText;
        private Color defaultDefaultWatermarkTextColor = SystemColors.GrayText;
        private string defaultPrompt;
        // BaseEdit descendants that work properly:
        //    ButtonEdit
        //    CalcEdit
        //    ComboBoxEdit
        //    DateEdit
        //    MemoEdit
        //    MRUEdit
        //    TextEdit
        private List<Type> invalidBaseEditDescendantList = new List<Type>() { 
            typeof(BaseSpinEdit),
            typeof(BlobBaseEdit),
            typeof(CheckedComboBoxEdit),
            typeof(CheckEdit),
            typeof(ColorEdit),
            // ComboBox - obsolete
            typeof(FontEdit),
            typeof(HyperLinkEdit),
            typeof(ImageComboBoxEdit),
            typeof(ImageEdit),
            typeof(LookUpEdit),
            typeof(LookUpEditBase),
            typeof(MarqueeProgressBarControl),
            typeof(MemoExEdit),
            // PickImage - obsolete
            typeof(PictureEdit),
            typeof(PopupBaseAutoSearchEdit),
            typeof(PopupBaseEdit),
            typeof(PopupContainerEdit),
            typeof(ProgressBarBaseControl),
            typeof(ProgressBarControl),
            typeof(RadioGroup),
            typeof(RangeTrackBarControl),
            typeof(SpinEdit),
            typeof(TimeEdit),
            typeof(TrackBarControl),
            typeof(ZoomTrackBarControl)
        };
        private IServicesHelper servicesHelper;
        private Dictionary<BaseEdit, WatermarkInputPromptItem> watermarkControls;


        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultPrompt">The default prompt.</param>
        /// <param name="defaultWatermarkTextColor">Default color of the watermark text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="defaultTextColor">Default color of the text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public WatermarkInputPromptManager(string defaultPrompt, Color defaultWatermarkTextColor, Color defaultTextColor, IServicesHelper servicesHelper)
        {
            watermarkControls = new Dictionary<BaseEdit, WatermarkInputPromptItem>();
            if (defaultWatermarkTextColor == Color.Empty)
                DefaultWatermarkTextColor = defaultDefaultWatermarkTextColor;
            else
                DefaultWatermarkTextColor = defaultWatermarkTextColor;
            if (defaultTextColor == Color.Empty)
                DefaultTextColor = defaultDefaultTextColor;
            else
                DefaultTextColor = defaultTextColor;
            this.servicesHelper = servicesHelper;
            this.defaultPrompt = defaultPrompt;
            ThrowExceptionWhenAddingInvalidControl = false;
            AllowWatermarksWithDxValidationProviderKludge = false;
            WhitespaceInputRemovesPrompt = true;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultWatermarkTextColor">Default color of the watermark text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="defaultTextColor">Default color of the text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public WatermarkInputPromptManager(Color defaultWatermarkTextColor, Color defaultTextColor, IServicesHelper servicesHelper)
            : this(String.Empty, defaultWatermarkTextColor, defaultTextColor, servicesHelper) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultPrompt">The default prompt.</param>
        /// <param name="defaultWatermarkTextColor">Default color of the watermark text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="defaultTextColor">Default color of the text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        public WatermarkInputPromptManager(string defaultPrompt, Color defaultWatermarkTextColor, Color defaultTextColor)
            : this(defaultPrompt, defaultWatermarkTextColor, defaultTextColor, null) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultWatermarkTextColor">Default color of the watermark text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        /// <param name="defaultTextColor">Default color of the text.
        /// Passing in Color.Empty here will cause the default value to be used.</param>
        public WatermarkInputPromptManager(Color defaultWatermarkTextColor, Color defaultTextColor)
            : this(defaultWatermarkTextColor, defaultTextColor, null) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultPrompt">The default prompt.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public WatermarkInputPromptManager(string defaultPrompt, IServicesHelper servicesHelper)
            : this(defaultPrompt, Color.Empty, Color.Empty, servicesHelper) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="servicesHelper">The services helper.</param>
        public WatermarkInputPromptManager(IServicesHelper servicesHelper)
            : this(Color.Empty, Color.Empty, servicesHelper) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        /// <param name="defaultPrompt">The default prompt.</param>
        public WatermarkInputPromptManager(string defaultPrompt) : this(defaultPrompt, null) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkInputPromptManager"/> class.
        /// </summary>
        public WatermarkInputPromptManager() : this(String.Empty) { }


        /// <summary>
        /// Gets or sets a value indicating whether to use the kludge that allows 
        /// watermarks with the DxValidationProvider. As the property states, this
        /// is a kludge and is not guaranteed to work in future versions of
        /// Developer Express.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the watermark manager should use the kludge that 
        /// 	allows watermarks with the DxValidationProvider; otherwise, <c>false</c>.
        /// </value>
        public bool AllowWatermarksWithDxValidationProviderKludge { get; set; }

        /// <summary>
        /// Gets the count of managed controls.
        /// </summary>
        /// <value>The count.</value>
        public int Count { get { return watermarkControls.Count; } }

        /// <summary>
        /// Gets or sets the default prompt. If a prompt is not added to the
        /// watermark control, the default prompt is used.
        /// </summary>
        /// <value>The default prompt.</value>
        public string DefaultPrompt
        {
            get
            {
                if (String.IsNullOrEmpty(defaultPrompt))
                    defaultPrompt = GetServicesHelper().GetString("WatermarkDefaultPrompt");
                return defaultPrompt;
            }
            set { defaultPrompt = value; }
        }

        /// <summary>
        /// Gets the default color of the text.
        /// </summary>
        /// <value>The default color of the text.</value>
        public Color DefaultTextColor { get; private set; }

        /// <summary>
        /// Gets the default color of the watermark text.
        /// </summary>
        /// <value>The default color of the watermark text.</value>
        public Color DefaultWatermarkTextColor { get; private set; }

        /// <summary>
        /// Gets or sets a value indicating whether the WatermarkInputPromptManager 
        /// will throw exception when adding an invalid control.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the WatermarkInputPromptManager should throw 
        /// 	exception when adding invalid control; otherwise, <c>false</c>.
        /// </value>
        public bool ThrowExceptionWhenAddingInvalidControl { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether whitespace input into the
        /// control will remove the prompt. The default is true.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if whitespace input into the control removes the prompt; otherwise, <c>false</c>.
        /// </value>
        public bool WhitespaceInputRemovesPrompt { get; set; }


        /// <summary>
        /// Adds the specified WatermarkInputPromptItem to the list of managed 
        /// controls.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns><c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.</returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(WatermarkInputPromptItem item)
        {
            if (InvalidComboBoxEdit(item.EditControl) || invalidBaseEditDescendantList.Contains(item.EditControl.GetType()))
            {
                string errorMessage;
                if (InvalidComboBoxEdit(item.EditControl))
                    errorMessage = "ComboBoxEdit must have TextEditStyle set to Standard. Try using a good default value instead of a Watermark Input Prompt.";
                else
                    errorMessage = String.Format("Invalid BaseEdit descendant passed to WatermarkInputPromptManager: {0}.",
                        item.EditControl.GetType().ToString());
                GetServicesHelper().LogError(errorMessage);
                if (ThrowExceptionWhenAddingInvalidControl)
                    throw new ArgumentException(errorMessage);
                return false;
            }
            ConnectEvents(item.EditControl);
            ControlLeave(item.EditControl, EventArgs.Empty);
            SetPasswordChar(item);
            watermarkControls.Add(item.EditControl, item);
            return true;
        }

        /// <summary>
        /// Adds the specified edit control. The DefaultPrompt will be used.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns><c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.</returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(BaseEdit editControl)
        {
            return Add(new WatermarkInputPromptItem(editControl, String.Empty));
        }

        /// <summary>
        /// Adds the specified edit control and prompt to the list of managed
        /// controls. Uses the default text colors.
        /// </summary>
        /// <param name="editControl">The edit control to decorate with the watermark.</param>
        /// <param name="prompt">The watermark prompt.</param>
        /// <returns><c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.</returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(BaseEdit editControl, string prompt)
        {
            return Add(new WatermarkInputPromptItem(editControl, prompt));
        }

        /// <summary>
        /// Adds the specified edit control. The DefaultPrompt will be used.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <param name="passwordChar">The password char.</param>
        /// <returns><c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.</returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(BaseEdit editControl, char passwordChar)
        {
            return Add(new WatermarkInputPromptItem(editControl, String.Empty, passwordChar));
        }

        /// <summary>
        /// Adds the specified edit control and prompt to the list of managed
        /// controls. Uses the default text colors.
        /// </summary>
        /// <param name="editControl">The edit control to decorate with the watermark.</param>
        /// <param name="prompt">The watermark prompt.</param>
        /// <param name="passwordChar">The password character. If this is set then
        /// a TextEdit control will hide the content of the control.</param>
        /// <returns>
        /// 	<c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.
        /// </returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(BaseEdit editControl, string prompt, char passwordChar)
        {
            return Add(new WatermarkInputPromptItem(editControl, prompt, passwordChar));
        }

        /// <summary>
        /// Adds the specified edit control to the list of managed controls.
        /// </summary>
        /// <param name="editControl">The edit control to decorate with the watermark.</param>
        /// <param name="prompt">The watermark prompt.</param>
        /// <param name="watermarkTextColor">Color of the watermark text.</param>
        /// <param name="textColor">Color of the text.</param>
        /// <returns><c>true</c> if the control was added to the list of managed controls; otherwise <c>false</c>.</returns>
        /// <exception cref="ArgumentException">Thrown if the same control is added more than once.</exception>
        public bool Add(BaseEdit editControl, string prompt, Color watermarkTextColor,
            Color textColor)
        {
            return Add(new WatermarkInputPromptItem(editControl,
                prompt, default(char), watermarkTextColor, textColor));
        }

        /// <summary>
        /// Determines whether the manager contains the specified edit control.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns>
        /// 	<c>true</c> if the manager contains the specified edit control; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(BaseEdit editControl)
        {
            return watermarkControls.ContainsKey(editControl);
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, 
        /// or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            List<BaseEdit> keys = new List<BaseEdit>(watermarkControls.Keys);
            foreach (BaseEdit editControl in keys)
                Remove(editControl);
            watermarkControls.Clear();
        }

        /// <summary>
        /// Gets the password character associated with the edit control.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns></returns>
        public char GetPasswordChar(BaseEdit editControl)
        {
            WatermarkInputPromptItem item = null;
            if (TryGetItem(editControl, out item))
                return item.PasswordChar;
            else
                return default(char);
        }

        /// <summary>
        /// Gets the watermark prompt for the specified control. If the control is
        /// not found in the list of managed controls then String.Empty will be returned.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns>A watermark prompt.</returns>
        public string GetPrompt(BaseEdit editControl)
        {
            WatermarkInputPromptItem item = null;
            if (TryGetItem(editControl, out item))
            {
                if (String.IsNullOrEmpty(item.Prompt))
                    return DefaultPrompt;
                else
                    return item.Prompt;
            }
            else
                return String.Empty;
        }

        /// <summary>
        /// Gets the color of the text for the specified control. If the control is
        /// not found in the list of managed controls then Color.Empty will be returned.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns>The watermark text color.</returns>
        public Color GetTextColor(BaseEdit editControl)
        {
            WatermarkInputPromptItem item = null;
            if (TryGetItem(editControl, out item))
                if (item.WatermarkTextColor == Color.Empty)
                    return DefaultTextColor;
                else
                    return item.TextColor;
            else
                return DefaultTextColor;
        }

        /// <summary>
        /// Gets the color of the watermark text for the specified control. If the control is
        /// not found in the list of managed controls then Color.Empty will be returned.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        /// <returns>The watermark text color.</returns>
        public Color GetWatermarkTextColor(BaseEdit editControl)
        {
            WatermarkInputPromptItem item = null;
            if (TryGetItem(editControl, out item))
            {
                if (item.WatermarkTextColor == Color.Empty)
                    return DefaultWatermarkTextColor;
                else
                    return item.WatermarkTextColor;
            }
            else
                return DefaultWatermarkTextColor;
        }

        /// <summary>
        /// Removes the specified edit control from the list of managed controls.
        /// </summary>
        /// <param name="editControl">The edit control.</param>
        public void Remove(BaseEdit editControl)
        {
            GuardAgainst.ArgumentIsNotTypeOf(editControl, typeof(BaseEdit), "editControl");
            if (!watermarkControls.ContainsKey(editControl))
                return;

            DisconnectEvents(editControl);
            watermarkControls.Remove(editControl);
        }

        private void ConnectEvents(BaseEdit editControl)
        {
            editControl.CustomDisplayText += ControlCustomDisplayText;
            editControl.Leave += ControlLeave;
            editControl.Enter += ControlEnter;
            editControl.TextChanged += ControlTextChanged;
        }

        private void ControlCustomDisplayText(object sender, CustomDisplayTextEventArgs e)
        {
            if (e.Value == null || String.IsNullOrEmpty(e.Value.ToString())
                || (!WhitespaceInputRemovesPrompt && (e.Value is string) && (e.Value as String).Trim() == String.Empty))
                e.DisplayText = GetPrompt(sender as BaseEdit);
        }

        private void ControlEnter(object sender, EventArgs e)
        {
            BaseEdit editControl = sender as BaseEdit;
            EnablePasswordChar(editControl);
            editControl.Properties.Appearance.ForeColor = GetTextColor(editControl);
        }

        private void ControlLeave(object sender, EventArgs e)
        {
            BaseEdit editControl = sender as BaseEdit;
            string text = editControl.Text;
            if (!WhitespaceInputRemovesPrompt && text != null)
                text = text.Trim();
            if (String.IsNullOrEmpty(text))
            {
                DisablePasswordChar(editControl);
                editControl.Properties.Appearance.ForeColor = GetWatermarkTextColor(editControl);
                if (AllowWatermarksWithDxValidationProviderKludge)
                    DoTheDxValidationProviderKludge(editControl);
            }
        }

        private void ControlTextChanged(object sender, EventArgs e)
        {
            // if the password is set programmatically, the ControlEnter event must be fired.
            if (GetPasswordChar(sender as BaseEdit) != default(char))
                ControlEnter(sender, EventArgs.Empty);
        }

        private void DisablePasswordChar(BaseEdit editControl)
        {
            SetEnablePasswordChar(editControl, false);
        }

        private void DisconnectEvents(BaseEdit editControl)
        {
            editControl.CustomDisplayText -= ControlCustomDisplayText;
            editControl.Leave -= ControlLeave;
            editControl.Enter -= ControlEnter;
            editControl.TextChanged -= ControlTextChanged;
        }

        private void DoTheDxValidationProviderKludge(BaseEdit editor)
        {
            // This is a kludge which allows the watermark to be used with
            // the DxValidationProvider. Setting the NullText property to an
            // arbitrary value and back again forces the CustomDisplayText event
            // to be fired and properly displayed.
            string nullText = editor.Properties.NullText;
            try
            {
                editor.Properties.NullText = Guid.NewGuid().ToString();
            }
            finally
            {
                editor.Properties.NullText = nullText;
            }
        }

        private void EnablePasswordChar(BaseEdit editControl)
        {
            SetEnablePasswordChar(editControl, true);
        }

        private IServicesHelper GetServicesHelper()
        {
            if (servicesHelper == null)
                servicesHelper = new DefaultServicesHelper();
            return servicesHelper;
        }

        private bool InvalidComboBoxEdit(BaseEdit editControl)
        {
            return editControl is ComboBoxEdit
                && (editControl as ComboBoxEdit).Properties.TextEditStyle != TextEditStyles.Standard;
        }

        private void SetEnablePasswordChar(BaseEdit editControl, bool enabled)
        {
            if (editControl is TextEdit)
            {
                char passwordChar = GetPasswordChar(editControl);
                if (passwordChar != default(char))
                {
                    TextEdit textEdit = editControl as TextEdit;
                    if (enabled)
                        textEdit.Properties.PasswordChar = passwordChar;
                    else
                        textEdit.Properties.PasswordChar = default(char);
                }
            }
        }

        private void SetPasswordChar(WatermarkInputPromptItem item)
        {
            if (item.EditControl is TextEdit)
            {
                TextEdit textEdit = item.EditControl as TextEdit;
                if (textEdit.Properties.PasswordChar != default(char))
                {
                    item.PasswordChar = textEdit.Properties.PasswordChar;
                    textEdit.Properties.PasswordChar = default(char);
                }
            }
        }

        private bool TryGetItem(BaseEdit editControl, out WatermarkInputPromptItem item)
        {
            return watermarkControls.TryGetValue(editControl, out item);
        }
    }
}