#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core
{
    using System;
    using System.Configuration;
    using System.Drawing;
    using System.IO;
    using System.Reflection;
    using System.Resources;
    using System.Windows.Forms;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// <para>DevExpressContrib cannot work in complete isolation. This is the
    /// default IServicesHelper implementation for logging and localization
    /// services. Logging in the default implementation does nothing.</para>
    /// </summary>
    public class DefaultServicesHelper : IServicesHelper
    {

        /// <summary>
        /// Gets the localized string.
        /// </summary>
        /// <param name="localizerString">The localizer key.</param>
        /// <returns>The localized string.</returns>
        public virtual string GetString(string localizerString)
        {
            return GetString(localizerString, Assembly.GetCallingAssembly());
        }

        /// <summary>
        /// Gets the localized string.
        /// </summary>
        /// <param name="localizerString">The localizer key.</param>
        /// <param name="callingAssembly">The calling assembly.</param>
        /// <returns>The localized string.</returns>
        public virtual string GetString(string localizerString, Assembly callingAssembly)
        {
            ResourceManager resourceMan = ResourceHelper.GetAssemblyResourceManager(callingAssembly);
            if (resourceMan != null)
                return resourceMan.GetString(localizerString);
            LogError(String.Format("Couldn't find localized resource {0} in assembly {1}", localizerString, callingAssembly.FullName));
            return null;
        }

        /// <summary>
        /// Logs an error. In the DefaultServicesHelper this does nothing.
        /// </summary>
        /// <param name="message">The message.</param>
        public virtual void LogError(string message) { }

        /// <summary>
        /// Logs the exception. In the DefaultServicesHelper this calls overloaded LogException method.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public virtual void LogException(string message, Exception exception)
        {
            LogException(message, exception, null);
        }

        /// <summary>
        /// Logs the exception. In the DefaultServicesHelper this calls overloaded LogException method.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="details">The exception details.</param>
        public virtual void LogException(string message, Exception exception, string details)
        {
            LogException(message, exception, details, null);
        }

        /// <summary>
        /// Logs the exception. In the DefaultServicesHelper this does nothing.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="details">The exception details.</param>
        /// <param name="screenshot">Screenshot of the exception.</param>
        public virtual void LogException(string message, Exception exception, string details, Image screenshot) { }

        /// <summary>
        /// Logs a fatal error. In the DefaultServicesHelper this does nothing.
        /// </summary>
        /// <param name="message">The message.</param>
        public virtual void LogFatal(string message) { }

        /// <summary>
        /// Logs an informational message. In the DefaultServicesHelper this does nothing.
        /// </summary>
        /// <param name="message">The message.</param>
        public virtual void LogInformation(string message) { }

        /// <summary>
        /// Logs a warning. In the DefaultServicesHelper this does nothing.
        /// </summary>
        /// <param name="message">The message.</param>
        public virtual void LogWarning(string message) { }

        /// <summary>
        /// Reads a string value from a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>Value associated with the key.</returns>
        public virtual string ReadConfigString(string key)
        {
            return ReadConfigString(key, UserMachine.CurrentUser);
        }

        /// <summary>
        /// Reads a string value from a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <returns>Value associated with the key.</returns>
        public virtual string ReadConfigString(string key, UserMachine userMachine)
        {
            return ReadConfigString(key, userMachine, AppCommon.ThisApplication);
        }

        /// <summary>
        /// Reads a string value from a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <param name="appCommon">Whether this value is associated with the current application or all applications from this company.</param>
        /// <returns>Value associated with the key.</returns>
        public virtual string ReadConfigString(string key, UserMachine userMachine, AppCommon appCommon)
        {
            Configuration config = GetConfigurationFile();
            if (config.AppSettings.Settings[key] != null)
                return config.AppSettings.Settings[key].Value;
            else
                return String.Empty;
        }

        /// <summary>
        /// Writes a string value to a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        public virtual void WriteConfigString(string key, string value)
        {
            WriteConfigString(key, value, UserMachine.CurrentUser);
        }

        /// <summary>
        /// Writes a string value to a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        public virtual void WriteConfigString(string key, string value, UserMachine userMachine)
        {
            WriteConfigString(key, value, userMachine, AppCommon.ThisApplication);
        }

        /// <summary>
        /// Writes a string value to a configuration system.
        /// For the DefaultServicesHelper, the configuration system consists of
        /// reading and writing to a Settings.config file in the application directory.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        /// <param name="userMachine">Whether this value is associated with the current user or the local machine.</param>
        /// <param name="appCommon">Whether this value is associated with the current application or all applications from this company.</param>
        public virtual void WriteConfigString(string key, string value, UserMachine userMachine, AppCommon appCommon)
        {
            Configuration config = GetConfigurationFile();

            config.AppSettings.Settings.Remove(key);
            config.AppSettings.Settings.Add(key, value);
            config.Save(ConfigurationSaveMode.Modified);
            ConfigurationManager.RefreshSection("appSettings");
        }

        private Configuration GetConfigurationFile()
        {
            return ConfigurationManager.OpenMappedExeConfiguration(new ExeConfigurationFileMap
                {
                    ExeConfigFilename = String.Format("{0}{1}Settings.config", Path.GetDirectoryName(Application.ExecutablePath),
                      Path.DirectorySeparatorChar)
                }, ConfigurationUserLevel.None);
        }
    }
}
