#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core
{
    using System;
    using System.Text.RegularExpressions;

    /// <summary>
    /// TopLevelDomains represent valid, non-country, last portion of the Internet domain name e.g. .com, .net, .edu, etc.
    /// Top level domains defined at http://en.wikipedia.org/wiki/Top_level_domains (last checked 15 February 2009).
    /// </summary>
    public enum TopLevelDomains
    {
        /// <summary>
        /// Generic top level domains consisting of:
        /// biz, com, info, name, net, org, pro
        /// </summary>
        Generic,

        /// <summary>
        /// Sponsored top level domains consisting of: 
        /// aero, asia, cat, coop, edu, gov, int, jobs, mil, mobi, museum, tel, travel
        /// </summary>
        Sponsored,

        /// <summary>
        /// Both Generic and Sponsored top level domains
        /// </summary>
        All
    }

    /// <summary>
    /// EmailHelper provides a simple methods to assist in with email tasks.
    /// </summary>
    public static class EmailHelper
    {

        /// <summary>
        /// Are the email addresses well formed.
        /// </summary>
        /// <param name="semicolonDelimitedEmailAddresses">The semicolon delimited email addresses.</param>
        /// <param name="firstInvalidEmailAddress">The first invalid email address.</param>
        /// <returns><c>true</c> if all email addresses in the list are well formed.</returns>
        public static bool AreEmailAddressesWellFormed(string semicolonDelimitedEmailAddresses,
            out string firstInvalidEmailAddress)
        {
            return AreEmailAddressesWellFormed(semicolonDelimitedEmailAddresses, false,
                TopLevelDomains.All, out firstInvalidEmailAddress);
        }

        /// <summary>
        /// Are the email addresses well formed.
        /// </summary>
        /// <param name="semicolonDelimitedEmailAddresses">The semicolon delimited email addresses.</param>
        /// <param name="checkCommonTopLevelDomains">if set to <c>true</c> check common top level domains.</param>
        /// <param name="firstInvalidEmailAddress">The first invalid email address.</param>
        /// <returns><c>true</c> if all email addresses in the list are well formed.</returns>
        public static bool AreEmailAddressesWellFormed(string semicolonDelimitedEmailAddresses,
            bool checkCommonTopLevelDomains, out string firstInvalidEmailAddress)
        {
            return AreEmailAddressesWellFormed(semicolonDelimitedEmailAddresses,
                checkCommonTopLevelDomains, TopLevelDomains.All, out firstInvalidEmailAddress);
        }

        /// <summary>
        /// Are the email addresses well formed.
        /// </summary>
        /// <param name="semicolonDelimitedEmailAddresses">The semicolon delimited email addresses.</param>
        /// <param name="checkCommonTopLevelDomains">if set to <c>true</c> check common top level domains.</param>
        /// <param name="domains">The top level domains to check against.</param>
        /// <param name="firstInvalidEmailAddress">The first invalid email address.</param>
        /// <returns><c>true</c> if all email addresses in the list are well formed.</returns>
        public static bool AreEmailAddressesWellFormed(string semicolonDelimitedEmailAddresses,
            bool checkCommonTopLevelDomains, TopLevelDomains domains, out string firstInvalidEmailAddress)
        {
            bool result = true;
            firstInvalidEmailAddress = String.Empty;
            string[] addresses = semicolonDelimitedEmailAddresses.Split(';');
            foreach (string address in addresses)
            {
                if (IsEmailAddressWellFormed(address, checkCommonTopLevelDomains, domains))
                    continue;

                result = false;
                firstInvalidEmailAddress = address;
                break;
            }
            return result;
        }

        /// <summary>
        /// Gets a string of top level internet domains delimited by the Vertical Line character.
        /// Top level domains defined at http://en.wikipedia.org/wiki/Top_level_domains (last checked 15 February 2009).
        /// </summary>
        /// <param name="domains">The TopLevelDomains enum defining which domains to return.</param>
        /// <returns>Top level internet domains.</returns>
        public static string GetTopLevelInternetDomains(TopLevelDomains domains)
        {
            const string genericDomains = "biz|com|info|name|net|org|pro";
            const string sponsoredDomains = "aero|asia|cat|coop|edu|gov|int|jobs|mil|mobi|museum|tel|travel";
            string result;
            switch (domains)
            {
                case TopLevelDomains.Generic:
                    result = genericDomains;
                    break;
                case TopLevelDomains.Sponsored:
                    result = sponsoredDomains;
                    break;
                case TopLevelDomains.All:
                    result = String.Format("{0}|{1}", genericDomains, sponsoredDomains);
                    break;
                default:
                    throw new InvalidOperationException(String.Format("Invalid TopLevelDomains: {0}",domains.ToString()));
            }
            return result;
        }

        /// <summary>
        /// Determines whether the specified email address is well formed.
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <returns><c>true</c> if the email address is well formed.</returns>
        public static bool IsEmailAddressWellFormed(string emailAddress)
        {
            return IsEmailAddressWellFormed(emailAddress, false, TopLevelDomains.All);
        }

        /// <summary>
        /// Determines whether the specified email address is well formed.
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="checkCommonTopLevelDomains">if set to <c>true</c> check common top level domains.</param>
        /// <returns><c>true</c> if the email address is well formed.</returns>
        public static bool IsEmailAddressWellFormed(string emailAddress, bool checkCommonTopLevelDomains)
        {
            return IsEmailAddressWellFormed(emailAddress, checkCommonTopLevelDomains, TopLevelDomains.All);
        }

        /// <summary>
        /// Determines whether the specified email address is well formed.
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="checkCommonTopLevelDomains">if set to <c>true</c> check common top level domains.</param>
        /// <param name="domains">The top level domains to check against.</param>
        /// <returns><c>true</c> if the email address is well formed.</returns>
        public static bool IsEmailAddressWellFormed(string emailAddress,
            bool checkCommonTopLevelDomains, TopLevelDomains domains)
        {
            GuardAgainst.ArgumentIsNull(emailAddress, "emailAddress");

            const string patternBase = @"\A[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+";

            string pattern;
            if (checkCommonTopLevelDomains)
                pattern = patternBase + String.Format(@"(?:[A-Z]{{2}}|{0})\z", GetTopLevelInternetDomains(domains));
            else
                pattern = String.Format(@"{0}[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\z", patternBase);
            Regex regex = new Regex(pattern, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);
            return regex.Match(emailAddress.Trim()).Success;
        }

        ///// <summary>
        ///// Sends an email message via SMTP (Simple Mail Transport Protocol).
        ///// </summary>
        ///// <param name="from">'From' email address.</param>
        ///// <param name="to">'To' email address.</param>
        ///// <param name="subject">The subject of the email.</param>
        ///// <param name="body">The body of the email.</param>
        ///// <param name="emailConnection">The email connection information.</param>
        ///// <param name="attachments">The optional email attachments.</param>
        ///// 
        ///// <exception cref="ArgumentNullException">From or To is null reference.</exception>
        ///// <exception cref="ArgumentException">From or To is String.Empty.</exception>
        ///// <exception cref="FormatException">From or To is not a well formed email address.</exception>
        ///// <exception cref="InvalidOperationException">
        ///// <list>
        /////   <item>Host is null or String.Empty</item>
        /////   <item>Port is 0</item>
        /////   <item>SmptClient is already sending a message via SendAsync</item>
        ///// </list>
        ///// </exception>
        ///// <exception cref="ObjectDisposedException">The object has already been disposed.</exception>
        ///// <exception cref="SmtpException">
        ///// <list>
        /////   <item>SMTP connection failed</item>
        /////   <item>Authorization failed</item>
        /////   <item>The SMTP server timed out</item>
        ///// </list>
        ///// </exception>
        ///// <exception cref="SmtpFailedRecipientsException">The message could not be delivered to one of the email recipients.</exception>
        //[CoverageExclude]
        //public static void SendSmtp(string from, string to, string subject, string body,
        //    EmailConnectionInformation emailConnection, params EmailAttachment[] attachments)
        //{
        //    using (MailMessage mailMessage = new MailMessage(from, to, subject, body))
        //    {
        //        foreach (EmailAttachment attachment in attachments)
        //            mailMessage.Attachments.Add(new Attachment(attachment.Value, attachment.Name, attachment.MimeEncoding));
        //        SmtpClient smtpClient = new SmtpClient(emailConnection.Host, emailConnection.Port);
        //        if (!String.IsNullOrEmpty(emailConnection.Username))
        //            smtpClient.Credentials = new NetworkCredential(emailConnection.Username, emailConnection.Password);
        //        smtpClient.Send(mailMessage);
        //    }
        //}
    }
}