#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.IO;
    using System.Net.Mail;
    using System.Net.Mime;
    using System.Reflection;
    using System.Text;
    using System.Threading;
    using System.Windows.Forms;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// Base class to handle most of the exception handler methods and functionality.
    /// </summary>
    public abstract class ExceptionHandlerBase
    {
        private int defaultExitCode = 3856;
        private string dialogCaption;
        private string emailInformationCaption;
        private string emailInformationText;
        private string emailSubjectText;
        private const string GlobalExceptionHandlerFromEmailKey = "GlobalExceptionHandlerFromEmail";
        private string howAffectedCaption;
        private string howAffectedFatalText;
        private string howAffectedNonFatalText;
        private string whatHappenedCaption;
        private string whatHappenedText;
        private string whatToDoCaption;
        private string whatToDoText;
        private string whatToDoTextWithEmail;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DevExpressContrib.Core.ExceptionHandlerBase">ExceptionHandlerBase</see> class. 
        /// </summary>
        public ExceptionHandlerBase(IServicesHelper servicesHelper, bool useExceptionHandlerDuringDebugging)
        {
            ServicesHelper = servicesHelper;
            ShowEmailControls = true;
            ScreenShotMode = ScreenShotModes.AllMonitors;
            AttachEventHandlers(useExceptionHandlerDuringDebugging);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DevExpressContrib.Core.ExceptionHandlerBase">ExceptionHandlerBase</see> class. 
        /// </summary>
        public ExceptionHandlerBase(IServicesHelper servicesHelper) : this(servicesHelper, false) { }

        // the default is a random number
        /// <summary>
        /// Gets or sets the default exit code to pass back to the underlying 
        /// operating system if the application is forced to close on a fatal exception.
        /// </summary>
        /// <value>The default exit code.</value>
        public virtual int DefaultExitCode
        {
            get { return defaultExitCode; }
            set
            {
                if (defaultExitCode == value)
                    return;
                defaultExitCode = value;
            }
        }

        /// <summary>
        /// Gets or sets the name of the product. This is displayed in some of the explanatory text.
        /// </summary>
        /// <value>The name of the product.</value>
        public virtual string ProductName { get; set; }

        /// <summary>
        /// Gets or sets the screen shot mode. The default is to obtain a screenshot of all monitors.
        /// </summary>
        /// <value>The screen shot mode.</value>
        public ScreenShotModes ScreenShotMode { get; set; }

        /// <summary>
        /// Gets or sets the send exception email handler.
        /// </summary>
        /// <value>The send exception email handler.</value>
        public ISendExceptionEmail SendExceptionEmail { get; set; }

        /// <summary>
        /// Gets or sets the services helper.
        /// </summary>
        /// <value>The services helper.</value>
        public IServicesHelper ServicesHelper { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether the dialog box should show email controls.
        /// </summary>
        /// <value><c>true</c> if showing email controls; otherwise, <c>false</c>.</value>
        public bool ShowEmailControls { get; set; }

        /// <summary>
        /// Gets a value indicating whether this handler supports emailing the exception back to your company.
        /// </summary>
        /// <value><c>true</c> if email is supported; otherwise, <c>false</c>.</value>
        public virtual bool SupportsEmail { get { return true; } }

        /// <summary>
        /// Gets a value indicating whether the exception handler should be used during debugging.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the exception handler should be used during debugging; otherwise, <c>false</c>.
        /// </value>
        public virtual bool UseExceptionHandlerDuringDebugging { get; set; }

        /// <summary>
        /// Gets or sets user defined additional information interface. This can be used to inject
        /// a user control into the exception handler.
        /// </summary>
        /// <value>The user defined additional information interface.</value>
        public virtual IExceptionHandlerUserDefined UserDefined
        {
            get { return null; }
            set { }
        }

        /// <summary>
        /// Initializes the exception handler class.
        /// </summary>
        public virtual void AttachEventHandlers(bool useExceptionHandlerDuringDebugging)
        {
            // if we are debugging we already have a global exception handler - it's Visual Studio.
            if (Debugger.IsAttached && !useExceptionHandlerDuringDebugging)
                return;
            AppDomain.CurrentDomain.UnhandledException += AppDomainUnhandledException;
            Application.ThreadException += ThreadHandleException;
        }

        /// <summary>
        /// Gets and formats exception details. This will be a multi-line string
        /// with information about the exception and all of the loaded assemblies
        /// within the AppDomain.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <returns>Exception detail string.</returns>
        public virtual string ExceptionDetails(Exception exception)
        {
            string loadedAssembliesText = GetServicesHelper().GetString("LoadedAssemblies");
            string assemblyNameText = GetServicesHelper().GetString("AssemblyName");
            string assemblyLocationText = GetServicesHelper().GetString("AssemblyLocation");

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} : Local Time{1}", DateTime.Now, Environment.NewLine);
            stringBuilder.AppendFormat("{0} : UTC{1}", DateTime.UtcNow, Environment.NewLine);
            stringBuilder.AppendFormat("{0}{1}", "-------------------------", Environment.NewLine);
            stringBuilder.AppendFormat("OS Version: {0}{1}", Environment.OSVersion, Environment.NewLine);
            stringBuilder.AppendFormat("Processor Count: {0}{1}", Environment.ProcessorCount, Environment.NewLine);
            stringBuilder.AppendFormat("Computer Name: {0}{1}", Environment.MachineName, Environment.NewLine);
            stringBuilder.AppendFormat("User Name: {0}{1}", Environment.UserName, Environment.NewLine);
            stringBuilder.AppendFormat("Command Line: {0}{1}", Environment.CommandLine, Environment.NewLine);
            stringBuilder.AppendFormat("{0}{1}", "-------------------------", Environment.NewLine);
            stringBuilder.AppendFormat("{0}{1}{2}", exception.ToString(), Environment.NewLine, Environment.NewLine);
            stringBuilder.AppendFormat("{0}{1}", loadedAssembliesText, Environment.NewLine);
            stringBuilder.AppendFormat("{0}{1}", "-------------------------", Environment.NewLine);
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                AssemblyName assemblyName = assembly.GetName();
                stringBuilder.AppendFormat("{0}\t{1}{2}", assemblyNameText, assemblyName.FullName, Environment.NewLine);
                stringBuilder.AppendFormat("{0}\t{1}{2}", assemblyLocationText, assemblyName.CodeBase, Environment.NewLine);
                stringBuilder.Append(Environment.NewLine);
            }
            return stringBuilder.ToString();
        }

        /// <summary>
        /// Gets the exception display text. This allows you to obtain the display text of the dialog.
        /// </summary>
        /// <param name="exceptionHandlerDisplayText">The default text to obtain.</param>
        /// <returns>The display text associated with the supplied ExceptionHandlerDisplayText.</returns>
        public string GetExceptionDisplayText(ExceptionHandlerDisplayText exceptionHandlerDisplayText)
        {
            switch (exceptionHandlerDisplayText)
            {
                case ExceptionHandlerDisplayText.DialogCaption:
                    return GetExceptionDisplayText(ref dialogCaption, "DefaultDialogCaption");
                case ExceptionHandlerDisplayText.EmailInformationText:
                    return GetExceptionDisplayText(ref emailInformationText, "EmailInformationDefaultText");
                case ExceptionHandlerDisplayText.HowAffectedFatalText:
                    return GetExceptionDisplayText(ref howAffectedFatalText, "HowAffectedFatalDefaultText");
                case ExceptionHandlerDisplayText.HowAffectedNonFatalText:
                    return GetExceptionDisplayText(ref howAffectedNonFatalText, "HowAffectedNonFatalDefaultText");
                case ExceptionHandlerDisplayText.WhatHappenedText:
                    return GetExceptionDisplayText(ref whatHappenedText, "WhatHappenedDefaultText");
                case ExceptionHandlerDisplayText.WhatToDoText:
                    return GetExceptionDisplayText(ref whatToDoText, "WhatToDoDefaultText");
                case ExceptionHandlerDisplayText.WhatToDoTextWithEmail:
                    return GetExceptionDisplayText(ref whatToDoTextWithEmail, "WhatToDoDefaultTextWithEmail");
                case ExceptionHandlerDisplayText.EmailInformationCaption:
                    return GetExceptionDisplayText(ref emailInformationCaption, "EmailInformationDefaultCaption");
                case ExceptionHandlerDisplayText.HowAffectedCaption:
                    return GetExceptionDisplayText(ref howAffectedCaption, "HowAffectedDefaultCaption");
                case ExceptionHandlerDisplayText.WhatHappenedCaption:
                    return GetExceptionDisplayText(ref whatHappenedCaption, "WhatHappenedDefaultCaption");
                case ExceptionHandlerDisplayText.WhatToDoCaption:
                    return GetExceptionDisplayText(ref whatToDoCaption, "WhatToDoDefaultCaption");
                case ExceptionHandlerDisplayText.EmailSubjectText:
                    return GetSubjectDisplayText();
                default:
                    throw new InvalidOperationException(String.Format("Invalid ExceptionHandlerDisplayText: {0}", exceptionHandlerDisplayText.ToString()));
            }
        }

        /// <summary>
        /// Sets the exception display text. This allows you to customize the display text of the dialog.
        /// </summary>
        /// <param name="exceptionHandlerDisplayText">The default text to set.</param>
        /// <param name="value">The value of the display text associated with the supplied ExceptionHandlerDisplayText.</param>
        public void SetExceptionDisplayText(ExceptionHandlerDisplayText exceptionHandlerDisplayText, string value)
        {
            switch (exceptionHandlerDisplayText)
            {
                case ExceptionHandlerDisplayText.DialogCaption:
                    dialogCaption = value;
                    break;
                case ExceptionHandlerDisplayText.EmailInformationText:
                    emailInformationText = value;
                    break;
                case ExceptionHandlerDisplayText.HowAffectedFatalText:
                    howAffectedFatalText = value;
                    break;
                case ExceptionHandlerDisplayText.HowAffectedNonFatalText:
                    howAffectedNonFatalText = value;
                    break;
                case ExceptionHandlerDisplayText.WhatHappenedText:
                    whatHappenedText = value;
                    break;
                case ExceptionHandlerDisplayText.WhatToDoText:
                    whatToDoText = value;
                    break;
                case ExceptionHandlerDisplayText.WhatToDoTextWithEmail:
                    whatToDoTextWithEmail = value;
                    break;
                case ExceptionHandlerDisplayText.EmailInformationCaption:
                    emailInformationCaption = value;
                    break;
                case ExceptionHandlerDisplayText.HowAffectedCaption:
                    howAffectedCaption = value;
                    break;
                case ExceptionHandlerDisplayText.WhatHappenedCaption:
                    whatHappenedCaption = value;
                    break;
                case ExceptionHandlerDisplayText.WhatToDoCaption:
                    whatToDoCaption = value;
                    break;
                case ExceptionHandlerDisplayText.EmailSubjectText:
                    emailSubjectText = value;
                    break;
                default:
                    throw new InvalidOperationException(String.Format("Invalid ExceptionHandlerDisplayText: {0}", exceptionHandlerDisplayText.ToString()));
            }
        }

        /// <summary>
        /// Shows the specified exception.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <returns>
        /// DialogResult of the dialog showing the exception details.
        /// </returns>
        public DialogResult Show(Exception exception)
        {
            return Show(exception, new ExceptionHandlerArgs());
        }

        /// <summary>
        /// Shows the specified exception.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <param name="args">Information about the exception.</param>
        /// <returns>
        /// DialogResult of the dialog showing the exception details.
        /// </returns>
        public abstract DialogResult Show(Exception exception, ExceptionHandlerArgs args);

        /// <summary>
        /// Determines whether this instance can send email or not.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can send email; otherwise, <c>false</c>.
        /// </returns>
        protected bool CanSendEmail()
        {
            return SupportsEmail
                && ShowEmailControls
                && SendExceptionEmail != null
                && SendExceptionEmail.CanSendEmail();
        }

        /// <summary>
        /// Gets the email body content.
        /// </summary>
        /// <param name="exception">The exception from which to obtain the details.</param>
        /// <returns>The email body content</returns>
        protected string GetEmailBody(Exception exception)
        {
            StringBuilder body = new StringBuilder();
            if (UserDefined != null)
            {
                body.Append(UserDefined.Data);
                body.AppendLine();
                body.AppendLine();
            }
            body.Append(ExceptionDetails(exception));
            return body.ToString();
        }

        /// <summary>
        /// Gets the screenshot.
        /// </summary>
        /// <returns>Image of the sreenshot.</returns>
        protected virtual Image GetScreenshot()
        {
            switch (ScreenShotMode)
            {
                case ScreenShotModes.AllMonitors:
                    return ScreenshotHelper.SnapAllMonitorScreenshot();
                case ScreenShotModes.AllOpenForms:
                    return ScreenshotHelper.SnapAllOpenFormsScreenshot();
                case ScreenShotModes.FirstOpenForm:
                    return ScreenshotHelper.SnapFirstOpenFormScreenshot();
                case ScreenShotModes.PrimaryMonitor:
                    return ScreenshotHelper.SnapPrimaryMonitorScreenshot();
                default:
                    return ScreenshotHelper.SnapAllMonitorScreenshot();
            }
        }

        /// <summary>
        /// Gets the services helper. If there is no services helper a default one
        /// will be created for you.
        /// </summary>
        /// <returns>An implementation of the IServicesHelper</returns>
        protected IServicesHelper GetServicesHelper()
        {
            if (ServicesHelper == null)
                ServicesHelper = new DefaultServicesHelper();
            return ServicesHelper;
        }

        /// <summary>
        /// Reads the from email address from a configuration system.
        /// </summary>
        /// <returns>the from email address (or String.Empty if the address is not stored).</returns>
        protected string ReadFromEmailAddress()
        {
            return GetServicesHelper().ReadConfigString(GlobalExceptionHandlerFromEmailKey,
                UserMachine.CurrentUser, AppCommon.AllApplications);
        }

        /// <summary>
        /// Sends the exception email.
        /// </summary>
        /// <param name="exception">The exception.</param>
        /// <param name="fromEmailAddress">From email address.</param>
        /// <param name="includeScreenshotInEmail">if set to <c>true</c> includes screenshot in email.</param>
        /// <param name="screenshot">The screenshot.</param>
        /// <returns><c>true</c> if the email could be sent; otherwise, <c>false</c>.</returns>
        protected bool SendEmail(Exception exception, string fromEmailAddress, bool includeScreenshotInEmail, Image screenshot)
        {
            bool result = false;
            if (CanSendEmail())
            {
                SendExceptionEmail.Subject = String.Format(GetExceptionDisplayText(ExceptionHandlerDisplayText.EmailSubjectText),
                    exception.GetType().ToString(), ProductName);
                SendExceptionEmail.Body = GetEmailBody(exception);
                if (fromEmailAddress == null)
                    fromEmailAddress = String.Empty;
                SendExceptionEmail.FromEmailAddress = fromEmailAddress;

                MemoryStream screenshotStream = null;
                try
                {
                    if (includeScreenshotInEmail && screenshot != null)
                    {
                        screenshotStream = new MemoryStream();
                        screenshot.Save(screenshotStream, ImageFormat.Png);
                        screenshotStream.Position = 0;
                        SendExceptionEmail.Attachments.Add(new Attachment(screenshotStream, "Screenshot.png", MediaTypeNames.Application.Octet));
                    }

                    string errorMessage;
                    result = SendExceptionEmail.SendEmail(out errorMessage);
                    if (!result)
                        GetServicesHelper().LogError(String.Format("Could not send unhandled exception email: {0}", errorMessage));
                }
                catch (Exception sendMailException)
                {
                    // at this point we already in trouble. Catch the exception and attempt to log it.
                    GetServicesHelper().LogException("Could not send unhandled exception email.", sendMailException);
                }
                finally
                {
                    if (screenshotStream != null)
                        screenshotStream.Dispose();
                }
            }
            return result;
        }

        /// <summary>
        /// Sets the default exception handler values.
        /// </summary>
        /// <param name="args">The arguments.</param>
        protected virtual void SetDefaultExceptionHandlerValues(ExceptionHandlerArgs args)
        {
            if (String.IsNullOrEmpty(args.Caption))
                args.Caption = GetExceptionDisplayText(ExceptionHandlerDisplayText.DialogCaption);
            if (String.IsNullOrEmpty(args.WhatHappenedText))
                args.WhatHappenedText = GetExceptionDisplayText(ExceptionHandlerDisplayText.WhatHappenedText);
            if (String.IsNullOrEmpty(args.EmailInformationText))
                args.EmailInformationText = GetExceptionDisplayText(ExceptionHandlerDisplayText.EmailInformationText);
            if (String.IsNullOrEmpty(args.HowAffectedText))
            {
                if (args.IsFatal)
                    args.HowAffectedText = GetExceptionDisplayText(ExceptionHandlerDisplayText.HowAffectedFatalText);
                else
                    args.HowAffectedText = GetExceptionDisplayText(ExceptionHandlerDisplayText.HowAffectedNonFatalText);
            }
            if (String.IsNullOrEmpty(args.WhatToDoText))
            {
                if (CanSendEmail())
                    args.WhatToDoText = GetExceptionDisplayText(ExceptionHandlerDisplayText.WhatToDoTextWithEmail);
                else
                    args.WhatToDoText = GetExceptionDisplayText(ExceptionHandlerDisplayText.WhatToDoText);
            }

            if (String.IsNullOrEmpty(args.WhatHappenedCaption))
                args.WhatHappenedCaption = GetExceptionDisplayText(ExceptionHandlerDisplayText.WhatHappenedCaption);
            if (String.IsNullOrEmpty(args.EmailInformationCaption))
                args.EmailInformationCaption = GetExceptionDisplayText(ExceptionHandlerDisplayText.EmailInformationCaption);
            if (String.IsNullOrEmpty(args.HowAffectedCaption))
                args.HowAffectedCaption = GetExceptionDisplayText(ExceptionHandlerDisplayText.HowAffectedCaption);
            if (String.IsNullOrEmpty(args.WhatToDoCaption))
                args.WhatToDoCaption = GetExceptionDisplayText(ExceptionHandlerDisplayText.WhatToDoCaption);
        }

        /// <summary>
        /// Writes the from email address to a configuration system.
        /// </summary>
        /// <param name="fromEmailAddress">From email address.</param>
        protected void WriteFromEmailAddress(string fromEmailAddress)
        {
            GetServicesHelper().WriteConfigString(GlobalExceptionHandlerFromEmailKey,
                fromEmailAddress, UserMachine.CurrentUser, AppCommon.AllApplications);
        }

        private void AppDomainUnhandledException(Object sender, UnhandledExceptionEventArgs e)
        {
            if (e.ExceptionObject is Exception)
                Show(e.ExceptionObject as Exception);
            else
                Show(new Exception(e.ExceptionObject.ToString()));
        }

        private string GetExceptionDisplayText(ref string displayText, string defaultTextKey)
        {
            if (String.IsNullOrEmpty(displayText))
                displayText = String.Format(GetServicesHelper().GetString(defaultTextKey), ProductName);
            return displayText;
        }

        private string GetSubjectDisplayText()
        {
            if (String.IsNullOrEmpty(emailSubjectText))
                emailSubjectText = GetServicesHelper().GetString("EmailSubjectDefaultText");
            return emailSubjectText;
        }

        private void ThreadHandleException(object sender, ThreadExceptionEventArgs e)
        {
            Show(e.Exception);
        }
    }
}
