#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core
{
    using System;

    /// <summary>
    /// <para>GuardAgainst implements a set of <a href="http://c2.com/cgi/wiki?GuardClause">guard clauses</a>.</para>
    /// <para>Methods in this class should be phrased where the method name reads as positive
    /// when it throws an exception.</para>
    /// </summary>
    public static class GuardAgainst
    {

        /// <summary>
        /// Throws an exception if the argument is greater then the supplied value.
        /// </summary>
        /// <param name="argument">The argument.</param>
        /// <param name="minimum">The minimum value.</param>
        /// <param name="argumentName">Name of the argument.</param>
        public static void ArgumentIntegerIsGreaterThan(int argument, int minimum, string argumentName)
        {
            ArgumentIntegerIsGreaterThan(argument, minimum, argumentName, String.Empty);
        }

        /// <summary>
        /// Throws an exception if the argument is greater then the supplied value.
        /// </summary>
        /// <param name="argument">The argument.</param>
        /// <param name="minimum">The minimum value.</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <param name="message">An additional message to add to the exception.</param>
        public static void ArgumentIntegerIsGreaterThan(int argument, int minimum, string argumentName, string message)
        {
            if (argument > minimum)
            {
                string execptionMessage = String.Format("Argument '{0}' must not be greater than {1}.", argumentName, minimum);
                if (!String.IsNullOrEmpty(message))
                    execptionMessage += String.Format(" {0}", message);
                throw new GuardAgainstArgumentException(execptionMessage);
            }
        }

        /// <summary>
        /// Throws an exception of the argument is less than or equal to zero.
        /// </summary>
        /// <param name="argument">The argument.</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when the argument is less than or equal to zero.</exception>
        public static void ArgumentIntegerIsLessThanOrEqualToZero(long argument, string argumentName)
        {
            ArgumentIntegerOutOfRange(argument, 1, long.MaxValue, argumentName);
        }

        /// <summary>
        /// Throws an exception of the argument is less than or equal to zero.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when the argument is less than or equal to zero.</exception>
        public static void ArgumentIntegerIsLessThanOrEqualToZero(int argument, string argumentName)
        {
            ArgumentIntegerOutOfRange(argument, 1, int.MaxValue, argumentName);
        }

        /// <summary>
        /// Throws an exception if the argument is outside of the supplied range. The range check is inclusive.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="minimum">The minimum value (inclusive).</param>
        /// <param name="maximum">The maximum value (inclusive).</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when the argument is outside of the supplied range.</exception>
        public static void ArgumentIntegerOutOfRange(long argument, long minimum, long maximum, string argumentName)
        {
            if (argument < minimum || argument > maximum)
                throw new GuardAgainstArgumentException(String.Format(
                    "Argument '{0}' must be between {1} and {2} (inclusive).",
                    argumentName, minimum, maximum));
        }

        /// <summary>
        /// Throws an exception if the argument is outside of the supplied range. The range check is inclusive.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="minimum">The minimum value (inclusive).</param>
        /// <param name="maximum">The maximum value (inclusive).</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when the argument is outside of the supplied range.</exception>
        public static void ArgumentIntegerOutOfRange(int argument, int minimum, int maximum, string argumentName)
        {
            if (argument < minimum || argument > maximum)
                throw new GuardAgainstArgumentException(String.Format(
                    "Argument '{0}' must be between {1} and {2} (inclusive).",
                    argumentName, minimum, maximum));
        }

        /// <summary>
        /// Throws an exception if the argument is not of the supplied type.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="requiredType">The required argument type.</param>
        /// <param name="argumentName">Name of the argument.</param>
        public static void ArgumentIsNotTypeOf(object argument, Type requiredType, string argumentName)
        {
            if (argument.GetType().IsInstanceOfType(requiredType))
                throw new GuardAgainstArgumentException(String.Format("Argument '{0}' is not of required type {1}.",
                    argumentName, requiredType.ToString()));
        }

        /// <summary>
        /// Throws an exception if the argument is null.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentNullException">Thrown when the argument is null.</exception>
        public static void ArgumentIsNull(object argument, string argumentName)
        {
            if (argument == null)
                throw new GuardAgainstArgumentException(String.Format("Argument '{0}' cannot be null.", argumentName));
        }

        /// <summary>
        /// Throws an exception if the string argument is null or empty.
        /// </summary>
        /// <param name="argument">The argument instance.</param>
        /// <param name="argumentName">Name of the argument.</param>
        /// <exception cref="ArgumentNullException">Thrown when the argument is null or empty.</exception>
        public static void ArgumentIsNullOrEmpty(string argument, string argumentName)
        {
            if (String.IsNullOrEmpty(argument))
                throw new GuardAgainstArgumentException(String.Format("Argument '{0}' cannot be null or empty.", argumentName));
        }

        /// <summary>
        /// Throws an exception if the value is not greater than the supplied maxValue.
        /// </summary>
        /// <param name="value">The value instance.</param>
        /// <param name="maxValue">The max value of the instance.</param>
        /// <param name="valueName">Name of the value.</param>
        public static void ValueIsGreaterThan(int value, int maxValue, string valueName)
        {
            if (value > maxValue)
                throw new GuardAgainstValueException(String.Format("Value '{0}' cannot be greater than {1}; it's value is {2}.",
                    valueName, maxValue, value));
        }

        /// <summary>
        /// Throws an exception if the value instance is not of the required type.
        /// </summary>
        /// <param name="value">The value instance.</param>
        /// <param name="requiredType">Required type.</param>
        /// <param name="valueName">Name of the value.</param>
        public static void ValueIsNotTypeOf(object value, Type requiredType, string valueName)
        {
            ValueTypeIsNotTypeOf(value.GetType(), requiredType, valueName);
        }

        /// <summary>
        /// Throws an exception if the value is null or empty.
        /// </summary>
        /// <param name="value">The value instance.</param>
        /// <param name="valueName">Name of the value.</param>
        public static void ValueIsNullOrEmpty(string value, string valueName)
        {
            if (String.IsNullOrEmpty(value))
                throw new GuardAgainstValueException(String.Format("Value '{0}' cannot be null or empty.", valueName));
        }

        /// <summary>
        /// Throws an exception if the value type is not of the required type.
        /// </summary>
        /// <param name="value">The value type.</param>
        /// <param name="requiredType">Required type.</param>
        /// <param name="valueName">Name of the value.</param>
        public static void ValueTypeIsNotTypeOf(Type value, Type requiredType, string valueName)
        {
            if (value != requiredType)
                throw new GuardAgainstValueException(String.Format("Value '{0}' is not of required type {1}.",
                    valueName, requiredType.ToString()));
        }
    }

    /// <summary>
    /// The exception that is thrown when an argument GuardAgainst fails. 
    /// </summary>
    public class GuardAgainstArgumentException : ArgumentException
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="GuardAgainstArgumentException"/> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public GuardAgainstArgumentException(string message) : base(message) { }
    }

    /// <summary>
    /// The exception that is thrown when a value GuardAgainst fails. 
    /// </summary>
    public class GuardAgainstValueException : InvalidOperationException
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="GuardAgainstValueException"/> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public GuardAgainstValueException(string message) : base(message) { }
    }
}
