#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Core
{
    using System;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.Windows.Forms;

    /// <summary>
    /// ScreenshotHelper provides methods to get various screenshot images. Not 
    /// tested for thread safeness.
    /// </summary>
    public static class ScreenshotHelper
    {

        /// <summary>
        /// Snaps a screenshot of all monitors attached to the system.
        /// </summary>
        /// <returns>Image of all monitors attached to the system.</returns>
        public static Image SnapAllMonitorScreenshot()
        {
            int xMinimum = 0;
            int xMaximum = 0;
            int yMinimum = 0;
            int yMaximum = 0;
            int screenOffsetFromZeroX = 0;
            int screenOffsetFromZeroY = 0;

            foreach (Screen screen in Screen.AllScreens)
            {
                if (screen.Bounds.Left < xMinimum)
                    xMinimum = screen.Bounds.Left;
                if (screen.Bounds.Top < yMinimum)
                    yMinimum = screen.Bounds.Top;
                if (screen.Bounds.Right > xMaximum)
                    xMaximum = screen.Bounds.Right;
                if (screen.Bounds.Bottom > yMaximum)
                    yMaximum = screen.Bounds.Bottom;
            }

            if (xMinimum < 0)
                screenOffsetFromZeroX = Math.Abs(xMinimum);
            if (yMinimum < 0)
                screenOffsetFromZeroY = Math.Abs(yMinimum);

            Bitmap screenshot = new Bitmap(xMaximum + screenOffsetFromZeroX, yMaximum + screenOffsetFromZeroY, PixelFormat.Format32bppArgb);
            using (Graphics screenShotGraphics = Graphics.FromImage(screenshot))
            {
                foreach (Screen screen in Screen.AllScreens)
                {
                    screenShotGraphics.CopyFromScreen(screen.Bounds.X, screen.Bounds.Y,
                        screen.Bounds.X + screenOffsetFromZeroX, screen.Bounds.Y + screenOffsetFromZeroY,
                        screen.Bounds.Size, CopyPixelOperation.SourceCopy);
                }
            }
            return screenshot;
        }

        /// <summary>
        /// Snaps a screenshot of all open forms in Application.OpenForms. This does
        /// not attempt to place the forms in their relative positions within the
        /// monitor array; instead it stacks them vertically.
        /// </summary>
        /// <returns>Image of all open forms.</returns>
        public static Image SnapAllOpenFormsScreenshot()
        {
            int height = 0;
            int width = 0;
            foreach (Form form in Application.OpenForms)
            {
                if (!form.InvokeRequired)
                {
                    if (form.Width > width)
                        width = form.Width;
                    height += form.Height;
                }
            }

            Bitmap screenshot = new Bitmap(width, height, PixelFormat.Format32bppArgb);
            using (Graphics screenShotGraphics = Graphics.FromImage(screenshot))
            {
                int top = 0;
                foreach (Form form in Application.OpenForms)
                {
                    if (!form.InvokeRequired)
                    {
                        screenShotGraphics.CopyFromScreen(form.Bounds.X, form.Bounds.Y,
                            0, top, form.Bounds.Size, CopyPixelOperation.SourceCopy);
                        top += form.Height;
                    }
                }
            }
            return screenshot;
        }

        /// <summary>
        /// Snaps a screenshot of the supplied control.
        /// </summary>
        /// <param name="control">The control.</param>
        /// <returns>Image of the supplied control.</returns>
        public static Image SnapControlScreenshot(Control control)
        {
            Bitmap screenshot = new Bitmap(control.Bounds.Width,
                control.Bounds.Height, PixelFormat.Format32bppArgb);
            using (Graphics screenShotGraphics = Graphics.FromImage(screenshot))
            {
                screenShotGraphics.CopyFromScreen(control.Bounds.X, control.Bounds.Y,
                    0, 0, control.Bounds.Size, CopyPixelOperation.SourceCopy);
            }
            return screenshot;
        }

        /// <summary>
        /// Snaps a screenshot of the first open form in Application.OpenForms.
        /// </summary>
        /// <returns>Image of the first open form.</returns>
        public static Image SnapFirstOpenFormScreenshot()
        {
            if (Application.OpenForms.Count > 0)
                return SnapControlScreenshot(Application.OpenForms[0]);
            else
                return null;
        }

        /// <summary>
        /// Snaps a screenshot of the primary monitor.
        /// </summary>
        /// <returns>Image of the primary monitor.</returns>
        public static Image SnapPrimaryMonitorScreenshot()
        {
            Bitmap screenshot = new Bitmap(Screen.PrimaryScreen.Bounds.Width,
                Screen.PrimaryScreen.Bounds.Height, PixelFormat.Format32bppArgb);
            using (Graphics screenShotGraphics = Graphics.FromImage(screenshot))
            {
                screenShotGraphics.CopyFromScreen(Screen.PrimaryScreen.Bounds.X,
                    Screen.PrimaryScreen.Bounds.Y, 0, 0, Screen.PrimaryScreen.Bounds.Size,
                    CopyPixelOperation.SourceCopy);
            }
            return screenshot;
        }
    }
}
