#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema.Interface
{
    using System;
    using System.Collections;
    using System.Reflection;
    using DevExpress.Xpo;
    using DevExpress.Xpo.DB;
    using DevExpressContrib.Core;

    /// <summary>
    /// SchemaVersionBase serves as a base class for implementations of ISchemaVersion.
    /// It contains most of the required functionality.
    /// </summary>
    /// <typeparam name="T">The data type used for the version information.</typeparam>
    public abstract class SchemaVersionBase<T> : ISchemaVersion, IDisposable
    {

        private const string ClassDoesNotHaveAPropertyNamed = "{0} does not have a property named {1}.";
        private Session schemaSession = null;
        //private const string schemaVersionHasMoreThanOneRecord = "The SchemaVersion class has more than one record.";


        /// <summary>
        /// Initializes a new instance of the <see cref="SchemaVersionBase&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="dataLayer">The XPO data layer to use when accessing the data store.</param>
        public SchemaVersionBase(IDataLayer dataLayer)
            : this()
        {
            DataLayer = dataLayer;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SchemaVersionBase&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string used to attach to the data store.</param>
        public SchemaVersionBase(string connectionString)
            : this()
        {
            ConnectionString = connectionString;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SchemaVersionBase&lt;T&gt;"/> class.
        /// </summary>
        public SchemaVersionBase() { }


        /// <summary>
        /// Gets or sets the connection string. If the DataLayer is also set this
        /// value is ignored.
        /// </summary>
        /// <value>The connection string.</value>
        public string ConnectionString { get; set; }

        /// <summary>
        /// Gets the current schema version from the data store.
        /// </summary>
        /// <value>
        /// The value (as an object) of the current data store schema version.
        /// Returns null of the schema version does not exist.
        /// </value>
        public object CurrentSchemaVersion { get { return GetCurrentSchemaVersion(); } }

        /// <summary>
        /// Gets or sets the XPO data layer use to access the data store.
        /// </summary>
        /// <value>The data layer.</value>
        public IDataLayer DataLayer { get; set; }

        /// <summary>
        /// Gets the DateTimeKind DateTime that the SchemaVersion was updated.
        /// </summary>
        /// <value>The DateTimeKind.</value>
        public virtual DateTimeKind DateVersionUpdatedKind { get { return DateTimeKind.Utc; } }

        /// <summary>
        /// Gets the property name of the DateTime that the SchemaVersion was updated.
        /// If this is null or empty, no attempt will be made to store the updated time.
        /// </summary>
        /// <value>The property name of the DateTime that the SchemaVersion was updated.</value>
        public virtual string DateVersionUpdatedPropertyName { get { return String.Empty; } }

        /// <summary>
        /// Determines whether the schema version has been updated to the
        /// <see cref="RequiredSchemaVersion"/> current. Uses the
        /// <see cref="CurrentSchemaVersion"/> as the value to check against.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the <see cref="CurrentSchemaVersion"/> value in the
        /// data store is equal to the <see cref="RequiredSchemaVersion"/>; otherwise, <c>false</c>.
        /// </value>
        public virtual bool HasSchemaVersionBeenUpdated
        {
            get { return new CaseInsensitiveComparer().Compare(RequiredSchemaVersion, CurrentSchemaVersion) == 0; }
        }

        /// <summary>
        /// Gets the required schema version (as an object). This value is what
        /// the schema version should be for the current version of the application.
        /// </summary>
        /// <value>The value (as an object) of the required data source schema version.</value>
        public object RequiredSchemaVersion { get { return GetRequiredSchemaVersion(); } }

        /// <summary>
        /// Gets the name of the SchemaVersion property in the data store.
        /// </summary>
        /// <value>The name of the SchemaVersion property.</value>
        public abstract string VersionPropertyName { get; }

        /// <summary>
        /// Gets the type of the XPO object from the data store. This must be a
        /// descendant of XPObjectBase.
        /// </summary>
        /// <value>The type of the XPO object.</value>
        public abstract Type XpoObjectType { get; }


        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            if (schemaSession != null)
                schemaSession.Dispose();
        }

        /// <summary>
        /// Gets the current schema version from the data source.
        /// </summary>
        /// <value>The value (as an object) of the current data source schema version.
        /// Returns null of the schema version does not exist.</value>
        /// <returns>The value (as an object) of the current data source schema version.
        /// Returns null of the schema version does not exist.</returns>
        public T GetCurrentSchemaVersion()
        {
            return (T)GetPropertyValue(VersionPropertyName, typeof(T), default(T));
        }

        /// <summary>
        /// Gets the date that the SchemaVersion was updated.
        /// </summary>
        /// <returns>A DateTime that the SchemaVersion was updated.</returns>
        public DateTime GetDateVersionUpdated()
        {
            if (String.IsNullOrEmpty(DateVersionUpdatedPropertyName))
                return DateTime.MinValue;

            return (DateTime)GetPropertyValue(DateVersionUpdatedPropertyName, typeof(DateTime), DateTime.MinValue);
        }

        /// <summary>
        /// Gets the required schema version (as an object). This value is what
        /// the schema version should be for the current version of the application.
        /// </summary>
        /// <returns>
        /// The value (as an object) of the required data source schema version.
        /// </returns>
        public abstract T GetRequiredSchemaVersion();

        /// <summary>
        /// Updates the schema version to the <see cref="RequiredSchemaVersion"/>.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the version was updated; otherwise <f>false</f>.
        /// </returns>
        public virtual bool UpdateSchemaVersion()
        {
            Session session = GetSession();
            using (XPCollection collection = new XPCollection(session, XpoObjectType))
            {
                GuardAgainst.ValueIsGreaterThan(collection.Count, 1, "collection.Count");
                XPBaseObject item;
                if (collection.Count == 0)
                    item = (XPBaseObject)Activator.CreateInstance(XpoObjectType, session);
                else
                    item = (XPBaseObject)collection[0];
                return UpdateSchemaVersion(item);
            }
        }

        private IDataLayer GetDataLayer()
        {
            if (DataLayer == null)
            {
                GuardAgainst.ValueIsNullOrEmpty(ConnectionString, "ConnectionString");
                DataLayer = XpoDefault.GetDataLayer(ConnectionString, AutoCreateOption.None);
            }
            return DataLayer;
        }

        private object GetPropertyValue(string propertyName, Type expectedType, object defaultReturnValue)
        {
            Session session = GetSession();
            using (XPCollection collection = new XPCollection(session, XpoObjectType))
            {
                GuardAgainst.ValueIsGreaterThan(collection.Count, 1, "collection.Count");
                if (collection.Count == 0)
                    return defaultReturnValue;
                else
                {
                    XPBaseObject item = (XPBaseObject)collection[0];
                    PropertyInfo propertyInfo = XpoObjectType.GetProperty(propertyName);
                    if (propertyInfo != null)
                    {
                        GuardAgainst.ValueTypeIsNotTypeOf(propertyInfo.PropertyType, expectedType, "propertyInfo.PropertyType");
                        return propertyInfo.GetValue(item, null);
                    }
                    else
                        throw new InvalidOperationException(String.Format(ClassDoesNotHaveAPropertyNamed, XpoObjectType.ToString(), VersionPropertyName));
                }
            }
        }

        private Session GetSession()
        {
            if (schemaSession == null)
                schemaSession = new Session(GetDataLayer());
            return schemaSession;
        }

        private bool SetPropertyValue(XPBaseObject item, string propertyName, Type expectedType, object value)
        {
            PropertyInfo propertyInfo = XpoObjectType.GetProperty(propertyName);
            if (propertyInfo != null)
            {
                GuardAgainst.ValueTypeIsNotTypeOf(propertyInfo.PropertyType, expectedType, "propertyInfo.PropertyType");
                propertyInfo.SetValue(item, value, null);
                item.Save();
                return true;
            }
            else
                throw new InvalidOperationException(String.Format(ClassDoesNotHaveAPropertyNamed,
                    XpoObjectType.ToString(), propertyName));
        }

        private bool UpdateSchemaVersion(XPBaseObject item)
        {
            bool result = SetPropertyValue(item, VersionPropertyName, typeof(T), RequiredSchemaVersion);
            if (result && !String.IsNullOrEmpty(DateVersionUpdatedPropertyName))
                result = SetPropertyValue(item, DateVersionUpdatedPropertyName, typeof(DateTime),
                    DateTime.SpecifyKind(DateTime.Now, DateVersionUpdatedKind));
            return result;
        }
    }
}