#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema.Interface
{
    using System;
    using DevExpress.Xpo;

    /// <summary>
    /// Information about a log entry
    /// </summary>
    public class UpdateSchemaLogEntry : XPObject
    {

        private int counter;
        private string exceptionText;
        private string exceptionType;
        private DateTime logTime;
        private string message;
        private UpdateSchemaLogLevel updateSchemaLogLevel;


        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchemaLogEntry"/> class.
        /// </summary>
        /// <param name="session">The session.</param>
        /// <param name="message">The log message.</param>
        /// <param name="exceptionType">Type of the exception.</param>
        /// <param name="exceptionText">The exception text.</param>
        /// <param name="updateSchemaLogLevel">The log level.</param>
        /// <param name="counter">The counter.</param>
        public UpdateSchemaLogEntry(Session session, string message, string exceptionType,
            string exceptionText, UpdateSchemaLogLevel updateSchemaLogLevel, int counter)
            : this(session)
        {
            Counter = counter;
            Message = message;
            if (!String.IsNullOrEmpty(exceptionType))
                ExceptionType = exceptionType;
            if (!String.IsNullOrEmpty(exceptionText))
                ExceptionText = exceptionText;
            UpdateSchemaLogLevel = updateSchemaLogLevel;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchemaLogEntry"/> class.
        /// </summary>
        /// <param name="session">The session.</param>
        public UpdateSchemaLogEntry(Session session)
            : base(session)
        {
            LogTime = DateTime.Now;
        }


        /// <summary>
        /// Gets or sets the counter. Log entries can be added faster than the
        /// resolution of DateTime. So add a counter to preserve order.
        /// </summary>
        /// <value>The counter.</value>
        public int Counter
        {
            get { return counter; }
            set { SetPropertyValue("Counter", ref counter, value); }
        }

        /// <summary>
        /// Gets or sets the exception text.
        /// </summary>
        /// <value>The exception text.</value>
        [Size(1024)]
        public string ExceptionText
        {
            get { return exceptionText; }
            set { SetPropertyValue("ExceptionText", ref exceptionText, value); }
        }

        /// <summary>
        /// Gets or sets the type of the exception.
        /// </summary>
        /// <value>The type of the exception.</value>
        [Size(50)]
        public string ExceptionType
        {
            get { return exceptionType; }
            set { SetPropertyValue("ExceptionType", ref exceptionType, value); }
        }

        /// <summary>
        /// Gets the full exception text with the exception type and the explanitory text.
        /// </summary>
        /// <value>The full exception.</value>
        [NonPersistent]
        public string FullException
        {
            get
            {
                if (String.IsNullOrEmpty(ExceptionText))
                    return null;
                else
                    return String.Format("{0} - {1}", ExceptionType, ExceptionText);
            }
        }

        /// <summary>
        /// Gets or sets the time for this entry.
        /// </summary>
        /// <value>The time of this entry.</value>
        public DateTime LogTime
        {
            get { return logTime; }
            set { SetPropertyValue("LogTime", ref logTime, value); }
        }

        /// <summary>
        /// Gets or sets the log message.
        /// </summary>
        /// <value>The log message.</value>
        [Size(256)]
        public string Message
        {
            get { return message; }
            set { SetPropertyValue("Message", ref message, value); }
        }

        /// <summary>
        /// Gets or sets the log level for this entry.
        /// </summary>
        /// <value>The log level.</value>
        [ValueConverter(typeof(UpdateSchemaLogLevelConverter))]
        public UpdateSchemaLogLevel UpdateSchemaLogLevel
        {
            get { return updateSchemaLogLevel; }
            set { SetPropertyValue("UpdateSchemaLogLevel", ref updateSchemaLogLevel, value); }
        }
    }
}
