#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema.Interface
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Linq;
    using DevExpress.Xpo;
    using DevExpress.Xpo.DB;

    /// <summary>
    /// The logging class with which to log information about a process.
    /// </summary>
    public class UpdateSchemaLogger : IDisposable
    {

        private int counter;
        private XPCollection<UpdateSchemaLogEntry> entryCollection;
        private InMemoryDataStore inMemoryDataStore;
        private Session session;


        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchemaLogger"/> class.
        /// </summary>
        public UpdateSchemaLogger()
        {
            inMemoryDataStore = new InMemoryDataStore(new DataSet("Log"), AutoCreateOption.DatabaseAndSchema);
            DataLayer = new SimpleDataLayer(inMemoryDataStore);
            session = new Session(DataLayer);
            entryCollection = new XPCollection<UpdateSchemaLogEntry>(session);
            counter = 0;
        }


        /// <summary>
        /// Gets the IDataLayer for the logger data.
        /// </summary>
        /// <value>The logger IDataLayer.</value>
        public IDataLayer DataLayer { get; private set; }


        /// <summary>
        /// Returns the count of entries with the specified log level.
        /// </summary>
        /// <param name="logLevel">The log level.</param>
        /// <returns>Count of entries with the specified log level.</returns>
        public int Count(UpdateSchemaLogLevel logLevel)
        {
            IEnumerable<DataRow> entries = inMemoryDataStore.Data.Tables["UpdateSchemaLogEntry"].AsEnumerable();
            return (from data in entries
                    where data.Field<string>("UpdateSchemaLogLevel").Equals(logLevel.ToString())
                    select data).Count();
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            if (session != null)
            {
                session.Disconnect();
                session.Dispose();
            }
            if (DataLayer != null)
            {
                if (DataLayer.Connection != null)
                    DataLayer.Connection.Close();
                DataLayer.Dispose();
            }
        }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        public void Initialize()
        {
            XpoUpdateSchema();
        }

        /// <summary>
        /// Logs the specified message using the level LogLevel.Information.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Log(string message)
        {
            Log(message, UpdateSchemaLogLevel.Information);
        }

        /// <summary>
        /// Logs the specified message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="level">The logging level.</param>
        public void Log(string message, UpdateSchemaLogLevel level)
        {
            Log(message, String.Empty, String.Empty, level);
        }

        /// <summary>
        /// Logs the specified message using the level LogLevel.Error.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exceptionType">Type of the exception.</param>
        /// <param name="exceptionText">The text of the exception.</param>
        public void Log(string message, string exceptionType, string exceptionText)
        {
            Log(message, exceptionType, exceptionText, UpdateSchemaLogLevel.Error);
        }

        /// <summary>
        /// Logs the specified message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exceptionType">Type of the exception.</param>
        /// <param name="exceptionText">The text of the exception.</param>
        /// <param name="level">The logging level.</param>
        public void Log(string message, string exceptionType, string exceptionText, UpdateSchemaLogLevel level)
        {
            using (UpdateSchemaLogEntry entry = new UpdateSchemaLogEntry(session, message, exceptionType, exceptionText, level, counter++))
            {
                entry.Save();
                entryCollection.Add(entry);
            }
        }

        /// <summary>
        /// Saves the log to the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file to save.</param>
        public void Save(string fileName)
        {
            inMemoryDataStore.Data.WriteXml(fileName, XmlWriteMode.WriteSchema);
        }

        private void XpoUpdateSchema()
        {
            Type[] types = new Type[] { typeof(UpdateSchemaLogEntry) };
            session.UpdateSchema(types);
            session.CreateObjectTypeRecords(types);
        }
    }
}
