#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// Class to store information about what is to be updated.
    /// </summary>
    public class UpdateItemInfo
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemInfo"/> class.
        /// </summary>
        /// <param name="updateSchemaType">Type of the schema update.</param>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="size">The new size of the column.</param>
        /// <param name="ignoreConditions">The update schema conditions to ignore.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName,
            string columnName, int size, UpdateSchemaIgnoreConditions ignoreConditions, IServicesHelper servicesHelper)
            : this(updateSchemaType, tableName, columnName, ignoreConditions, servicesHelper)
        {
            GuardAgainst.ArgumentIntegerOutOfRange(size, -1, Int32.MaxValue, "size");
            Size = size;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemInfo"/> class.
        /// </summary>
        /// <param name="updateSchemaType">Type of the schema update.</param>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="oldColumnName">Old name of the column.</param>
        /// <param name="ignoreConditions">The update schema conditions to ignore.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName,
            string columnName, string oldColumnName, UpdateSchemaIgnoreConditions ignoreConditions, IServicesHelper servicesHelper)
            : this(updateSchemaType, tableName, columnName, ignoreConditions, servicesHelper)
        {
            GuardAgainst.ArgumentIsNullOrEmpty(oldColumnName, "oldColumnName");
            OldColumnName = oldColumnName;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemInfo"/> class.
        /// </summary>
        /// <param name="updateSchemaType">Type of the schema update.</param>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="ignoreConditions">The update schema conditions to ignore.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName,
            string columnName, UpdateSchemaIgnoreConditions ignoreConditions,
            IServicesHelper servicesHelper)
        {
            GuardAgainst.ArgumentIsNullOrEmpty(tableName, "tableName");
            GuardAgainst.ArgumentIsNullOrEmpty(columnName, "columnName");

            this.servicesHelper = servicesHelper;
            UpdateSchemaType = updateSchemaType;
            TableName = tableName;
            ColumnName = columnName;
            IgnoreConditions = ignoreConditions;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemInfo"></see> class.
        /// </summary>
        /// <param name="updateSchemaType">Type of the schema update.</param>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="size">The new size of the column.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName, string columnName, int size, IServicesHelper servicesHelper)
            : this(updateSchemaType, tableName, columnName, size, UpdateSchemaIgnoreConditions.None, servicesHelper) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemInfo"></see> class.
        /// </summary>
        /// <param name="updateSchemaType">Type of the schema update.</param>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="oldColumnName">Old name of the column.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName, string columnName,
            string oldColumnName, IServicesHelper servicesHelper)
            : this(updateSchemaType, tableName, columnName, oldColumnName, UpdateSchemaIgnoreConditions.None, servicesHelper) { }


        /// <summary>
        /// Gets or sets the name of the column.
        /// </summary>
        /// <value>The name of the column.</value>
        public string ColumnName { get; set; }

        /// <summary>
        /// Gets or sets the update schema conditions to ignore. This may be "None".
        /// </summary>
        /// <value>The conditions to ignore.</value>
        public UpdateSchemaIgnoreConditions IgnoreConditions { get; set; }

        /// <summary>
        /// Gets or sets the old name of the column for renaming columns.
        /// </summary>
        /// <value>The old name of the column.</value>
        public string OldColumnName { get; set; }

        private IServicesHelper servicesHelper { get; set; }

        /// <summary>
        /// Gets or sets the new size of a string column.
        /// </summary>
        /// <value>The new size of the string column.</value>
        public int Size { get; set; }

        /// <summary>
        /// Gets or sets the name of the table.
        /// </summary>
        /// <value>The name of the table.</value>
        public string TableName { get; set; }

        /// <summary>
        /// Gets or sets the type of the schema update.
        /// </summary>
        /// <value>The type of the schema update.</value>
        public UpdateSchemaType UpdateSchemaType { get; set; }


        //public UpdateItemInfo(UpdateSchemaType updateSchemaType, string tableName,
        //    string columnName, UpdateSchemaIgnoreConditions ignoreConditions)
        //    : this(updateSchemaType, tableName, columnName, ignoreConditions, null) { }
        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            string result;
            switch (UpdateSchemaType)
            {
                case UpdateSchemaType.ShrinkColumnStringLength:
                    result = String.Format(servicesHelper.GetString("UpdateItemInfoToStringShrinkColumnStringLength"),
                        TableName, ColumnName, Size);
                    break;
                case UpdateSchemaType.GrowColumnStringLength:
                    result = String.Format(servicesHelper.GetString("UpdateItemInfoToStringGrowColumnStringLength"),
                        TableName, ColumnName, Size);
                    break;
                case UpdateSchemaType.RenameColumn:
                    result = String.Format(servicesHelper.GetString("UpdateItemInfoToStringRenameColumn"),
                        TableName, OldColumnName, TableName, ColumnName);
                    break;
                case UpdateSchemaType.DeleteColumn:
                    result = String.Format(servicesHelper.GetString("UpdateItemInfoToStringDeleteColumn"),
                        TableName, ColumnName);
                    break;
                default:
                    result = String.Format(servicesHelper.GetString("UpdateItemInfoToStringDefault"),
                        UpdateSchemaType.ToString(), TableName, ColumnName);
                    break;
            }
            return result;
        }
    }
}
