#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using DevExpress.Xpo;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// Class which queries an assemblies meta-data and gathers all of the requested updates.
    /// </summary>
    internal class UpdateItemManager
    {

        private IServicesHelper servicesHelper;
        private List<UpdateItemInfo> updateItems;
        private UpdateSchemaLogger updateSchemaLogger;


        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateItemManager"/> class.
        /// </summary>
        /// <param name="updateItems">The list to store update items.</param>
        /// <param name="updateSchemaLogger">The logging object.</param>
        /// <param name="servicesHelper">The services helper.</param>
        internal UpdateItemManager(List<UpdateItemInfo> updateItems,
            UpdateSchemaLogger updateSchemaLogger, IServicesHelper servicesHelper)
        {
            GuardAgainst.ArgumentIsNull(updateItems, "updateItems");
            GuardAgainst.ArgumentIsNull(servicesHelper, "servicesHelper");

            this.servicesHelper = servicesHelper;
            this.updateSchemaLogger = updateSchemaLogger;
            this.updateItems = updateItems;
        }


        private static Attribute GetAttribute(MemberInfo member, Type attributeType)
        {
            object[] attribute = member.GetCustomAttributes(attributeType, true);
            if (attribute.Length >= 1)
                return (Attribute)attribute[0];
            return null;
        }

        private static Attribute GetAttribute(Type type, Type attributeType)
        {
            object[] attribute = type.GetCustomAttributes(attributeType, true);
            if (attribute.Length >= 1)
                return (Attribute)attribute[0];
            return null;
        }

        private string GetColumnName(MemberInfo member)
        {
            if (IsPersistent(member))
            {
                PersistentAttribute attribute = (PersistentAttribute)GetAttribute(member, typeof(PersistentAttribute));
                if (attribute != null && !String.IsNullOrEmpty(attribute.MapTo))
                    return attribute.MapTo;
                else
                    return member.Name;
            }
            return null;
        }

        private Type GetPropertyType(MemberInfo memberItem)
        {
            if (memberItem.MemberType == MemberTypes.Property)
                return (Type)memberItem.GetType().GetProperty("PropertyType").GetValue(memberItem, null);
            if (memberItem.MemberType == MemberTypes.Field)
                return (Type)memberItem.GetType().GetProperty("FieldType").GetValue(memberItem, null);
            throw new InvalidOperationException("Invalid member type. Must be either a Field or Property.");
        }

        private static MapInheritanceType GetTableMapType(Type type)
        {
            MapInheritanceAttribute attribute = (MapInheritanceAttribute)GetAttribute(type, typeof(MapInheritanceAttribute));
            return attribute == null ? MapInheritanceType.OwnTable : attribute.MapType;
        }

        private static bool IsPersistent(MemberInfo member)
        {
            NonPersistentAttribute nonPersistent = (NonPersistentAttribute)GetAttribute(member, typeof(NonPersistentAttribute));
            return (nonPersistent == null);
        }

        private static bool IsPersistent(Type type)
        {
            object[] attribute = type.GetCustomAttributes(typeof(NonPersistentAttribute), false);
            return (attribute.Length == 0);
        }

        private void RetrieveUpdateItems(IUpdateSchemaProvider updateSchemaProvider, MemberInfo[] memberInfoItems, string tableName)
        {
            foreach (MemberInfo memberItem in memberInfoItems)
            {
                string columnName = GetColumnName(memberItem);
                if (GetPropertyType(memberItem) == typeof(string) || updateSchemaProvider.ColumnIsString(tableName, columnName))
                {
                    int currentColumnSize = updateSchemaProvider.GetStringColumnSize(tableName, columnName);
                    SizeAttribute sizeAttribute = (SizeAttribute)GetAttribute(memberItem, typeof(SizeAttribute));
                    int requiredColumnSize = (sizeAttribute == null) ? SizeAttribute.DefaultStringMappingFieldSize : sizeAttribute.Size;
                    if (currentColumnSize != requiredColumnSize)
                    {
                        UpdateSchemaType updateType = (currentColumnSize < requiredColumnSize) ? UpdateSchemaType.GrowColumnStringLength : UpdateSchemaType.ShrinkColumnStringLength;
                        ResizeOptionsAttribute resizeOptions = (ResizeOptionsAttribute)GetAttribute(memberItem, typeof(ResizeOptionsAttribute));
                        if (resizeOptions == null)
                            updateItems.Add(new UpdateItemInfo(updateType, tableName, columnName, requiredColumnSize, servicesHelper));
                        else
                            updateItems.Add(new UpdateItemInfo(updateType, tableName, columnName, requiredColumnSize, resizeOptions.IgnoreConditions, servicesHelper));
                    }
                }
                object[] renameAttributes = memberItem.GetCustomAttributes(typeof(RenameColumnAttribute), true);
                foreach (object item in renameAttributes)
                {
                    RenameColumnAttribute renameColumnAttribute = (RenameColumnAttribute)item;
                    updateItems.Add(new UpdateItemInfo(UpdateSchemaType.RenameColumn, tableName,
                        columnName, renameColumnAttribute.OldColumnName, renameColumnAttribute.IgnoreConditions, servicesHelper));
                }
            }
        }

        /// <summary>
        /// Gets the name of the table from the meta-data of the supplied type.
        /// </summary>
        /// <param name="type">The supplied type.</param>
        /// <returns>The name of the table for the supplied type.</returns>
        internal static string GetTableName(Type type)
        {
            if (IsPersistent(type) && GetTableMapType(type) == MapInheritanceType.OwnTable)
            {
                MapToAttribute mapToAttribute = (MapToAttribute)GetAttribute(type, typeof(MapToAttribute));
                PersistentAttribute persistentAttribute = (PersistentAttribute)GetAttribute(type, typeof(PersistentAttribute));
                if (persistentAttribute != null && !String.IsNullOrEmpty(persistentAttribute.MapTo))
                    return persistentAttribute.MapTo;
                else if (mapToAttribute != null && !String.IsNullOrEmpty(mapToAttribute.MappingName))
                    return mapToAttribute.MappingName;
                else
                    return type.Name;
            }
            if (type.BaseType != typeof(XPBaseObject))
                return GetTableName(type.BaseType);
            return null;
        }

        /// <summary>
        /// Retrieves the update items.
        /// </summary>
        /// <param name="assemblies">The assemblies in which to look for requested updates.</param>
        /// <param name="updateSchemaProvider">The update schema provider.</param>
        /// <returns><c>true</c> if there were no problems obtaining the update items; otherwise <c>false</c>.</returns>
        internal bool RetrieveUpdateItems(Assembly[] assemblies, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            foreach (Assembly assembly in assemblies)
            {
                Type[] types = assembly.GetTypes();
                if (!RetrieveUpdateItems(types, updateSchemaProvider))
                    result = false;
            }
            return result;
        }

        internal bool RetrieveUpdateItems(Type[] types, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            foreach (Type type in types)
            {
                if (type.IsSubclassOf(typeof(XPBaseObject)))
                {
                    string tableName = GetTableName(type);
                    if (!String.IsNullOrEmpty(tableName))
                    {
                        object[] attributes = type.GetCustomAttributes(typeof(DeleteColumnAttribute), true);
                        foreach (object item in attributes)
                        {
                            DeleteColumnAttribute deleteColumnAttribute = (DeleteColumnAttribute)item;
                            updateItems.Add(new UpdateItemInfo(UpdateSchemaType.DeleteColumn, tableName,
                                deleteColumnAttribute.ColumnNameToDelete, deleteColumnAttribute.IgnoreConditions,
                                servicesHelper));
                        }

                        PropertyInfo[] propertyInfoItems = type.GetProperties();
                        RetrieveUpdateItems(updateSchemaProvider, propertyInfoItems, tableName);

                        FieldInfo[] fieldInfoItems = type.GetFields();
                        RetrieveUpdateItems(updateSchemaProvider, fieldInfoItems, tableName);
                    }
                }
            }
            foreach (UpdateItemInfo item in updateItems)
            {
                updateSchemaLogger.Log(String.Format(servicesHelper.GetString("UpdateItemInformation"), item.ToString()));
            }
            return result;
        }
    }
}
