#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using DevExpress.Xpo;
    using DevExpress.Xpo.DB;
    using DevExpress.Xpo.Exceptions;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// The main class for the DevExpressContrib.Xpo.UpdateSchema framework. For the
    /// end using developer this should be the only class with which they interact.
    /// </summary>
    public class UpdateSchema : IDisposable
    {

        private Dictionary<string, Type> availableProviders = null;
        private ISchemaVersion schemaVersion = null;
        private List<UpdateItemInfo> updateItems;


        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchema"/> class.
        /// </summary>
        /// <param name="connectionString">The database specific connection string.</param>
        /// <param name="schemaVersion">Class used to update the schema version.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateSchema(string connectionString, ISchemaVersion schemaVersion, IServicesHelper servicesHelper)
        {
            GuardAgainst.ArgumentIsNullOrEmpty(connectionString, "connectionString");

            this.schemaVersion = schemaVersion;
            ServicesHelper = servicesHelper;
            ConnectionString = connectionString;
            AutoCreateOption = AutoCreateOption.DatabaseAndSchema;
            UpdateSchemaTypes = UpdateSchemaType.ShrinkColumnStringLength
                | UpdateSchemaType.GrowColumnStringLength
                | UpdateSchemaType.RenameColumn
                | UpdateSchemaType.DeleteColumn;
            UpdateSchemaLogger = new UpdateSchemaLogger();
            updateItems = new List<UpdateItemInfo>();
            AdditionalProviderAssemblies = new List<Assembly>();
            AbortUpdateWithFirstError = false;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchema"></see> class.
        /// </summary>
        /// <param name="connectionString">The database specific connection string.</param>
        /// <param name="schemaVersion">Class used to update the schema version.</param>
        public UpdateSchema(string connectionString, ISchemaVersion schemaVersion) : this(connectionString, schemaVersion, null) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchema"/> class.
        /// </summary>
        /// <param name="connectionString">The database specific connection string.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public UpdateSchema(string connectionString, IServicesHelper servicesHelper) : this(connectionString, null, servicesHelper) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSchema"></see> class.
        /// </summary>
        /// <param name="connectionString">The database specific connection string.</param>
        public UpdateSchema(string connectionString) : this(connectionString, null, null) { }


        /// <summary>
        /// Gets or sets a value indicating whether to abort the update when the first error occurs.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if aborting the update when the first error occurs; otherwise, <c>false</c>.
        /// </value>
        public bool AbortUpdateWithFirstError { get; set; }

        /// <summary>
        /// Gets or sets the additional assemblies in which to check for update schema providers.
        /// </summary>
        /// <value>The additional assemblies to check.</value>
        public List<Assembly> AdditionalProviderAssemblies { get; private set; }

        /// <summary>
        /// Gets or sets the <see cref="AutoCreateOption" />.
        /// </summary>
        /// <value>The auto create option.</value>
        public AutoCreateOption AutoCreateOption { get; set; }

        /// <summary>
        /// Gets the connection string which was passed into the constructor.
        /// </summary>
        /// <value>The connection string.</value>
        public string ConnectionString { get; private set; }

        /// <summary>
        /// Gets the number of errors encountered during the update.
        /// </summary>
        /// <value>The error count.</value>
        public int ErrorCount
        {
            get { return UpdateSchemaLogger.Count(UpdateSchemaLogLevel.Error); }
        }

        /// <summary>
        /// Gets or sets the services helper.
        /// </summary>
        /// <value>The services helper.</value>
        public IServicesHelper ServicesHelper { get; set; }

        /// <summary>
        /// Gets or sets the information logger.
        /// </summary>
        /// <value>The logger.</value>
        public UpdateSchemaLogger UpdateSchemaLogger { get; private set; }

        /// <summary>
        /// Gets or sets the update schema types that should be performed. If the
        /// using developer does not want to perform some of the updates this can
        /// be set to something other than the default which is all supported updates.
        /// </summary>
        /// <value>The update schema types.</value>
        public UpdateSchemaType UpdateSchemaTypes { get; set; }

        /// <summary>
        /// Gets the number of errors encountered during the update.
        /// </summary>
        /// <value>The warning count.</value>
        public int WarningCount
        {
            get { return UpdateSchemaLogger.Count(UpdateSchemaLogLevel.Warning); }
        }


        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            UpdateSchemaLogger.Dispose();
        }

        /// <summary>
        /// Saves the log to the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void SaveLog(string fileName)
        {
            UpdateSchemaLogger.Save(fileName);
        }

        /// <summary>
        /// Performs the updates on the supplied assembly.
        /// </summary>
        /// <param name="assembly">The assembly where the persistent XPO data is defined.</param>
        /// <returns><c>true</c> if the assembly is updated without error; otherwise <c>false</c>.</returns>
        public bool Update(Assembly assembly)
        {
            GuardAgainst.ArgumentIsNull(assembly, "assembly");

            return Update(new Assembly[] { assembly });
        }

        /// <summary>
        /// Performs the updates on the supplied assemblies.
        /// </summary>
        /// <param name="assemblies">The list of assemblies where the persistent XPO data is defined.</param>
        /// <returns><c>true</c> if the assemblies are updated without error; otherwise <c>false</c>.</returns>
        public bool Update(Assembly[] assemblies)
        {
            GuardAgainst.ArgumentIsNull(assemblies, "assemblies");
            GuardAgainst.ArgumentIntegerIsLessThanOrEqualToZero(assemblies.Length, "assemblies.Length");

            return Update(assemblies, new Type[] { });
        }

        /// <summary>
        /// Performs the updates on the specified type.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns><c>true</c> if the type is updated without error; otherwise <c>false</c>.</returns>
        public bool Update(Type type)
        {
            GuardAgainst.ArgumentIsNull(type, "type");

            return Update(new Type[] { type });
        }

        /// <summary>
        /// Performs the updates on the specified types.
        /// </summary>
        /// <param name="types">The list of types where the persistent XPO data is defined.</param>
        /// <returns><c>true</c> if the types are updated without error; otherwise <c>false</c>.</returns>
        public bool Update(Type[] types)
        {
            GuardAgainst.ArgumentIsNull(types, "types");
            GuardAgainst.ArgumentIntegerIsLessThanOrEqualToZero(types.Length, "types.Length");

            return Update(new Assembly[] { }, types);
        }

        /// <summary>
        /// Performs the updates on the specified assemblies and/or types.
        /// </summary>
        /// <param name="assemblies">The list of assemblies where the persistent XPO data is defined.</param>
        /// <param name="types">The list of types where the persistent XPO data is defined.</param>
        /// <returns><c>true</c> if the assemblies and/or types are updated without error; otherwise <c>false</c>.</returns>
        public bool Update(Assembly[] assemblies, Type[] types)
        {
            if (!ThereAreAssemblies(assemblies) && !ThereAreTypes(types))
                throw new ArgumentException("There must be at least one assembly or one type.");

            bool result = true;
            ServicesHelperInitialize();
            UpdateSchemaLogger.Initialize();
            UpdateSchemaLogger.Log(ServicesHelper.GetString("UpdateStarted"));
            IUpdateSchemaProvider updateSchemaProvider = null;
            try
            {
                updateSchemaProvider = GetUpdateSchemaProvider();
                UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("XpoProviderTypeString"), updateSchemaProvider.XpoProviderTypeString));
                updateSchemaProvider.UpdateSchemaLogger = UpdateSchemaLogger;
                if (updateSchemaProvider.DatabaseExists())
                {
                    updateSchemaProvider.Open();
                    RetrieveUpdateItems(assemblies, types, updateSchemaProvider);
                    if (updateItems.Count == 0)
                        return XpoUpdateSchema(assemblies, types);
                    result = UpdateItems(UpdateSchemaOrder.BeforeXpoSchemaUpdate, updateSchemaProvider)
                        && XpoUpdateSchema(assemblies, types)
                        && UpdateItems(UpdateSchemaOrder.AfterXpoSchemaUpdate, updateSchemaProvider);
                }
                else
                    result = XpoUpdateSchema(assemblies, types);
            }
            finally
            {
                if (updateSchemaProvider != null)
                    updateSchemaProvider.Dispose();
            }
            UpdateSchemaLogger.Log(ServicesHelper.GetString("UpdateComplete"));
            if (result && schemaVersion != null)
                result = UpdateSchemaVersion();

            if (result)
                result = ErrorCount == 0;
            return result;
        }

        private bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            if (UpdateAllowed(updateSchemaProvider, updateItem))
            {
                try
                {
                    result = updateSchemaProvider.DeleteColumn(order, updateItem);
                }
                catch (Exception ex)
                {
                    LogException(updateItem, ex);
                }
            }
            return result;
        }

        private void GetAvailableProviders()
        {
            availableProviders = new Dictionary<string, Type>();
            UpdateSchemaProviderRegister.GetAvailableProviders(availableProviders, AdditionalProviderAssemblies);
        }

        private IUpdateSchemaProvider GetUpdateSchemaProvider()
        {
            if (availableProviders == null)
                GetAvailableProviders();

            string xpoProviderName;
            const string CannotFindAppropriateUpdateSchemaProvider = "Cannot find appropriate DevExpressContrib.Xpo.UpdateSchema provider.";
            if (UpdateSchemaProviderHelper.TryGetXpoProvider(ConnectionString, out xpoProviderName))
            {
                Type providerType;
                if (availableProviders.TryGetValue(xpoProviderName.ToUpper(), out providerType))
                {
                    string connectionString = UpdateSchemaProviderHelper.RemoveXpoProviderTypeParameter(ConnectionString);
                    return (IUpdateSchemaProvider)Activator.CreateInstance(providerType, new object[] { connectionString, ServicesHelper });
                }
            }
            throw new InvalidOperationException(CannotFindAppropriateUpdateSchemaProvider,
                new CannotFindAppropriateConnectionProviderException(ConnectionString));
        }

        private void LogException(UpdateItemInfo updateItem, Exception exception)
        {
            string exceptionMessage = exception.Message;
            if (exception.InnerException != null)
                exceptionMessage += String.Format(" - {0}", exception.InnerException.Message);
            UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemFailure"), updateItem.ToString()), exception.GetType().ToString(), exceptionMessage, UpdateSchemaLogLevel.Error);
        }

        private bool ModifyColumnStringLength(UpdateSchemaOrder order, UpdateItemInfo updateItem, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            if (UpdateAllowed(updateSchemaProvider, updateItem))
            {
                bool growingStringLength = updateItem.UpdateSchemaType == UpdateSchemaType.GrowColumnStringLength;
                bool ignoreTruncate = UpdateSchemaIgnoreConditionsHelper.ContainsIgnoreCondition(updateItem.IgnoreConditions, UpdateSchemaIgnoreConditions.TruncateString);
                result = growingStringLength;
                if (!result)
                {
                    bool stringsWillBeTruncated = updateSchemaProvider.ColumnStringValuesWillBeTruncated(updateItem.TableName, updateItem.ColumnName, updateItem.Size);
                    result = ignoreTruncate || !stringsWillBeTruncated;
                    if (ignoreTruncate && stringsWillBeTruncated)
                    {
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("AttemptingToIgnoreTruncatedStrings"), updateItem.ToString()), UpdateSchemaLogLevel.Warning);
                    }
                }
                if (result)
                    try
                    {
                        result = updateSchemaProvider.ModifyStringColumnSize(order, updateItem);
                    }
                    catch (Exception ex)
                    {
                        LogException(updateItem, ex);
                    }
            }
            return result;
        }

        private bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            if (UpdateAllowed(updateSchemaProvider, updateItem))
            {
                try
                {
                    result = updateSchemaProvider.RenameColumn(order, updateItem);
                }
                catch (Exception ex)
                {
                    LogException(updateItem, ex);
                }
            }
            return result;
        }

        private bool RetrieveUpdateItems(Assembly[] assemblies, Type[] types, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            UpdateItemManager updateItemManager = new UpdateItemManager(updateItems, UpdateSchemaLogger, ServicesHelper);
            UpdateSchemaLogger.Log(ServicesHelper.GetString("RetrievingUpdateItems"));
            if (ThereAreTypes(types))
                result = updateItemManager.RetrieveUpdateItems(types, updateSchemaProvider);
            if (result && ThereAreAssemblies(assemblies))
                result = updateItemManager.RetrieveUpdateItems(assemblies, updateSchemaProvider);
            return result;
        }

        private void ServicesHelperInitialize()
        {
            if (ServicesHelper == null)
                ServicesHelper = new DefaultServicesHelper();
        }

        private bool ThereAreAssemblies(Assembly[] assemblies)
        {
            return (assemblies != null && assemblies.Length > 0);
        }

        private bool ThereAreTypes(Type[] types)
        {
            return (types != null && types.Length > 0);
        }

        private bool UpdateAllowed(IUpdateSchemaProvider updateSchemaProvider, UpdateItemInfo updateItem)
        {
            bool providerSupportsUpdateSchemaType = updateSchemaProvider.SupportsUpdateSchemaType(updateItem.UpdateSchemaType);
            bool allowThisUpdateType = UpdateSchemaTypeHelper.ContainsUpdateSchemaType(UpdateSchemaTypes, updateItem.UpdateSchemaType);
            bool ignoreIndex = UpdateSchemaIgnoreConditionsHelper.ContainsIgnoreCondition(updateItem.IgnoreConditions, UpdateSchemaIgnoreConditions.Index);
            bool columnHasIndex = updateSchemaProvider.ColumnHasIndex(updateItem.TableName, updateItem.ColumnName);
            bool ignoreForeignKey = UpdateSchemaIgnoreConditionsHelper.ContainsIgnoreCondition(updateItem.IgnoreConditions, UpdateSchemaIgnoreConditions.ForeignKey);
            bool columnHasForeignKey = updateSchemaProvider.ColumnHasForeignKeyRelationship(updateItem.TableName, updateItem.ColumnName);

            bool result = providerSupportsUpdateSchemaType
                && allowThisUpdateType
                && (ignoreIndex || !columnHasIndex)
                && (ignoreForeignKey || !columnHasForeignKey);
            if (!result)
            {
                if (!providerSupportsUpdateSchemaType)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("NoProviderSupportForUpdateType"), updateItem.ToString()));
                if (!allowThisUpdateType)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateTypeIsNotAllowed"), updateItem.ToString()));
                if (ignoreIndex && columnHasIndex)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("AttemptingToIgnoreIndex"), updateItem.ToString()), UpdateSchemaLogLevel.Warning);
                if (ignoreForeignKey && columnHasForeignKey)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("AttemptingToIgnoreForeignKey"), updateItem.ToString()), UpdateSchemaLogLevel.Warning);
            }
            return result;
        }

        private bool UpdateItems(UpdateSchemaOrder order, IUpdateSchemaProvider updateSchemaProvider)
        {
            bool result = true;
            foreach (UpdateItemInfo item in updateItems)
            {
                switch (item.UpdateSchemaType)
                {
                    case UpdateSchemaType.ShrinkColumnStringLength:
                    case UpdateSchemaType.GrowColumnStringLength:
                        if (!ModifyColumnStringLength(order, item, updateSchemaProvider) && AbortUpdateWithFirstError)
                            result = false;
                        break;
                    case UpdateSchemaType.DeleteColumn:
                        if (!DeleteColumn(order, item, updateSchemaProvider) && AbortUpdateWithFirstError)
                            result = false;
                        break;
                    case UpdateSchemaType.RenameColumn:
                        if (!RenameColumn(order, item, updateSchemaProvider) && AbortUpdateWithFirstError)
                            result = false;
                        break;
                }
                if (!result)
                    break;
            }
            if (!result)
                UpdateSchemaLogger.Log(ServicesHelper.GetString("UpdatesAborted"));
            return result;
        }

        private bool UpdateSchemaVersion()
        {
            bool result = true;
            if (String.IsNullOrEmpty(schemaVersion.ConnectionString) && schemaVersion.DataLayer == null)
                schemaVersion.ConnectionString = ConnectionString;
            if (schemaVersion.HasSchemaVersionBeenUpdated)
                UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateSchemaVersionNotNeeded"),
                    schemaVersion.RequiredSchemaVersion));
            else
            {
                result = schemaVersion.UpdateSchemaVersion();
                if (result)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateSchemaVersionSuccess"),
                        schemaVersion.RequiredSchemaVersion));
                else
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateSchemaVersionFailure"),
                        schemaVersion.RequiredSchemaVersion, schemaVersion.CurrentSchemaVersion),
                        UpdateSchemaLogLevel.Error);
            }
            return result;
        }

        private bool XpoUpdateSchema(Assembly[] assemblies, Type[] types)
        {
            UpdateSchemaLogger.Log(ServicesHelper.GetString("XpoUpdateSchemaStarted"));
            using (IDataLayer dataLayer = XpoDefault.GetDataLayer(ConnectionString, AutoCreateOption))
            {
                using (Session session = new Session(dataLayer))
                {
                    if (ThereAreAssemblies(assemblies))
                    {
                        session.UpdateSchema(assemblies);
                        session.CreateObjectTypeRecords(assemblies);
                    }
                    if (ThereAreTypes(types))
                    {
                        session.UpdateSchema(types);
                        session.CreateObjectTypeRecords(types);
                    }
                    session.Connection.Close();
                }
                dataLayer.Connection.Close();
            }
            UpdateSchemaLogger.Log(ServicesHelper.GetString("XpoUpdateSchemaComplete"));
            return true;
        }
    }
}
