#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Globalization;
    using DevExpress.Xpo.DB;
    using DevExpress.Xpo.DB.Helpers;
    using DevExpress.Xpo.Exceptions;
    using DevExpressContrib.Core;

    /// <summary>
    /// Helper class for update schema providers.
    /// </summary>
    public static class UpdateSchemaProviderHelper
    {

        /// <summary>
        /// Removes the XPO provider type parameter from the connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns>The connection string without the XPO provider type parameter.</returns>
        public static string RemoveXpoProviderTypeParameter(string connectionString)
        {
            GuardAgainst.ArgumentIsNullOrEmpty(connectionString, "connectionString");
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            connectionStringParser.RemovePartByName(DataStoreBase.XpoProviderTypeParameterName);
            return connectionStringParser.GetConnectionString();
        }

        /// <summary>
        /// Tries the get XPO provider type string from the connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="xpoProviderName">The returned XPO provider type string.</param>
        /// <returns><c>true</c> if we could get the XPO provider type string; otherwise <c>false</c>.</returns>
        public static bool TryGetXpoProvider(string connectionString, out string xpoProviderName)
        {
            GuardAgainst.ArgumentIsNullOrEmpty(connectionString, "connectionString");
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            xpoProviderName = connectionStringParser.GetPartByName(DataStoreBase.XpoProviderTypeParameterName);
            if (String.IsNullOrEmpty(xpoProviderName))
                xpoProviderName = null;

            if (!String.IsNullOrEmpty(xpoProviderName))
                return true;
            else
            {
                if (!String.IsNullOrEmpty(connectionStringParser.GetPartByName("Initial Catalog"))
                    && (String.IsNullOrEmpty(connectionStringParser.GetPartByName("Provider"))))
                {
                    xpoProviderName = MSSqlConnectionProvider.XpoProviderTypeString;
                    return true;
                }
                else
                {
                    string provider = connectionStringParser.GetPartByName("Provider").ToLower(CultureInfo.InvariantCulture);
                    if (provider.Contains("microsoft.ace.oledb") || provider.Contains("microsoft.jet.oledb"))
                    {
                        xpoProviderName = AccessConnectionProvider.XpoProviderTypeString;
                        return true;
                    }
                    else
                        throw new CannotFindAppropriateConnectionProviderException(connectionString);
                }
            }
        }
    }
}
