#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using DevExpressContrib.Core;

    /// <summary>
    /// Retrieves a list all of the providers that can be used for schema updates.
    /// </summary>
    public static class UpdateSchemaProviderRegister
    {

        /// <summary>
        /// Gets the available providers from the standard schema update assmblies.
        /// </summary>
        /// <param name="availableProviders">The list of available providers to be filled in.</param>
        public static void GetAvailableProviders(Dictionary<string, Type> availableProviders)
        {
            List<Assembly> additionalAssemblies = new List<Assembly>();
            GetAvailableProviders(availableProviders, additionalAssemblies);
        }

        /// <summary>
        /// Gets the available providers from the supplied assemblies as well as the 
        /// standard schema update assemblies.
        /// </summary>
        /// <param name="availableProviders">The list of available providers to be filled in.</param>
        /// <param name="additionalAssemblies">The additional assemblies to search for providers.</param>
        public static void GetAvailableProviders(Dictionary<string, Type> availableProviders, List<Assembly> additionalAssemblies)
        {
            GuardAgainst.ArgumentIsNull(availableProviders, "availableProviders");
            GuardAgainst.ArgumentIsNull(additionalAssemblies, "additionalAssemblies");

            const string UpdateSchemaProvidersAssemblyName = "DevExpressContrib.Xpo.UpdateSchema.Providers";
            List<Assembly> assembliesToCheck = new List<Assembly>(additionalAssemblies);
            assembliesToCheck.Add(Assembly.GetExecutingAssembly());
            Assembly schemaUpdateBonusProviderAssembly = GetAssemblyFromName(UpdateSchemaProvidersAssemblyName);
            if (schemaUpdateBonusProviderAssembly != null
                && !assembliesToCheck.Contains(schemaUpdateBonusProviderAssembly))
                assembliesToCheck.Add(schemaUpdateBonusProviderAssembly);

            foreach (Assembly assembly in assembliesToCheck)
            {
                Type[] types = assembly.GetTypes();
                foreach (Type type in types)
                {
                    object[] attributes = type.GetCustomAttributes(typeof(UpdateSchemaProviderAttribute), false);
                    if (attributes.Length > 0)
                    {
                        string xpoProviderTypeString = (attributes[0] as UpdateSchemaProviderAttribute).XpoProviderTypeString.ToUpper();
                        if (!availableProviders.ContainsKey(xpoProviderTypeString))
                            availableProviders.Add((attributes[0] as UpdateSchemaProviderAttribute).XpoProviderTypeString.ToUpper(), type);
                    }
                }
            }
        }

        private static Assembly GetAssemblyFromName(string assemblyName)
        {
            Assembly assemblyResult = null;
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if (assembly.FullName.StartsWith(assemblyName))
                {
                    assemblyResult = assembly;
                    break;
                }
            }
            return assemblyResult;
        }
    }
}
