#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

// Comment out the following define if you don't want the provider compiled into
// your application.
#define MSAccess

#if (MSAccess)
namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Data;
    using System.Data.OleDb;
    using System.Linq;
    using System.Reflection;
    using DevExpress.Xpo.DB;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// Implements the database specific functionality for the Microsoft Access database.
    /// </summary>
    [UpdateSchemaProvider(AccessConnectionProvider.XpoProviderTypeString)]
    public class AccessUpdateSchemaProvider : BaseUpdateSchemaProvider, IUpdateSchemaProvider
    {

        private OleDbConnection connection;


        /// <summary>
        /// Initializes a new instance of the <see cref="AccessUpdateSchemaProvider"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public AccessUpdateSchemaProvider(string connectionString, IServicesHelper servicesHelper)
            : base(servicesHelper)
        {
            ConnectionString = UpdateSchemaProviderHelper.RemoveXpoProviderTypeParameter(connectionString);
            connection = new OleDbConnection();
        }


        /// <summary>
        /// Gets the maximum size of the string before the provider has to change
        /// to a different data type.
        /// </summary>
        /// <value>The maximum size of the string.</value>
        public int MaximumStringSize
        {
            get { return MSSqlConnectionProvider.MaximumStringSize; }
        }

        /// <summary>
        /// Gets the xpo provider type string for this provider.
        /// </summary>
        /// <value>The xpo provider type string.</value>
        public string XpoProviderTypeString
        {
            get { return AccessConnectionProvider.XpoProviderTypeString; }
        }


        /// <summary>
        /// Determines if the supplied column exists within the supplied table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column exists; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnExists(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("Columns"))
                return ProviderLinqHelper.ColumnExists(tableName, columnName, schema);
        }

        /// <summary>
        /// Determines if the given column has a foreign key relationship. 
        /// ColumnHasForeignKeyRelationship checks for columns on both sides of the relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasForeignKeyRelationship(string tableName, string columnName)
        {
            DataTable data = null;
            try
            {
                foreach (DataRow table in TableNames().Rows)
                {
                    string name = table.Field<string>("TABLE_NAME");
                    DataTable schema = connection.GetOleDbSchemaTable(OleDbSchemaGuid.Foreign_Keys, new object[] { null, null, null, null, null, name });
                    if (data == null)
                        data = schema.Clone();
                    foreach (DataRow row in schema.Rows)
                        data.ImportRow(row);
                }
                //data.WriteXml(String.Format(@"C:\Temp\{0}.{1}.ForeignKey.xml", tableName, columnName), XmlWriteMode.WriteSchema);
                return (from result in data.AsEnumerable()
                        where (result.Field<string>("PK_TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                               && result.Field<string>("PK_COLUMN_NAME").ToUpper().Equals(columnName.ToUpper()))
                               || (result.Field<string>("FK_TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                               && result.Field<string>("FK_COLUMN_NAME").ToUpper().Equals(columnName.ToUpper()))
                        select result).Count() >= 1;
            }
            finally
            {
                if (data != null)
                    data.Dispose();
            }
        }

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has an index; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasIndex(string tableName, string columnName)
        {
            using (DataTable data = connection.GetOleDbSchemaTable(OleDbSchemaGuid.Indexes, new object[] { null, null, null, null, tableName }))
            {
                //data.WriteXml(String.Format(@"C:\Temp\{0}.{1}.Indexes.xml", tableName, columnName), XmlWriteMode.WriteSchema);
                return (from result in data.AsEnumerable()
                        where result.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                               && result.Field<string>("COLUMN_NAME").ToUpper().Equals(columnName.ToUpper())
                        select result).Count() >= 1;
            }
        }

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column holds a string value; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnIsString(string tableName, string columnName)
        {
            /* When using connection.GetSchema("Columns") Access returns the datatype as an int.
             * So use the OleDbType to figure out if this column is a string column or not. */
            OleDbType dataType = GetColumnDataType(tableName, columnName);
            if (dataType == OleDbType.VarWChar
                || dataType == OleDbType.LongVarWChar
                || dataType == OleDbType.WChar
                || dataType == OleDbType.Char)
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <returns>
        /// 	<c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize)
        {
            using (DataTable data = FillDataTable(String.Format("SELECT {0} from {1}", columnName, tableName)))
                return ProviderLinqHelper.ColumnStringValuesWillBeTruncated(tableName, columnName, newSize, data);
        }

        /// <summary>
        /// Determines if the databases exists. Uses the ConnectionString to obtain
        /// information about the database.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the database exists; otherwise <c>false</c>.
        /// </returns>
        public bool DatabaseExists()
        {
            return FileBasedProviderHelper.DatabaseExists(ConnectionString);
        }

        /// <summary>
        /// Deletes the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            bool result = true;
            string tableName = updateItem.TableName;
            string columnNameToDelete = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate)
            {
                result = DeleteColumn(tableName, columnNameToDelete);
                if (UpdateSchemaLogger != null)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
            }
            return result;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            connection.Close();
            connection.Dispose();
        }

        /// <summary>
        /// Attempts to drop the database.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns><c>true</c> if the database was dropped; otherwise <c>false</c>.</returns>
        public static bool DropDatabase(string connectionString)
        {
            return FileBasedProviderHelper.DropDatabase(connectionString);
        }

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// The number of characters that the column can hold.
        /// </returns>
        public int GetStringColumnSize(string tableName, string columnName)
        {
            if (ColumnIsString(tableName, columnName))
            {
                using (DataTable schema = connection.GetSchema("Columns"))
                    return ProviderLinqHelper.GetStringColumnSize(tableName, columnName, schema);
            }
            return 0;
        }

        /// <summary>
        /// Modifies the size of the string column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool ModifyStringColumnSize(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnName = updateItem.ColumnName;
            int newSize = updateItem.Size;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && GetStringColumnSize(tableName, columnName) != newSize)
            {
                string columnDefinition;
                if (newSize > 0 && newSize <= AccessConnectionProvider.MaximumStringSize)
                    columnDefinition = String.Format("TEXT({0})", newSize);
                else
                    columnDefinition = "MEMO";
                ExecuteNonQuery(String.Format("ALTER TABLE {0} ALTER COLUMN {1} {2}",
                    tableName, columnName, columnDefinition));
                if (UpdateSchemaLogger != null)
                    UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
            }
            return true;
        }

        /// <summary>
        /// Opens this instance of the update schema provider.
        /// </summary>
        public void Open()
        {
            connection.ConnectionString = ConnectionString;
            connection.Open();
        }

        /// <summary>
        /// Rename the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string oldColumnName = updateItem.OldColumnName;
            string newColumnName = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.BeforeXpoSchemaUpdate
                && ColumnExists(tableName, oldColumnName)
                && !ColumnExists(tableName, newColumnName))
            {
                Type connectionType = Type.GetTypeFromProgID("ADODB.Connection");
                object connection = Activator.CreateInstance(connectionType);
                try
                {
                    SetProperty(connection, "ConnectionString", ConnectionString);
                    InvokeMethod(connection, "Open", new object[] { ConnectionString, null, null, 0 });
                    Type catalogType = Type.GetTypeFromProgID("ADOX.Catalog");
                    object catalog = Activator.CreateInstance(catalogType);
                    try
                    {
                        SetProperty(catalog, "ActiveConnection", connection);
                        object tables = GetProperty(catalog, "Tables");
                        if (tables != null)
                        {
                            object table = GetCollectionItemFromName(tables, tableName);
                            if (table != null)
                            {
                                object columns = GetProperty(table, "Columns");
                                if (columns != null)
                                {
                                    object column = GetCollectionItemFromName(columns, oldColumnName);
                                    if (column != null)
                                    {
                                        SetProperty(column, "Name", newColumnName);
                                        InvokeMethod(tables, "Refresh", null);
                                    }
                                }
                            }
                        }
                        InvokeMethod(connection, "Close", null);
                        if (UpdateSchemaLogger != null)
                            UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                    }
                    finally
                    {
                        catalog = null;
                    }
                }
                finally
                {
                    connection = null;
                }
            }
            return true;
        }

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <returns>
        /// 	<c>true</c> if the table exists; otherwise <c>false</c>.
        /// </returns>
        public bool TableExists(string tableName)
        {
            using (DataTable schema = connection.GetSchema("Tables"))
                return ProviderLinqHelper.TableExists(tableName, schema);
        }

        private bool DeleteColumn(string tableName, string columnNameToDelete)
        {
            if (ColumnExists(tableName, columnNameToDelete))
            {
                ExecuteNonQuery(String.Format("ALTER TABLE [{0}] DROP COLUMN [{1}]",
                    tableName, columnNameToDelete));
            }
            return true;
        }

        private void ExecuteNonQuery(string sql)
        {
            using (OleDbCommand cmd = new OleDbCommand(sql, connection))
            {
                cmd.ExecuteNonQuery();
            }
        }

        private DataTable FillDataTable(string sql)
        {
            DataTable dataTable = new DataTable("Data");
            using (OleDbCommand cmd = new OleDbCommand(sql, connection))
            {
                using (OleDbDataAdapter adapter = new OleDbDataAdapter(cmd))
                    adapter.Fill(dataTable);
            }
            return dataTable;
        }

        private object GetCollectionItemFromName(object collection, string targetName)
        {
            int count = Convert.ToInt32(GetProperty(collection, "Count"));
            for (int i = 0; i < count; i++)
            {
                object item = GetProperty(collection, "Item", i);
                if (item != null)
                {
                    object name = GetProperty(item, "Name");
                    if (name != null && String.Compare(name.ToString(), targetName, true) == 0)
                        return item;
                }
            }
            return null;
        }

        private OleDbType GetColumnDataType(string tableName, string columnName)
        {
            using (DataTable schemaTable = connection.GetOleDbSchemaTable(
                OleDbSchemaGuid.Columns, new object[] { null, null, tableName, columnName }))
            {
                if (schemaTable.Rows.Count == 1)
                {
                    DataRow row = schemaTable.Rows[0];
                    return (OleDbType)row["DATA_TYPE"];
                }
            }
            return OleDbType.Empty;
        }

        private object GetProperty(object target, string propertyName)
        {
            return target.GetType().InvokeMember(propertyName, BindingFlags.GetProperty, null, target, null);
        }

        private object GetProperty(object target, string propertyName, object value)
        {
            object[] oParam = new object[] { value };
            return target.GetType().InvokeMember(propertyName, BindingFlags.GetProperty, null, target, oParam);
        }

        private object GetProperty(object target, string propertyName, object value1, object value2)
        {
            return target.GetType().InvokeMember(propertyName, BindingFlags.GetProperty, null, target, new object[] { value1, value2 });
        }

        private object InvokeMethod(object target, string method, object[] parameters)
        {
            return target.GetType().InvokeMember(method, BindingFlags.InvokeMethod, null, target, parameters);
        }

        private void SetProperty(object target, string propertyName, object value)
        {
            object[] oParam = new object[] { value };
            target.GetType().InvokeMember(propertyName, BindingFlags.SetProperty, null, target, oParam);
        }

        private DataTable TableNames()
        {
            DataTable result = connection.GetSchema("tables", new string[] { null, null, null, "TABLE" });
            //result.WriteXml(@"C:\Temp\tables.xml", XmlWriteMode.WriteSchema);
            return result;
        }
    }
}
#endif // (MSAccess)
