#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System.IO;
    using DevExpress.Xpo.DB.Helpers;

    /// <summary>
    /// Helper functions for schema update providers that are based on a database
    /// file. For example Micrsoft Access and VistaDB 3 are both file based databases.
    /// </summary>
    public class FileBasedProviderHelper
    {

        private const string defaultDataSourcePartKey = "Data Source";


        /// <summary>
        /// Determines whether the databases file exists. This will use the default data source
        /// connection string key which is "Data Source".
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns><c>true</c> if the database file exists; otherwise <c>false</c>.</returns>
        public static bool DatabaseExists(string connectionString)
        {
            return DatabaseExists(connectionString, defaultDataSourcePartKey);
        }

        /// <summary>
        /// Determines whether the databases file exists.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="dataSourcePartKey">The data source connection string key.</param>
        /// <returns><c>true</c> if the database file exists; otherwise <c>false</c>.</returns>
        public static bool DatabaseExists(string connectionString, string dataSourcePartKey)
        {
            return File.Exists(GetDatabaseName(connectionString, dataSourcePartKey));
        }

        /// <summary>
        /// Deletes the database file. This will use the default data source
        /// connection string key which is "Data Source".
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns></returns>
        public static bool DropDatabase(string connectionString)
        {
            return DropDatabase(connectionString, defaultDataSourcePartKey);
        }

        /// <summary>
        /// Deletes the database file.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="dataSourcePartKey">The data source connection string key.</param>
        /// <returns><c>true</c> if the file is deleted; otherwise <c>false</c>.</returns>
        public static bool DropDatabase(string connectionString, string dataSourcePartKey)
        {
            string databaseName = GetDatabaseName(connectionString, dataSourcePartKey);
            if (DatabaseExists(connectionString, dataSourcePartKey))
                File.Delete(databaseName);
            return !File.Exists(databaseName);
        }

        /// <summary>
        /// Gets the name of the database file. This will use the default data source
        /// connection string key which is "Data Source".
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns></returns>
        public static string GetDatabaseName(string connectionString)
        {
            return GetDatabaseName(connectionString, defaultDataSourcePartKey);
        }

        /// <summary>
        /// Gets the name of the database file.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="dataSourcePartKey">The text of the data source connection string key.</param>
        /// <returns></returns>
        public static string GetDatabaseName(string connectionString, string dataSourcePartKey)
        {
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            return connectionStringParser.GetPartByName(dataSourcePartKey);
        }
    }
}
