#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// <para>Defines the interface to which all UpdateSchemaProviders must conform.</para>
    /// <para>Providers may also implement a static method to drop the database. This
    /// method should be implemented as: <c>public static bool DropDatabase(string connectionString)</c>.
    /// This method is used during testing.</para>
    /// </summary>
    public interface IUpdateSchemaProvider : IDisposable
    {

        /// <summary>
        /// Gets the connection string.
        /// </summary>
        /// <value>The connection string.</value>
        string ConnectionString { get; }

        /// <summary>
        /// Gets the maximum size of the string before the provider has to change
        /// to a different data type.
        /// </summary>
        /// <value>The maximum size of the string.</value>
        int MaximumStringSize { get; }

        /// <summary>
        /// Gets or sets the information logger.
        /// </summary>
        /// <value>The logger.</value>
        UpdateSchemaLogger UpdateSchemaLogger { get; set; }

        /// <summary>
        /// Gets the XPO provider type string for this provider.
        /// </summary>
        /// <value>The XPO provider type string.</value>
        string XpoProviderTypeString { get; }


        /// <summary>
        /// Determines if the supplied column exists within the given table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns><c>true</c> if the column exists; otherwise <c>false</c>.</returns>
        bool ColumnExists(string tableName, string columnName);

        /// <summary>
        /// Determines if the given column has a foreign key relationship. 
        /// ColumnHasForeignKeyRelationship should check for columns on both sides of the relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns><c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.</returns>
        bool ColumnHasForeignKeyRelationship(string tableName, string columnName);

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns><c>true</c> if the column has an index; otherwise <c>false</c>.</returns>
        bool ColumnHasIndex(string tableName, string columnName);

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns><c>true</c> if the column holds a string value; otherwise <c>false</c>.</returns>
        bool ColumnIsString(string tableName, string columnName);

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <returns><c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.</returns>
        bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize);

        /// <summary>
        /// Determines if the databases exists. Uses the <see cref="ConnectionString"/> to obtain
        /// information about the database.
        /// </summary>
        /// <returns><c>true</c> if the database exists; otherwise <c>false</c>.</returns>
        bool DatabaseExists();

        /// <summary>
        /// Deletes the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="itemInfo">The update item info.</param>
        /// <returns>
        /// 	<c>true</c> if the something was accomplished for the given UpdateSchemaOrder; otherwise <c>false</c>.
        /// </returns>
        bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo itemInfo);

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>The number of characters that the column can hold.</returns>
        int GetStringColumnSize(string tableName, string columnName);

        /// <summary>
        /// Modifies the size of the string column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="itemInfo">The update item info.</param>
        /// <returns>
        /// 	<c>true</c> if the something was accomplished for the given UpdateSchemaOrder; otherwise <c>false</c>.
        /// </returns>
        bool ModifyStringColumnSize(UpdateSchemaOrder order, UpdateItemInfo itemInfo);

        /// <summary>
        /// Opens this instance of the update schema provider.
        /// </summary>
        void Open();

        /// <summary>
        /// Rename the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="itemInfo">The update item info.</param>
        /// <returns>
        /// 	<c>true</c> if the something was accomplished for the given UpdateSchemaOrder; otherwise <c>false</c>.
        /// </returns>
        bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo itemInfo);

        /// <summary>
        /// Determine whether the provider supports the supplied type of schema update.
        /// </summary>
        /// <param name="updateSchemaType">Type of update schema.</param>
        /// <returns><c>true</c> if the provider supports the supplied type of schema update; otherwise <c>false</c>.</returns>
        bool SupportsUpdateSchemaType(UpdateSchemaType updateSchemaType);

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <returns><c>true</c> if the table exists; otherwise <c>false</c>.</returns>
        bool TableExists(string tableName);
    }
}
