#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

// Comment out the following define if you don't want the provider compiled into
// your application.
#define MSSqlServer

#if (MSSqlServer)
namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Data;
    using System.Data.SqlClient;
    using DevExpress.Xpo.DB;
    using DevExpress.Xpo.DB.Helpers;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;

    /// <summary>
    /// Implements the database specific functionality for the Microsoft Access database.
    /// </summary>
    [UpdateSchemaProvider(MSSqlConnectionProvider.XpoProviderTypeString)]
    public class MSSqlUpdateSchemaProvider : BaseUpdateSchemaProvider, IUpdateSchemaProvider
    {

        private SqlConnection connection;
        private const string initialCatalog = "initial catalog";


        /// <summary>
        /// Initializes a new instance of the <see cref="MSSqlUpdateSchemaProvider"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public MSSqlUpdateSchemaProvider(string connectionString, IServicesHelper servicesHelper)
            : base(servicesHelper)
        {
            ConnectionString = UpdateSchemaProviderHelper.RemoveXpoProviderTypeParameter(connectionString);
            connection = new SqlConnection();
        }


        /// <summary>
        /// Gets the maximum size of the string before the provider has to change
        /// to a different data type.
        /// </summary>
        /// <value>The maximum size of the string.</value>
        public int MaximumStringSize
        {
            get { return MSSqlConnectionProvider.MaximumStringSize; }
        }

        /// <summary>
        /// Gets the XPO provider type string for this provider.
        /// </summary>
        /// <value>The XPO provider type string.</value>
        public string XpoProviderTypeString
        {
            get { return MSSqlConnectionProvider.XpoProviderTypeString; }
        }


        /// <summary>
        /// Determines if the supplied column exists within the supplied table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column exists; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnExists(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("Columns"))
                return ProviderLinqHelper.ColumnExists(tableName, columnName, schema);
        }

        /// <summary>
        /// Determines if the given column has a foreign key relationship. 
        /// ColumnHasForeignKeyRelationship checks for columns on both sides of the relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasForeignKeyRelationship(string tableName, string columnName)
        {
            DataTable result = null;
            try
            {
                foreach (DataRow table in TableNames().Rows)
                {
                    string name = table.Field<string>("Name");
                    using (DataTable schema = FillDataTable(String.Format("EXECUTE sp_fkeys @fktable_name='{0}'", name)))
                    {
                        if (result == null)
                            result = schema.Clone();
                        foreach (DataRow row in schema.Rows)
                            result.ImportRow(row);
                    }
                }
                //result.WriteXml(@"C:\Temp\FK.xml", XmlWriteMode.WriteSchema);
                return ProviderLinqHelper.ColumnHasForeignKeyRelationship(tableName, columnName, result);
            }
            finally
            {
                if (result != null)
                    result.Dispose();
            }
        }

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has an index; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasIndex(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("IndexColumns"))
                return ProviderLinqHelper.ColumnHasIndex(tableName, columnName, schema);
        }

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column holds a string value; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnIsString(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("Columns"))
                return ProviderLinqHelper.ColumnIsString(tableName, columnName, schema,
                    new string[] { "nvarchar", "varchar" });
        }

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <returns>
        /// <c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize)
        {
            using (DataTable data = FillDataTable(String.Format("SELECT {0} from {1}", columnName, tableName)))
                return ProviderLinqHelper.ColumnStringValuesWillBeTruncated(tableName, columnName, newSize, data);
        }

        /// <summary>
        /// Determines if the databases exists. Uses the ConnectionString to obtain
        /// information about the database.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the database exists; otherwise <c>false</c>.
        /// </returns>
        public bool DatabaseExists()
        {
            return DatabaseExists(ConnectionString);
        }

        /// <summary>
        /// Deletes the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnNameToDelete = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && ColumnExists(tableName, columnNameToDelete))
            {
                string sql = String.Format("ALTER TABLE [{0}] DROP COLUMN [{1}]",
                    tableName, columnNameToDelete);
                using (SqlCommand cmd = new SqlCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            connection.Close();
            connection.Dispose();
        }

        /// <summary>
        /// Attempts to drop the database.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns><c>true</c> if the database was dropped; otherwise <c>false</c>.</returns>
        public static bool DropDatabase(string connectionString)
        {
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            string databaseName = connectionStringParser.GetPartByName(initialCatalog);
            if (DatabaseExists(connectionString))
            {
                string masterConnectionString = GetMasterConnectionString(connectionString);
                using (SqlConnection tempConnection = new SqlConnection(masterConnectionString))
                {
                    tempConnection.Open();
                    string sql = String.Format("DROP DATABASE [{0}]", databaseName);
                    using (SqlCommand cmd = new SqlCommand(sql, tempConnection))
                    {
                        SqlConnection.ClearAllPools();
                        cmd.ExecuteNonQuery();
                    }
                    tempConnection.Close();
                }
            }
            return !DatabaseExists(connectionString);
        }

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// The number of characters that the column can hold.
        /// </returns>
        public int GetStringColumnSize(string tableName, string columnName)
        {
            if (ColumnIsString(tableName, columnName))
            {
                using (DataTable schema = connection.GetSchema("Columns"))
                    return ProviderLinqHelper.GetStringColumnSize(tableName, columnName, schema);
            }
            return 0;
        }

        /// <summary>
        /// Modifies the size of the string column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool ModifyStringColumnSize(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnName = updateItem.ColumnName;
            int newSize = updateItem.Size;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && GetStringColumnSize(tableName, columnName) != newSize)
            {
                string sql;
                const string sqlFormat = "ALTER TABLE {0} ALTER COLUMN {1} nvarchar({2})";
                if (newSize > 0 && newSize <= MSSqlConnectionProvider.MaximumStringSize)
                    sql = String.Format(sqlFormat, tableName, columnName, newSize);
                else
                    sql = String.Format(sqlFormat, tableName, columnName, "max");
                using (SqlCommand cmd = new SqlCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Opens this instance of the update schema provider.
        /// </summary>
        public void Open()
        {
            connection.ConnectionString = ConnectionString;
            connection.Open();
        }

        /// <summary>
        /// Rename the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string oldColumnName = updateItem.OldColumnName;
            string newColumnName = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.BeforeXpoSchemaUpdate
                && ColumnExists(tableName, oldColumnName)
                && !ColumnExists(tableName, newColumnName))
            {
                string sql = String.Format("EXEC sp_rename @objname = '{0}.{1}', @newname = '{2}', @objtype = 'COLUMN'",
                    tableName, oldColumnName, newColumnName);
                using (SqlCommand cmd = new SqlCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <returns>
        /// 	<c>true</c> if the table exists; otherwise <c>false</c>.
        /// </returns>
        public bool TableExists(string tableName)
        {
            using (DataTable schema = connection.GetSchema("Tables"))
                return ProviderLinqHelper.TableExists(tableName, schema);
        }

        private static bool DatabaseExists(string connectionString)
        {
            string masterConnectionString = GetMasterConnectionString(connectionString);
            using (SqlConnection tempConnection = new SqlConnection(masterConnectionString))
            {
                try
                {
                    tempConnection.Open();
                    int count;
                    ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
                    string databaseName = connectionStringParser.GetPartByName(initialCatalog);
                    string sql = String.Format("SELECT Count(*) FROM sys.databases WHERE [name]='{0}'", databaseName);
                    using (SqlCommand cmd = new SqlCommand(sql, tempConnection))
                        count = (int)cmd.ExecuteScalar();
                    return (count > 0);
                }
                catch (SqlException)
                {
                    return false;
                }
                finally
                {
                    tempConnection.Close();
                }
            }
        }

        private DataTable FillDataTable(string sql)
        {
            DataTable dataTable = new DataTable("Data");
            using (SqlCommand cmd = new SqlCommand(sql, connection))
            {
                using (SqlDataAdapter adapter = new SqlDataAdapter(cmd))
                    adapter.Fill(dataTable);
            }
            return dataTable;
        }

        private static string GetMasterConnectionString(string connectionString)
        {
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            connectionStringParser.RemovePartByName(DataStoreBase.XpoProviderTypeParameterName);
            connectionStringParser.RemovePartByName(initialCatalog);
            return String.Format("{0}=master;{1}", initialCatalog, connectionStringParser.GetConnectionString());
        }

        private DataTable TableNames()
        {
            return FillDataTable("SELECT TABLE_NAME AS Name FROM INFORMATION_SCHEMA.TABLES WHERE table_type = 'BASE TABLE'");
        }
    }
}
#endif // (MSSqlServer)
