#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Linq;

    /// <summary>
    /// Update schema provider helper class which uses Linq to determine database
    /// properties.
    /// </summary>
    public static class ProviderLinqHelper
    {

        /// <summary>
        /// Determines if the supplied column exists within the supplied table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="schema">The schema data table.</param>
        /// <returns>
        /// 	<c>true</c> if the column exists; otherwise <c>false</c>.
        /// </returns>
        public static bool ColumnExists(string tableName, string columnName, DataTable schema)
        {
            IEnumerable<DataRow> columnsDataTable = schema.AsEnumerable();
            return (from data in columnsDataTable
                    where data.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                           && data.Field<string>("COLUMN_NAME").ToUpper().Equals(columnName.ToUpper())
                    select data).Count() == 1;
        }

        /// <summary>
        /// Determines if the given column has a foreign key relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="schema">The DataTable holding the foreign key relationship schema.</param>
        /// <returns>
        /// 	<c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.
        /// </returns>
        public static bool ColumnHasForeignKeyRelationship(string tableName, string columnName, DataTable schema)
        {
            IEnumerable<DataRow> foreignKeyDataTable = schema.AsEnumerable();
            return (from data in foreignKeyDataTable
                    where (data.Field<string>("PKTABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                                && data.Field<string>("PKCOLUMN_NAME").ToUpper().Equals(columnName.ToUpper()))
                           || (data.Field<string>("FKTABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                                && data.Field<string>("FKCOLUMN_NAME").ToUpper().Equals(columnName.ToUpper()))
                    select data).Count() == 1;
        }

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="schema">The DataTable holding the IndexColumns schema.</param>
        /// <returns>
        /// 	<c>true</c> if the column has an index; otherwise <c>false</c>.
        /// </returns>
        public static bool ColumnHasIndex(string tableName, string columnName, DataTable schema)
        {
            IEnumerable<DataRow> indexColumnsDataTable = schema.AsEnumerable();
            return (from data in indexColumnsDataTable
                    where data.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                           && data.Field<string>("COLUMN_NAME").ToUpper().Equals(columnName.ToUpper())
                    select data).Count() == 1;
        }

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="schema">The schema data table.</param>
        /// <param name="validDataTypes">The valid data types.</param>
        /// <returns>
        /// 	<c>true</c> if the column holds a string value; otherwise <c>false</c>.
        /// </returns>
        public static bool ColumnIsString(string tableName, string columnName, DataTable schema, string[] validDataTypes)
        {
            for (int i = 0; i < validDataTypes.Length; i++)
                validDataTypes[i] = validDataTypes[i].ToUpper();

            IEnumerable<DataRow> columnsDataTable = schema.AsEnumerable();
            return (from data in columnsDataTable
                    where data.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                           && data.Field<string>("COLUMN_NAME").ToUpper().Equals(columnName.ToUpper())
                           && validDataTypes.ToList<string>().Contains(data.Field<string>("DATA_TYPE").ToUpper())
                    select data).Count() == 1;
        }

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <param name="dataTable">The data table containing the columns to check.</param>
        /// <returns>
        /// 	<c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.
        /// </returns>
        public static bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize, DataTable dataTable)
        {
            IEnumerable<DataRow> enumerableDataTable = dataTable.AsEnumerable();
            return (from results in enumerableDataTable
                    where results.Field<string>(columnName).Length > newSize
                    select results).Count() > 0;
        }

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="schema">The schema data table.</param>
        /// <returns>
        /// The number of characters that the column can hold.
        /// </returns>
        public static int GetStringColumnSize(string tableName, string columnName, DataTable schema)
        {
            /* Here select via the non-type safe DataRow indexer to obtain the max length.
             * This is done because in Microsoft Access this field is a long whereas in
             * Microsoft SQL Server this is an int. So use the DataRow indexer and cast to an Int32. */
            IEnumerable<DataRow> columnsDataTable = schema.AsEnumerable();
            return (from data in columnsDataTable
                    where data.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                           && data.Field<string>("COLUMN_NAME").ToUpper().Equals(columnName.ToUpper())
                    select Convert.ToInt32(data["CHARACTER_MAXIMUM_LENGTH"])).First();
        }

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="schema">The schema data table.</param>
        /// <returns>
        /// 	<c>true</c> if the table exists; otherwise <c>false</c>.
        /// </returns>
        public static bool TableExists(string tableName, DataTable schema)
        {
            IEnumerable<DataRow> columnsDataTable = schema.AsEnumerable();
            return (from data in columnsDataTable
                    where data.Field<string>("TABLE_NAME").ToUpper().Equals(tableName.ToUpper())
                    select data).Count() == 1;
        }
    }
}
