#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

// Comment out the following define if you don't want the provider compiled into
// your application.
//#define ASA

#if (ASA)
namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Data;
    using System.Linq;
    using DevExpress.Xpo.DB;
    using DevExpress.Xpo.DB.Helpers;
    using DevExpressContrib.Core.Interface;
    using iAnywhere.Data.AsaClient;

    /// <summary>
    /// Implements the database specific functionality for the Sybase Adaptive Server Anywhere.
    /// </summary>
    [UpdateSchemaProvider(AsaConnectionProvider.XpoProviderTypeString)]
    public class AsaUpdateSchemaProvider : BaseUpdateSchemaProvider, IUpdateSchemaProvider
    {

        private AsaConnection connection;


        /// <summary>
        /// Initializes a new instance of the <see cref="AccessUpdateSchemaProvider"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public AsaUpdateSchemaProvider(string connectionString, IServicesHelper servicesHelper)
            : base(servicesHelper)
        {
            ConnectionString = UpdateSchemaProviderHelper.RemoveXpoProviderTypeParameter(connectionString);
            connection = new AsaConnection();
        }


        /// <summary>
        /// Gets the maximum size of the string before the provider has to change
        /// to a different data type.
        /// </summary>
        /// <value>The maximum size of the string.</value>
        public int MaximumStringSize
        {
            get { return AsaConnectionProvider.MaximumStringSize; }
        }

        /// <summary>
        /// Gets the xpo provider type string for this provider.
        /// </summary>
        /// <value>The xpo provider type string.</value>
        public string XpoProviderTypeString
        {
            get { return AsaConnectionProvider.XpoProviderTypeString; }
        }


        /// <summary>
        /// Determines if the supplied column exists within the given table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column exists; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnExists(string tableName, string columnName)
        {
            string sql = String.Format("SELECT cname FROM SYS.SYSCOLUMNS WHERE tname='{0}' AND cname='{1}'", tableName, columnName);
            using (AsaCommand command = new AsaCommand(sql, connection))
            {
                string result = (string)command.ExecuteScalar();
                if (result == null)
                    return false;
                return true;
            }
        }

        /// <summary>
        /// Determines if the given column has a foreign key relationship.
        /// ColumnHasForeignKeyRelationship should check for columns on both sides of the relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasForeignKeyRelationship(string tableName, string columnName)
        {
            string sql = String.Format("SELECT pt.table_name AS PrimaryTable, pc.column_name AS PrimaryColumn,"
                + " ft.table_name AS ForeignTable, fc.column_name AS ForeignColumn"
                + " FROM sysfkcol c"
                + " JOIN sysforeignkey f on (f.foreign_key_id = c.foreign_key_id) AND (f.foreign_table_id = c.foreign_table_id)"
                + " JOIN systable ft on (ft.table_id = f.foreign_table_id)"
                + " JOIN systable pt on (pt.table_id = f.primary_table_id)"
                + " JOIN syscolumn fc on (c.foreign_column_id = fc.column_id) AND (f.foreign_table_id = fc.table_id)"
                + " JOIN syscolumn pc on (c.primary_column_id = pc.column_id) AND (f.primary_table_id = pc.table_id)"
                + " WHERE UPPER(ft.table_name) = '{0}' AND ((UPPER(PrimaryColumn) = '{1}') OR (UPPER(ForeignColumn) = '{1}'))"
                + " UNION"
                + " SELECT tname, cname, null, null"
                + " FROM Sys.sysColumns"
                + " WHERE "
                + " UPPER(tname)='{0}' AND UPPER(cname)='{1}' AND in_primary_key='Y'",
                tableName.ToUpper(), columnName.ToUpper());

            int count;
            using (DataTable indexes = FillDataTable(sql))
            {
                count = indexes.Rows.Count;
            }
            return count > 0;
        }

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has an index; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasIndex(string tableName, string columnName)
        {
            string sql = String.Format("SELECT t.table_name, c.column_name"
                + " FROM sysixcol i"
                + " JOIN systable t on (t.table_id = i.table_id)"
                + " JOIN syscolumn c on (i.column_id = c.column_id) AND (i.table_id = c.table_id)"
                + " WHERE UPPER(t.table_name) = '{0}' AND UPPER(c.column_name)='{1}'"
                + " UNION"
                + " select tname, cname"
                + " FROM Sys.sysColumns"
                + " WHERE UPPER(tname)='{0}' AND UPPER(cname)='{1}' AND in_primary_key='Y'",
                tableName.ToUpper(), columnName.ToUpper());
            int count;
            using (DataTable indexes = FillDataTable(sql))
            {
                count = indexes.Rows.Count;
            }
            return count > 0;
        }

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column holds a string value; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnIsString(string tableName, string columnName)
        {
            string[] validDataTypes = new string[] { "VARCHAR", "TEXT", "LONG VARCHAR" };
            string sql = String.Format("SELECT coltype FROM SYS.SYSCOLUMNS WHERE tname = '{0}' AND cname = '{1}'",
                tableName, columnName);
            using (AsaCommand command = new AsaCommand(sql, connection))
            {
                string dataType = (string)command.ExecuteScalar();
                if (dataType == null)
                    return false;
                return validDataTypes.ToList<string>().Contains(dataType.ToUpper());
            }
        }

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <returns>
        /// 	<c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize)
        {
            using (DataTable data = FillDataTable(String.Format("SELECT {0} from {1}", columnName, tableName)))
            {
                return ProviderLinqHelper.ColumnStringValuesWillBeTruncated(tableName, columnName, newSize, data);
            }
        }

        /// <summary>
        /// Determines if the databases exists. Uses the ConnectionString to obtain
        /// information about the database.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the database exists; otherwise <c>false</c>.
        /// </returns>
        public bool DatabaseExists()
        {
            return DatabaseExists(ConnectionString);
        }

        /// <summary>
        /// Determines if the supplied database exists.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns>
        /// 	<c>true</c> if the database exists; otherwise <c>false</c>.
        /// </returns>
        public static bool DatabaseExists(string connectionString)
        {
            using (AsaConnection asaConnection = new AsaConnection(connectionString))
            {
                try
                {
                    asaConnection.Open();
                    string sql = "SELECT table_name FROM SYS.SYSTABLE WHERE TABLE_TYPE = 'BASE' AND table_name = 'XPObjectType'";
                    using (AsaCommand command = new AsaCommand(sql, asaConnection))
                    {
                        string tableName = (string)command.ExecuteScalar();
                        return !String.IsNullOrEmpty(tableName);
                    }
                }
                catch (AsaException)
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Deletes the column.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnNameToDelete = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && ColumnExists(tableName, columnNameToDelete))
            {
                string sql = String.Format("ALTER TABLE {0} DROP {1}",
                    tableName, columnNameToDelete);
                using (AsaCommand cmd = new AsaCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            connection.Close();
            connection.Dispose();
        }

        /// <summary>
        /// Attempts to drop the database. For ASA this means that we clear out the database
        /// and not actually drop it.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns><c>true</c> if the database was cleared out; otherwise <c>false</c>.</returns>
        public static bool DropDatabase(string connectionString)
        {
            ConnectionStringParser connectionStringParser = new ConnectionStringParser(connectionString);
            connectionStringParser.RemovePartByName(DataStoreBase.XpoProviderTypeParameterName);
            connectionString = connectionStringParser.GetConnectionString();

            if (!DatabaseExists(connectionString))
                return true;
            using (AsaConnection asaConnection = new AsaConnection())
            {
                asaConnection.ConnectionString = connectionString;
                asaConnection.Open();
                using (AsaCommand cmd = new AsaCommand("SELECT t.table_name FROM SYS.SYSTABLE t JOIN SYS.SYSUSERPERM u"
                    + " ON t.creator=u.user_id WHERE t.TABLE_TYPE = 'BASE' AND u.user_name NOT IN ('sys', 'dbo')"
                    + " AND u.user_name NOT LIKE 'rs_sys%'", asaConnection))
                {
                    using (AsaDataAdapter adapter = new AsaDataAdapter(cmd))
                    {
                        using (DataTable tables = new DataTable("TableNames"))
                        {
                            adapter.Fill(tables);
                            foreach (DataRow row in tables.Rows)
                            {
                                string sql = "DROP TABLE " + row.Field<string>("table_name");
                                using (AsaCommand command = new AsaCommand(sql, asaConnection))
                                {
                                    command.ExecuteNonQuery();
                                }
                            }
                        }
                    }
                }
            }
            return !DatabaseExists(connectionString);
        }

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// The number of characters that the column can hold.
        /// </returns>
        public int GetStringColumnSize(string tableName, string columnName)
        {
            int length = 0;
            if (ColumnIsString(tableName, columnName))
            {
                string sql = String.Format("SELECT length FROM SYS.SYSCOLUMNS WHERE tname = '{0}' AND cname = '{1}'",
                    tableName, columnName);
                using (AsaCommand command = new AsaCommand(sql, connection))
                {
                    length = (Int16)command.ExecuteScalar();
                }
            }
            return length;
        }

        /// <summary>
        /// Modifies the size of the string column.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool ModifyStringColumnSize(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnName = updateItem.ColumnName;
            int newSize = updateItem.Size;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && GetStringColumnSize(tableName, columnName) != newSize)
            {
                string sql;
                if (newSize > 0 && newSize <= MSSqlConnectionProvider.MaximumStringSize)
                    sql = String.Format("ALTER TABLE {0} MODIFY {1} varchar({2})", tableName, columnName, newSize);
                else
                    sql = String.Format("ALTER TABLE {0} MODIFY {1} LONG VARCHAR", tableName, columnName);
                using (AsaCommand command = new AsaCommand(sql, connection))
                {
                    command.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Opens this instance of the update schema provider.
        /// </summary>
        public void Open()
        {
            connection.ConnectionString = ConnectionString;
            connection.Open();
        }

        /// <summary>
        /// Renames the column.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string oldColumnName = updateItem.OldColumnName;
            string newColumnName = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.BeforeXpoSchemaUpdate
                && ColumnExists(tableName, oldColumnName)
                && !ColumnExists(tableName, newColumnName))
            {
                string sql = String.Format("ALTER TABLE {0} RENAME {1} TO {2}",
                    tableName, oldColumnName, newColumnName);
                using (AsaCommand cmd = new AsaCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(ServicesHelper.GetString("UpdateItemSuccess"), updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <returns>
        /// 	<c>true</c> if the table exists; otherwise <c>false</c>.
        /// </returns>
        public bool TableExists(string tableName)
        {
            string sql = String.Format("SELECT table_name FROM SYS.SYSTABLE WHERE table_name='{0}'", tableName);
            using (AsaCommand command = new AsaCommand(sql, connection))
            {
                string result = (string)command.ExecuteScalar();
                if (result == null)
                    return false;
                return true;
            }
        }

        private DataTable FillDataTable(string sql)
        {
            DataTable dataTable = new DataTable("Data");
            using (AsaCommand cmd = new AsaCommand(sql, connection))
            {
                using (AsaDataAdapter adapter = new AsaDataAdapter(cmd))
                    adapter.Fill(dataTable);
            }
            return dataTable;
        }

        // HKEY_CURRENT_USER\Software\Sybase\Adaptive Server Anywhere
        // HKEY_LOCAL_MACHINE\SOFTWARE\Sybase\Adaptive Server Anywhere\9.0
    }
}
#endif // (ASA)
