#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

// Comment out the following define if you don't want the provider compiled into
// your application. You will want to comment this out if you don't have VistaDB
// on your development machine.
//#define VistaDB3Base

#if (VistaDB3Base)
namespace DevExpressContrib.Xpo.UpdateSchema
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Data;
    using DevExpress.Xpo.DB;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Interface;
    using DevExpressContrib.Xpo.UpdateSchema.Properties;
    using VistaDB;
    using VistaDB.DDA;
    using VistaDB.Provider;

    /// <summary>
    /// Implements the database specific functionality for the VistaDB database.
    /// </summary>
    [UpdateSchemaBonusProviderAttribute]
    [UpdateSchemaProvider(VistaDB3ConnectionProvider.XpoProviderTypeString)]
    public class VistaDB3UpdateSchemaProvider : BaseUpdateSchemaProvider, IUpdateSchemaProvider
    {

        /// <summary>
        /// The VistaDB 3 connection object.
        /// </summary>
        protected VistaDBConnection connection;


        /// <summary>
        /// Initializes a new instance of the <see cref="VistaDB3UpdateSchemaProvider"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="servicesHelper">The services helper.</param>
        public VistaDB3UpdateSchemaProvider(string connectionString, IServicesHelper servicesHelper)
            : base(servicesHelper)
        {
            ConnectionString = connectionString;
            connection = new VistaDBConnection();
        }


        /// <summary>
        /// Gets the maximum size of the string before the provider has to change
        /// to a different data type.
        /// </summary>
        /// <value>The maximum size of the string.</value>
        public int MaximumStringSize
        {
            get { return MSSqlConnectionProvider.MaximumStringSize; }
        }

        /// <summary>
        /// Gets the xpo provider type string for this provider.
        /// </summary>
        /// <value>The xpo provider type string.</value>
        public string XpoProviderTypeString
        {
            get { return VistaDB3ConnectionProvider.XpoProviderTypeString; }
        }


        /// <summary>
        /// Determines if the supplied column exists within the supplied table.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column exists; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnExists(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("Columns"))
                return ProviderLinqHelper.ColumnExists(tableName, columnName, schema);
        }

        /// <summary>
        /// Determines if the given column has a foreign key relationship. 
        /// ColumnHasForeignKeyRelationship checks for columns on both sides of the relationship.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has a foreign key relationship; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasForeignKeyRelationship(string tableName, string columnName)
        {
            using (IVistaDBDDA DDAObj = VistaDBEngine.Connections.OpenDDA())
            {
                using (IVistaDBDatabase db = DDAObj.OpenDatabase(FileBasedProviderHelper.GetDatabaseName(ConnectionString),
                    VistaDBDatabaseOpenMode.NonexclusiveReadOnly, null))
                {
                    // VistaDB3 only allows foreign key relationships on the primary key identity.
                    // So get all of the columns in the primary key and use those in the check for
                    // foreign key relationships.
                    List<string> identity = GetIdentityFields(tableName);
                    IVistaDBRelationshipList list = db.Relationships;
                    foreach (DictionaryEntry entry in list)
                    {
                        if (entry.Value is IVistaDBRelationshipInformation)
                        {
                            IVistaDBRelationshipInformation relation = entry.Value as IVistaDBRelationshipInformation;
                            if ((string.Compare(relation.ForeignTable, tableName, true) == 0 && string.Compare(relation.ForeignKey, columnName, true) == 0)
                                || (string.Compare(relation.PrimaryTable, tableName, true) == 0 && identity.Contains(columnName.ToUpper())))
                                return true;
                        }
                    }
                    return false;
                }
            }
        }

        /// <summary>
        /// Determines if the given column has an index.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column has an index; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnHasIndex(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("IndexColumns"))
                return ProviderLinqHelper.ColumnHasIndex(tableName, columnName, schema);
        }

        /// <summary>
        /// Determines if the column holds a string value.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// 	<c>true</c> if the column holds a string value; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnIsString(string tableName, string columnName)
        {
            using (DataTable schema = connection.GetSchema("Columns"))
                return ProviderLinqHelper.ColumnIsString(tableName, columnName, schema,
                    new string[] { "nvarchar", "varchar", "ntext", "text" });
        }

        /// <summary>
        /// Determines if any of the column string values will be truncated.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="newSize">The new size of the column.</param>
        /// <returns>
        /// 	<c>true</c> if at least one column value will be truncated; otherwise <c>false</c>.
        /// </returns>
        public bool ColumnStringValuesWillBeTruncated(string tableName, string columnName, int newSize)
        {
            using (DataTable data = FillDataTable(String.Format("SELECT {0} from {1}", columnName, tableName)))
                return ProviderLinqHelper.ColumnStringValuesWillBeTruncated(tableName, columnName, newSize, data);
        }

        /// <summary>
        /// Determines if the databases exists. Uses the ConnectionString to obtain
        /// information about the database.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the database exists; otherwise <c>false</c>.
        /// </returns>
        public bool DatabaseExists()
        {
            return FileBasedProviderHelper.DatabaseExists(ConnectionString);
        }

        /// <summary>
        /// Deletes the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool DeleteColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnNameToDelete = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && ColumnExists(tableName, columnNameToDelete))
            {
                string sql = String.Format("ALTER TABLE [{0}] DROP COLUMN [{1}]",
                    tableName, columnNameToDelete);
                using (VistaDBCommand cmd = new VistaDBCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(Resources.UpdateItemSuccess, updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            connection.Close();
            connection.Dispose();
        }

        /// <summary>
        /// Attempts to drop the database.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns><c>true</c> if the database was dropped; otherwise <c>false</c>.</returns>
        public static bool DropDatabase(string connectionString)
        {
            return FileBasedProviderHelper.DropDatabase(connectionString);
        }

        /// <summary>
        /// Gets the number of characters of the string column can hold.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="columnName">Name of the column.</param>
        /// <returns>
        /// The number of characters that the column can hold.
        /// </returns>
        public int GetStringColumnSize(string tableName, string columnName)
        {
            if (ColumnIsString(tableName, columnName))
            {
                using (DataTable schema = connection.GetSchema("Columns"))
                    return ProviderLinqHelper.GetStringColumnSize(tableName, columnName, schema);
            }
            return 0;
        }

        /// <summary>
        /// Modifies the size of the string column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool ModifyStringColumnSize(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string columnName = updateItem.ColumnName;
            int newSize = updateItem.Size;
            if (order == UpdateSchemaOrder.AfterXpoSchemaUpdate
                && GetStringColumnSize(tableName, columnName) != newSize)
            {
                string sql;
                const string sqlFormat = "ALTER TABLE {0} ALTER COLUMN {1} nvarchar({2})";
                if (newSize > 0 && newSize <= MaximumStringSize)
                    sql = String.Format(sqlFormat, tableName, columnName, newSize);
                else
                    sql = String.Format(sqlFormat, tableName, columnName, "max");
                using (VistaDBCommand cmd = new VistaDBCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(Resources.UpdateItemSuccess, updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Opens this instance of the update schema provider.
        /// </summary>
        public void Open()
        {
            connection.ConnectionString = ConnectionString;
            connection.Open();
        }

        /// <summary>
        /// Rename the column.
        /// </summary>
        /// <param name="order">Whether this method is being called before or after the XPO schema update.</param>
        /// <param name="updateItem">The update item.</param>
        /// <returns>
        /// 	<c>true</c>.
        /// </returns>
        public bool RenameColumn(UpdateSchemaOrder order, UpdateItemInfo updateItem)
        {
            string tableName = updateItem.TableName;
            string oldColumnName = updateItem.OldColumnName;
            string newColumnName = updateItem.ColumnName;
            if (order == UpdateSchemaOrder.BeforeXpoSchemaUpdate
                && ColumnExists(tableName, oldColumnName)
                && !ColumnExists(tableName, newColumnName))
            {
                string sql = String.Format("EXEC sp_rename('{0}.{1}', '{2}', 'column')",
                    tableName, oldColumnName, newColumnName);
                using (VistaDBCommand cmd = new VistaDBCommand(sql, connection))
                {
                    cmd.ExecuteNonQuery();
                    if (UpdateSchemaLogger != null)
                        UpdateSchemaLogger.Log(String.Format(Resources.UpdateItemSuccess, updateItem.ToString()));
                }
            }
            return true;
        }

        /// <summary>
        /// Determines if the supplied table exists.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <returns>
        /// 	<c>true</c> if the table exists; otherwise <c>false</c>.
        /// </returns>
        public bool TableExists(string tableName)
        {
            using (DataTable schema = connection.GetSchema("Tables"))
                return ProviderLinqHelper.TableExists(tableName, schema);
        }

        private DataTable FillDataTable(string sql)
        {
            DataTable dataTable = new DataTable("Data");
            using (VistaDBCommand cmd = new VistaDBCommand(sql, connection))
            {
                using (VistaDBDataAdapter adapter = new VistaDBDataAdapter(cmd))
                    adapter.Fill(dataTable);
            }
            return dataTable;
        }

        private List<string> GetIdentityFields(string tableName)
        {
            List<string> result = new List<string>();
            using (DataTable data = FillDataTable(String.Format("SELECT name from (SELECT name, typeID FROM [Database Schema]"
                + " WHERE (typeId = 6) AND (foreignReference = "
                + " (SELECT TOP 1 objectId FROM [Database Schema] WHERE TypeId = 1 AND name = '{0}')))", tableName)))
            {
                foreach (DataRow row in data.Rows)
                {
                    result.Add(row.Field<string>("name").Trim().ToUpper());
                }
            }
            return result;
        }
    }
}
#endif // (VistaDB3Base)
