#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.XtraGrid.DxValidateColumnProvider
{
    using System;
    using DevExpress.XtraEditors.DXErrorProvider;
    using DevExpress.XtraGrid.Columns;
    using DevExpress.XtraGrid.Views.Base;

    /// <summary>
    /// ColumnValidationRuleBase contains members to support all column validation.
    /// </summary>
    public abstract class ColumnValidationRuleBase : ValidationRule
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="ColumnValidationRuleBase"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        public ColumnValidationRuleBase(string defaultErrorText)
        {
            DefaultErrorText = defaultErrorText;
            ErrorType = ErrorType.Default;
            ErrorText = defaultErrorText;
            CaseSensitive = false;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ColumnValidationRuleBase"/> class.
        /// </summary>
        public ColumnValidationRuleBase() : this(String.Empty) { }


        /// <summary>
        /// Gets or sets the column to validate.
        /// </summary>
        /// <value>The column.</value>
        public GridColumn Column { get; set; }

        /// <summary>
        /// Gets or sets the default error text of the validator.
        /// </summary>
        /// <value>The name.</value>
        public string DefaultErrorText { get; set; }

        /// <summary>
        /// Gets or sets the row handle of the current row in the view.
        /// </summary>
        /// <value>The row handle.</value>
        public int RowHandle { get; set; }

        /// <summary>
        /// Gets or sets the view to which the column belongs.
        /// </summary>
        /// <value>The view to which the column belongs.</value>
        public BaseView View { get; set; }


        /// <summary>
        /// Determines whether this instance can be validated.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can be validated; otherwise, <c>false</c>.
        /// </returns>
        public virtual bool CanValidate()
        {
            return true;
        }

        /// <summary>
        /// Validates the specified value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<c>true</c> if the value (or values) are valid; otherwise <c>false</c>.
        /// </returns>
        public abstract bool Validate(object value);

        /// <summary>
        /// Gets a column based on a field name.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <returns>A GridColumn if there is a column associated with the fieldname GridColumn; otherwise <c>null</c>.</returns>
        protected GridColumn ColumnByFieldName(string fieldName)
        {
            return DxValidateColumnProviderHelper.ColumnByFieldName(View, fieldName);
        }

        /// <summary>
        /// Gets the current focused column.
        /// </summary>
        /// <returns></returns>
        protected GridColumn FocusedColumn()
        {
            return DxValidateColumnProviderHelper.FocusedColumn(View);
        }

        /// <summary>
        /// Gets the row cell value associated with the supplied fieldname. The row
        /// is the currently selected row of the active view.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <returns>Value of the row cell.</returns>
        protected object GetRowCellValue(string fieldName)
        {
            return GetRowCellValue(ColumnByFieldName(fieldName));
        }

        /// <summary>
        /// Gets the row cell value associated with the supplied GridColumn. The row
        /// is the currently selected row of the active view.
        /// </summary>
        /// <param name="column">The column.</param>
        /// <returns>Value of the row cell.</returns>
        protected object GetRowCellValue(GridColumn column)
        {
            return DxValidateColumnProviderHelper.GetRowCellValue(View, RowHandle, column);
        }

        /// <summary>
        /// Sets the column error to ErrorType.Critical for the supplied column.
        /// </summary>
        /// <param name="column">The column.</param>
        /// <param name="errorText">The error text.</param>
        protected void SetColumnError(GridColumn column, string errorText)
        {
            SetColumnError(column, errorText, ErrorType.Critical);
        }

        /// <summary>
        /// Sets the column error to ErrorType.Critical for the supplied column name.
        /// </summary>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="errorText">The error text.</param>
        protected void SetColumnError(string columnName, string errorText)
        {
            SetColumnError(columnName, errorText, ErrorType.Critical);
        }

        /// <summary>
        /// Sets the column error for the supplied column.
        /// </summary>
        /// <param name="column">The column.</param>
        /// <param name="errorText">The error text.</param>
        /// <param name="errorType">Type of the error.</param>
        protected void SetColumnError(GridColumn column, string errorText, ErrorType errorType)
        {
            DxValidateColumnProviderHelper.SetColumnError(View, column, errorText, errorType);
        }

        /// <summary>
        /// Sets the column error for the supplied column name.
        /// </summary>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="errorText">The error text.</param>
        /// <param name="errorType">Type of the error.</param>
        protected void SetColumnError(string columnName, string errorText, ErrorType errorType)
        {
            SetColumnError(ColumnByFieldName(columnName), errorText, errorType);
        }
    }
}
