#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.XtraGrid.DxValidateColumnProvider
{
    using System;
    using DevExpress.XtraEditors.DXErrorProvider;

    /// <summary>
    /// Compares the values of two objects based on a CompareControlOperator.
    /// </summary>
    public class CompareAgainstColumnValueValidator : ColumnValidationRuleBase, IColumnValidator
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareAgainstColumnValueValidator"/> class.
        /// </summary>
        /// <param name="compareAgainstColumnName">Name of the column to compare against.</param>
        /// <param name="compareOperator">The compare operator.</param>
        /// <param name="defaultErrorText">The default error text.</param>
        public CompareAgainstColumnValueValidator(string compareAgainstColumnName, CompareControlOperator compareOperator, string defaultErrorText)
            : base(defaultErrorText)
        {
            CompareAgainstColumnName = compareAgainstColumnName;
            CompareOperator = compareOperator;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareAgainstColumnValueValidator"></see> class.
        /// </summary>
        /// <param name="compareAgainstColumnName">Name of the column to compare against.</param>
        /// <param name="compareOperator">The compare operator.</param>
        public CompareAgainstColumnValueValidator(string compareAgainstColumnName, CompareControlOperator compareOperator)
            : this(compareAgainstColumnName, compareOperator, String.Empty) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareAgainstColumnValueValidator"/> class.
        /// </summary>
        /// <param name="compareAgainstColumnName">Name of the column to compare against.</param>
        /// <param name="defaultErrorText">The default error text.</param>
        public CompareAgainstColumnValueValidator(string compareAgainstColumnName, string defaultErrorText)
            : this(compareAgainstColumnName, CompareControlOperator.None, defaultErrorText) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="CompareAgainstColumnValueValidator"/> class.
        /// </summary>
        /// <param name="compareAgainstColumnName">Name of the column to compare against.</param>
        public CompareAgainstColumnValueValidator(string compareAgainstColumnName) : this(compareAgainstColumnName, String.Empty) { }


        /// <summary>
        /// Gets or sets the name of the column to compare against.
        /// </summary>
        /// <value>The name of the column to compare against.</value>
        public string CompareAgainstColumnName { get; set; }

        /// <summary>
        /// Gets or sets the compare operator.
        /// </summary>
        /// <value>The compare operator.</value>
        public CompareControlOperator CompareOperator { get; set; }


        /// <summary>
        /// Determines whether this instance can be validated.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can be validated; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanValidate()
        {
            if (CompareOperator == CompareControlOperator.None
                || String.IsNullOrEmpty(CompareAgainstColumnName))
                return false;
            return true;
        }

        /// <summary>
        /// Validates the specified value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<c>true</c> if the value (or values) are valid; otherwise <c>false</c>.
        /// </returns>
        public override bool Validate(object value)
        {
            bool result = ValidationHelper.Validate(value, CompareOperator, GetRowCellValue(CompareAgainstColumnName),
                null, null, base.CaseSensitive);
            if (!result && String.IsNullOrEmpty(DefaultErrorText))
                ErrorText = DefaultErrorText;
            return result;
        }
    }
}
