#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.XtraGrid.DxValidateColumnProvider
{
    using System;
    using System.Collections;
    using DevExpress.XtraEditors.DXErrorProvider;

    /// <summary>
    /// Checks whether a column value matches the specified condition.
    /// </summary>
    public class ConditionColumnValidator : ColumnValidationRuleBase, IColumnValidator
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        /// <param name="conditionOperator">The condition operator.</param>
        /// <param name="value1">The value1.</param>
        /// <param name="value2">The value2.</param>
        public ConditionColumnValidator(string defaultErrorText, ConditionOperator conditionOperator, object value1, object value2)
            : base(defaultErrorText)
        {
            ConditionOperator = conditionOperator;
            Value1 = value1;
            Value2 = value2;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        /// <param name="conditionOperator">The condition operator.</param>
        /// <param name="values">The values.</param>
        public ConditionColumnValidator(string defaultErrorText, ConditionOperator conditionOperator, ICollection values)
            : base(defaultErrorText)
        {
            ConditionOperator = conditionOperator;
            Values = values;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        /// <param name="conditionOperator">The condition operator.</param>
        /// <param name="value">The value.</param>
        public ConditionColumnValidator(string defaultErrorText, ConditionOperator conditionOperator, object value)
            : base(defaultErrorText)
        {
            ConditionOperator = conditionOperator;
            Value1 = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        /// <param name="conditionOperator">The condition operator.</param>
        public ConditionColumnValidator(string defaultErrorText, ConditionOperator conditionOperator)
            : base(defaultErrorText)
        {
            ConditionOperator = conditionOperator;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        /// <param name="defaultErrorText">The default error text.</param>
        public ConditionColumnValidator(string defaultErrorText) : this(defaultErrorText, ConditionOperator.None) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConditionColumnValidator"/> class.
        /// </summary>
        public ConditionColumnValidator() { }


        /// <summary>
        /// Gets or sets the condition operator for this rule.
        /// </summary>
        /// <value>The condition operator.</value>
        public ConditionOperator ConditionOperator { get; set; }

        /// <summary>
        /// Gets or sets the value1.
        /// </summary>
        /// <value>The value1.</value>
        public object Value1 { get; set; }

        /// <summary>
        /// Gets or sets the value2.
        /// </summary>
        /// <value>The value2.</value>
        public object Value2 { get; set; }

        /// <summary>
        /// Gets or sets the values.
        /// </summary>
        /// <value>The values.</value>
        public ICollection Values { get; set; }


        /// <summary>
        /// Determines whether this instance can be validated.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can be validated; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanValidate()
        {
            if (ConditionOperator == ConditionOperator.None)
                return false;
            return true;
        }

        /// <summary>
        /// Validates the specified value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<c>true</c> if the value (or values) are valid; otherwise <c>false</c>.
        /// </returns>
        public override bool Validate(object value)
        {
            bool result = ValidationHelper.Validate(value, ConditionOperator,
                Value1, Value2, Values, base.CaseSensitive);
            if (!result && !String.IsNullOrEmpty(DefaultErrorText))
                ErrorText = DefaultErrorText;
            return result;
        }
    }
}
