#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.XtraGrid.DxValidateColumnProvider
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using DevExpress.XtraEditors.Controls;
    using DevExpress.XtraEditors.DXErrorProvider;
    using DevExpress.XtraGrid.Columns;
    using DevExpress.XtraGrid.Views.Base;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;

    /// <summary>
    /// The DxValidationProvider supplies a convenient path to validating input. 
    /// However, it is more difficult to validate input in the GridControl. 
    /// The DxValidateColumnProvider is an attempt to make this process more convenient.
    /// </summary>
    public class DxValidateColumnProvider : IDisposable
    {

        private Delegate invalidRowExceptionDelegate;
        private Delegate validateDelegate;
        private Dictionary<GridColumn, IColumnValidator> validationRules;
        private List<BaseView> views;


        /// <summary>
        /// Initializes a new instance of the <see cref="DxValidateColumnProvider"/> class.
        /// </summary>
        /// <param name="validateAllColumnsOnRowValidate">if set to <c>true</c> validate all columns on when the row is validated.</param>
        /// <param name="args">The grid views to register with the validation.</param>
        public DxValidateColumnProvider(bool validateAllColumnsOnRowValidate, params BaseView[] args)
        {
            validationRules = new Dictionary<GridColumn, IColumnValidator>();
            validateDelegate = new ValidateRowEventHandler(ValidateRow);
            invalidRowExceptionDelegate = new InvalidRowExceptionEventHandler(InvalidRowException);
            views = new List<BaseView>();
            this.ValidateAllColumnsOnRowValidate = validateAllColumnsOnRowValidate;
            foreach (BaseView view in args)
                DoAddView(view);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DxValidateColumnProvider"/> class.
        /// </summary>
        /// <param name="args">The grid views to register with the validation.</param>
        public DxValidateColumnProvider(params BaseView[] args)
            : this(false, args) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="DxValidateColumnProvider"/> class.
        /// </summary>
        public DxValidateColumnProvider() : this(false) { }


        /// <summary>
        /// Gets or sets the row validator rule. This rule is fired when the row
        /// is about to lose focus. If the rule fails the user will not be allowed
        /// to leave the grid row. It is up to the rule to set an error on one of the
        /// columns and give information about how to fix the error.
        /// </summary>
        /// <value>The row validator rule.</value>
        public ColumnValidationRule RowValidator { get; set; }

        /// <summary>
        /// Gets or sets the services helper.
        /// </summary>
        /// <value>The services helper.</value>
        public IServicesHelper ServicesHelper { get; set; }

        /// <summary>
        /// <para>Gets a value indicating whether the system should validate all columns
        /// when the row is validated. Since the column validation only occurs when
        /// the specific column has changed, it is sometimes necessary to validate
        /// all columns at once. Setting this to true will run each managed columns
        /// <see cref="ColumnValidationRule"/> when the views ValidateRow event is called.
        /// </para>
        /// 
        /// <para>This is especially useful when using the <see cref="CompareAgainstColumnValueValidator"/> class.</para>
        /// 
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the system should validate all columns when the row is validated; otherwise, <c>false</c>.
        /// </value>
        public bool ValidateAllColumnsOnRowValidate { get; private set; }


        /// <summary>
        /// Adds the view to validate..
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="args">Additional views to add.</param>
        public void AddView(BaseView view, params BaseView[] args)
        {
            DoAddView(view);
            foreach (BaseView item in args)
                DoAddView(item);
        }

        /// <summary>
        /// Clears the column errors.
        /// </summary>
        public void ClearColumnErrors()
        {
            foreach (BaseView view in views)
                DxValidateColumnProviderHelper.ClearColumnErrors(view);
        }

        /// <summary>
        /// Determines whether the validator contains the specified column.
        /// </summary>
        /// <param name="column">The column.</param>
        /// <returns>
        /// 	<c>true</c> if the validator contains the specified column; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(GridColumn column)
        {
            return validationRules.ContainsKey(column);
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            for (int i = views.Count - 1; i >= 0; i--)
            {
                RemoveView(views[i]);
            }
        }

        /// <summary>
        /// Determines whether the validator has column errors within the specified view.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <returns>
        /// 	<c>true</c> if the validator has column errors within the specified view; otherwise, <c>false</c>.
        /// </returns>
        public bool HasColumnErrors(BaseView view)
        {
            int errorCount = 0;
            if (DxValidateColumnProviderHelper.HasColumnErrors(view))
                errorCount++;
            if (errorCount == 0)
                foreach (GridColumn column in validationRules.Keys)
                    if (DxValidateColumnProviderHelper.GetColumnErrorType(view, column) != ErrorType.None)
                        errorCount++;
            return errorCount > 0;
        }

        /// <summary>
        /// Removes the view from validation.
        /// </summary>
        /// <param name="view">The view to remove.</param>
        public void RemoveView(BaseView view)
        {
            view.ValidatingEditor -= ValidatingEditor;
            RemoveEvent(view, validateDelegate, "ValidateRow");
            RemoveEvent(view, invalidRowExceptionDelegate, "InvalidRowException");
            views.Remove(view);
        }

        /// <summary>
        /// Sets the column error.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="errorType">Type of the error.</param>
        /// <param name="viewParams">A list of views on which to set the error.</param>
        public void SetColumnError(string message, ErrorType errorType, params BaseView[] viewParams)
        {
            if (viewParams.Length == 0)
                foreach (BaseView view in views)
                    SetColumnError(DxValidateColumnProviderHelper.FocusedColumn(view), message, errorType, view);
            else
                foreach (BaseView view in views)
                    SetColumnError(DxValidateColumnProviderHelper.FocusedColumn(view), message, errorType, view);
        }

        /// <summary>
        /// Sets the column error.
        /// </summary>
        /// <param name="column">The column.</param>
        /// <param name="message">The message.</param>
        /// <param name="errorType">Type of the error.</param>
        /// <param name="viewParams">A list of views on which to set the error.</param>
        public void SetColumnError(GridColumn column, string message, ErrorType errorType, params BaseView[] viewParams)
        {
            if (viewParams.Length == 0)
                foreach (BaseView view in views)
                    SetColumnError(view, column, message, errorType);
            else
                foreach (BaseView view in viewParams)
                    SetColumnError(view, column, message, errorType);
        }

        /// <summary>
        /// Sets the column error.
        /// </summary>
        /// <param name="columnName">Name of the column.</param>
        /// <param name="message">The message.</param>
        /// <param name="errorType">Type of the error.</param>
        /// <param name="viewParams">A list of views on which to set the error.</param>
        public void SetColumnError(string columnName, string message, ErrorType errorType, params BaseView[] viewParams)
        {
            if (viewParams.Length == 0)
                SetColumnError(columnName, message, errorType);
            else
                foreach (BaseView view in views)
                    SetColumnError(DxValidateColumnProviderHelper.ColumnByFieldName(view, columnName), message, errorType, view);
        }

        /// <summary>
        /// Sets the column error.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="column">The column.</param>
        /// <param name="errorText">The error text.</param>
        /// <param name="errorType">Type of the error.</param>
        public void SetColumnError(BaseView view, GridColumn column, string errorText, ErrorType errorType)
        {
            DxValidateColumnProviderHelper.SetColumnError(view, column, errorText, errorType);
        }

        /// <summary>
        /// Sets the validation rule.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="rule">The rule.</param>
        /// <param name="view">The view to register.</param>
        public void SetValidationRule(string fieldName, ColumnValidationRule rule, BaseView view)
        {
            SetValidationRule(fieldName, rule as IColumnValidator, view);
        }

        /// <summary>
        /// Sets the validation rule.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="rule">The rule.</param>
        /// <param name="viewArray">The view array to register.</param>
        public void SetValidationRule(string fieldName, ColumnValidationRule rule, params BaseView[] viewArray)
        {
            SetValidationRule(fieldName, rule as IColumnValidator, viewArray);
        }

        /// <summary>
        /// Sets the validation rule.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="validator">The validator.</param>
        /// <param name="view">The view to register.</param>
        public void SetValidationRule(string fieldName, IColumnValidator validator, BaseView view)
        {
            GridColumn column = DxValidateColumnProviderHelper.ColumnByFieldName(view, fieldName);
            if (column == null)
                return;
            if (validator == null)
                validationRules.Remove(column);
            else if (validationRules.ContainsKey(column))
                validationRules[column] = validator;
            else
                validationRules.Add(column, validator);
        }

        /// <summary>
        /// Sets the validation rule.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="validator">The validator.</param>
        /// <param name="viewArray">The view array to register.</param>
        public void SetValidationRule(string fieldName, IColumnValidator validator, params BaseView[] viewArray)
        {
            // if there are items in the viewArray params then only do those
            if (viewArray.Length > 0)
                foreach (BaseView view in viewArray)
                    SetValidationRule(fieldName, validator, view);
            else // Set the rule for all of the managed views
                foreach (BaseView view in views)
                    SetValidationRule(fieldName, validator, view);
        }

        private void AddEvent(BaseView view, Delegate delegateToAdd, string eventName)
        {
            GuardAgainst.ArgumentIsNull(delegateToAdd, "delegateToAdd");
            EventInfo eventInfo = view.GetType().GetEvent(eventName);
            if (eventInfo != null)
                eventInfo.AddEventHandler(view, delegateToAdd);
            else
                GetServicesHelper().LogError(String.Format("Could not get EventInfo for event {0}.", eventName));
        }

        private void DoAddView(BaseView view)
        {
            view.ValidatingEditor += ValidatingEditor;
            AddEvent(view, validateDelegate, "ValidateRow");
            AddEvent(view, invalidRowExceptionDelegate, "InvalidRowException");
            views.Add(view);
        }

        private GridColumn FocusedColumn(BaseView view)
        {
            return DxValidateColumnProviderHelper.FocusedColumn(view);
        }

        private int FocusedRowHandle(BaseView view)
        {
            return DxValidateColumnProviderHelper.FocusedRowHandle(view);
        }

        private object GetRowCellValue(BaseView view, int rowHandle, GridColumn column)
        {
            return DxValidateColumnProviderHelper.GetRowCellValue(view, rowHandle, column);
        }

        private IServicesHelper GetServicesHelper()
        {
            if (ServicesHelper == null)
                ServicesHelper = new DefaultServicesHelper();
            return ServicesHelper;
        }

        private void InvalidRowException(object sender, InvalidRowExceptionEventArgs e)
        {
            e.ExceptionMode = ExceptionMode.NoAction;
        }

        private void RemoveEvent(BaseView view, Delegate delegateToRemove, string eventName)
        {
            GuardAgainst.ArgumentIsNull(delegateToRemove, "delegateToRemove");
            EventInfo eventInfo = view.GetType().GetEvent(eventName);
            if (eventInfo != null)
                eventInfo.AddEventHandler(view, delegateToRemove);
            else
                GetServicesHelper().LogError(String.Format("Could not get EventInfo for event {0}.", eventName));
        }

        private void ValidateIndividualColumns(object sender, ValidateRowEventArgs e)
        {
            BaseView view = sender as BaseView;
            foreach (GridColumn column in validationRules.Keys)
            {
                BaseContainerValidateEditorEventArgs validateEditorEventArgs = new BaseContainerValidateEditorEventArgs(GetRowCellValue(view, FocusedRowHandle(view), column));
                ValidatingEditor(view, column, validateEditorEventArgs);
                if (!validateEditorEventArgs.Valid)
                {
                    e.ErrorText = validateEditorEventArgs.ErrorText;
                    e.Valid = false;
                }
            }
        }

        private void ValidateRow(object sender, ValidateRowEventArgs e)
        {
            if (ValidateAllColumnsOnRowValidate)
                ValidateIndividualColumns(sender, e);

            if (RowValidator != null && sender is BaseView && e.Valid)
            {
                BaseView view = sender as BaseView;
                ColumnValidationRule rule = RowValidator;
                GridColumn column = FocusedColumn(view);
                rule.Column = column;
                rule.View = view;
                rule.RowHandle = e.RowHandle;
                if (rule.CanValidate())
                {
                    e.Valid = rule.Validate(GetRowCellValue(view, e.RowHandle, column));
                    e.ErrorText = rule.ErrorText;
                    if (!e.Valid)
                        SetColumnError(view, null, rule.ErrorText, rule.ErrorType);
                }
            }
        }

        private void ValidatingEditor(object sender, BaseContainerValidateEditorEventArgs e)
        {
            BaseView view = sender as BaseView;
            ValidatingEditor(view, FocusedColumn(view), e);
        }

        private void ValidatingEditor(BaseView view, GridColumn column, BaseContainerValidateEditorEventArgs e)
        {
            if (Contains(column) && column.View != null)
            {
                IColumnValidator rule = validationRules[column];
                rule.Column = column;
                rule.View = view;
                rule.RowHandle = FocusedRowHandle(view);
                if (rule.CanValidate())
                {
                    SetColumnError(view, column, String.Empty, ErrorType.None);
                    object value = null;
                    string errorMessage = String.Empty;
                    if (!DxValidateColumnProviderHelper.ValidateValue(e.Value, column.ColumnType, out value, out errorMessage))
                    {
                        e.Valid = false;
                        e.ErrorText = errorMessage;
                        SetColumnError(view, column, e.ErrorText, ErrorType.Critical);
                        return;
                    }

                    e.Valid = rule.Validate(value);
                    if (!e.Valid)
                    {
                        e.ErrorText = rule.ErrorText;
                        SetColumnError(view, column, rule.ErrorText, ErrorType.Critical);
                    }
                }
            }
        }
    }
}
