#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace DevExpressContrib.XtraGrid.DxValidateColumnProvider
{
    using System;
    using System.ComponentModel;
    using System.Reflection;
    using DevExpress.XtraEditors.DXErrorProvider;
    using DevExpress.XtraGrid.Columns;
    using DevExpress.XtraGrid.Views.Base;

    /// <summary>
    /// DxValidateColumnProviderHelper contains methods getting information about
    /// necessary to process the column validation.
    /// </summary>
    internal static class DxValidateColumnProviderHelper
    {

        internal static void ClearColumnErrors(BaseView view)
        {
            MethodInfo methodInfo = view.GetType().GetMethod("ClearColumnErrors");
            if (methodInfo != null)
            {
                methodInfo.Invoke(view, null);
                view.HideEditor();
            }
        }

        internal static GridColumn ColumnByFieldName(BaseView view, string fieldName)
        {
            GridColumn result = null;
            PropertyInfo property = view.GetType().GetProperty("Columns", typeof(GridColumnCollection));
            if (property != null)
            {
                GridColumnCollection columns = (GridColumnCollection)property.GetValue(view, null);
                MethodInfo methodInfo = property.PropertyType.GetMethod("ColumnByFieldName",
                    new Type[] { typeof(string) });
                if (methodInfo != null)
                    result = (GridColumn)methodInfo.Invoke(columns, new object[] { fieldName });
            }
            return result;
        }

        internal static GridColumn FocusedColumn(BaseView view)
        {
            GridColumn result = null;
            PropertyInfo propertyInfo = view.GetType().GetProperty("FocusedColumn");
            if (propertyInfo != null)
            {
                object column = propertyInfo.GetValue(view, null);
                result = (GridColumn)column;
            }
            return result;
        }

        internal static int FocusedRowHandle(BaseView view)
        {
            int result = -1;
            PropertyInfo propertyInfo = view.GetType().GetProperty("FocusedRowHandle");
            if (propertyInfo != null)
            {
                object focusedRowHandle = propertyInfo.GetValue(view, null);
                result = (int)focusedRowHandle;
            }
            return result;
        }

        internal static ErrorType GetColumnErrorType(BaseView view, GridColumn column)
        {
            ErrorType result;
            MethodInfo methodInfo = view.GetType().GetMethod("GetColumnErrorType",
                new Type[] { typeof(GridColumn) });
            if (methodInfo != null)
                result = (ErrorType)methodInfo.Invoke(view, new object[] { column });
            else
                result = ErrorType.None;
            return result;
        }

        internal static object GetRowCellValue(BaseView view, int rowHandle, GridColumn column)
        {
            object result;
            MethodInfo methodInfo = view.GetType().GetMethod("GetRowCellValue",
                new Type[] { typeof(int), typeof(GridColumn) });
            if (methodInfo != null)
                result = methodInfo.Invoke(view, new object[] { rowHandle, column });
            else
                result = null;
            return result;
        }

        internal static bool HasColumnErrors(BaseView view)
        {
            bool result;
            PropertyInfo propertyInfo = view.GetType().GetProperty("HasColumnErrors");
            if (propertyInfo != null)
                result = (bool)propertyInfo.GetValue(view, null);
            else
                result = false;
            return result;
        }

        internal static void SetColumnError(BaseView view, GridColumn column, string errorText, ErrorType errorType)
        {
            MethodInfo methodInfo = view.GetType().GetMethod("SetColumnError",
                new Type[] { typeof(GridColumn), typeof(string), typeof(ErrorType) });
            if (methodInfo != null)
                methodInfo.Invoke(view, new object[] { column, errorText, errorType });
        }

        internal static bool ValidateValue(object rawValue, Type columnType, out object value, out string errorMessage)
        {
            value = rawValue;
            errorMessage = String.Empty;

            if (rawValue == null)
                return true;

            bool result = true;
            Type rawValueType = rawValue.GetType();

            if (rawValueType != columnType)
            {
                TypeConverter converter = TypeDescriptor.GetConverter(columnType);
                if (!converter.CanConvertFrom(rawValueType))
                    return result;
                try
                {
                    value = converter.ConvertFrom(rawValue);
                }
                catch (Exception ex)
                {
                    result = false;
                    if (ex is FormatException)
                        errorMessage = ex.Message;
                    else if (ex.InnerException != null && ex.InnerException is FormatException)
                        errorMessage = ex.InnerException.Message;
                    else
                        throw ex;
                }
            }
            return result;
        }
    }
}
