#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace TestDevExpressContrib.Core
{
    using DevExpressContrib.Core;
    using NUnit.Framework;
    using NUnit.Framework.Extensions;
    using NUnit.Framework.SyntaxHelpers;

    [TestFixture]
    public class TestEmailHelper
    {

        [RowTest]
        [Row("My.Name@MyDomain.name", true, "")] // single email address in list
        [Row("My.Name@MyDomain.net;My.Name@MyDomain.org", true, "")] // two email addresses in list
        [Row("My.Name@MyDomain.biz;My.Name@MyDomain.com;My.Name@MyDomain.info", true, "")] // three email addresses in list
        [Row("My.Name@MyDomain.net; My.Name@MyDomain.org", true, "")] // with space next to delimiter
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.au", true, "")] // with country codes
        [Row("My.Name@MyDomain.xx;My.Name@MyDomain.aux", true, "")] // with invalid domain
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.aux", true, "")] // with invalid domain and space next to delimiter
        [Row("@;My.Name@MyDomain.xx", false, "@")]
        public void TestAreEmailAddressesValid(string emailAddress, bool expectedResult, string expectedFirstInvalidEmailAddress)
        {
            string firstInvalidEmailAddress;
            Assert.That(EmailHelper.AreEmailAddressesWellFormed(emailAddress, out firstInvalidEmailAddress), Is.EqualTo(expectedResult));
            Assert.That(expectedFirstInvalidEmailAddress.Trim(), Is.EqualTo(firstInvalidEmailAddress.Trim()));
        }

        [RowTest]
        [Row("My.Name@MyDomain.name", TopLevelDomains.All, true, "")] // single email address in list
        [Row("My.Name@MyDomain.net;My.Name@MyDomain.org", TopLevelDomains.All, true, "")] // two email addresses in list
        [Row("My.Name@MyDomain.biz;My.Name@MyDomain.com;My.Name@MyDomain.info", TopLevelDomains.All, true, "")] // three email addresses in list
        [Row("My.Name@MyDomain.net; My.Name@MyDomain.org", TopLevelDomains.All, true, "")] // with space next to delimiter
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.au", TopLevelDomains.All, true, "")] // with country codes
        [Row(";My.Name@MyDomain.xx;My.Name@MyDomain.au", TopLevelDomains.All, false, "")] // additional ; at start
        [Row("My.Name@MyDomain.xx;My.Name@MyDomain.au;", TopLevelDomains.All, false, "")] // additional ; at end
        [Row("My.Name@MyDomain.xx;;My.Name@MyDomain.au", TopLevelDomains.All, false, "")] // additional ; in middle
        [Row("My.Name@MyDomain.xx;My.Name@MyDomain.aux", TopLevelDomains.All, false, "My.Name@MyDomain.aux")] // with invalid domain
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.aux", TopLevelDomains.All, false, "My.Name@MyDomain.aux")] // with invalid domain and space next to delimiter
        public void TestAreEmailAddressesValid(string emailAddress, TopLevelDomains domains, bool expectedResult, string expectedFirstInvalidEmailAddress)
        {
            string firstInvalidEmailAddress;
            Assert.That(EmailHelper.AreEmailAddressesWellFormed(emailAddress, true, domains, out firstInvalidEmailAddress), Is.EqualTo(expectedResult));
            Assert.That(expectedFirstInvalidEmailAddress.Trim(), Is.EqualTo(firstInvalidEmailAddress.Trim()));
        }

        [RowTest]
        [Row("My.Name@MyDomain.name", true, "")] // single email address in list
        [Row("My.Name@MyDomain.net;My.Name@MyDomain.org", true, "")] // two email addresses in list
        [Row("My.Name@MyDomain.biz;My.Name@MyDomain.com;My.Name@MyDomain.info", true, "")] // three email addresses in list
        [Row("My.Name@MyDomain.net; My.Name@MyDomain.org", true, "")] // with space next to delimiter
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.au", true, "")] // with country codes
        [Row("My.Name@MyDomain.xx;My.Name@MyDomain.aux", false, "My.Name@MyDomain.aux")] // with invalid domain
        [Row("My.Name@MyDomain.xx; My.Name@MyDomain.aux", false, "My.Name@MyDomain.aux")] // with invalid domain and space next to delimiter
        public void TestAreEmailAddressesValid2(string emailAddress, bool expectedResult, string expectedFirstInvalidEmailAddress)
        {
            string firstInvalidEmailAddress;
            Assert.That(EmailHelper.AreEmailAddressesWellFormed(emailAddress, true, out firstInvalidEmailAddress), Is.EqualTo(expectedResult));
            Assert.That(expectedFirstInvalidEmailAddress.Trim(), Is.EqualTo(firstInvalidEmailAddress.Trim()));
        }

        [RowTest]
        [Row("example@testing.test", true)]
        [Row("example.example@test.testing.test", true)]
        [Row("1.2.3.4.5.6.7@1.2.3.4.5.6.com", true)]
        [Row("1.2.3.4.5.6.7@1.2.3.4.5.6", true)]
        [Row("@testing.com", false)] // no username
        [Row("Testing@test", false)] // no top level domain
        [Row("Testing@ test.com", false)] // invalid space
        [Row("Test", false)] // no @ separator
        [Row("@", false)] // just an @ separator
        [Row("", false)] // blank string
        [Row("My.Name@MyDomain.biz;My.Name@MyDomain.com", false)] // sent list into single email checking
        public void TestIsEmailAddressValid(string emailAddress, bool expectedResult)
        {
            Assert.That(EmailHelper.IsEmailAddressWellFormed(emailAddress), Is.EqualTo(expectedResult));
        }

        [Test]
        [ExpectedException(typeof(GuardAgainstArgumentException))]
        public void TestIsEmailAddressValidNull()
        {
            Assert.That(EmailHelper.IsEmailAddressWellFormed(null), Is.False);
        }

        [RowTest]
        [Row("My.Name@MyDomain.biz", true)]
        public void TestValidTopLevelDomain(string emailAddress, bool expectedResult)
        {
            Assert.That(EmailHelper.IsEmailAddressWellFormed(emailAddress, true), Is.EqualTo(expectedResult));
        }

        [RowTest]
        // all of the available generic top level domains
        [Row("My.Name@MyDomain.biz", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.com", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.info", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.name", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.net", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.org", TopLevelDomains.Generic, true)]
        [Row("My.Name@MyDomain.pro", TopLevelDomains.Generic, true)]
        // all of the available sponsored top level domains
        [Row("My.Name@MyDomain.aero", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.asia", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.cat", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.coop", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.edu", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.gov", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.int", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.jobs", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.mil", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.mobi", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.museum", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.tel", TopLevelDomains.Sponsored, true)]
        [Row("My.Name@MyDomain.travel", TopLevelDomains.Sponsored, true)]
        // all of the available top level domains
        [Row("My.Name@MyDomain.biz", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.com", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.info", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.name", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.net", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.org", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.pro", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.aero", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.asia", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.cat", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.coop", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.edu", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.gov", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.int", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.jobs", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.mil", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.mobi", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.museum", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.tel", TopLevelDomains.All, true)]
        [Row("My.Name@MyDomain.travel", TopLevelDomains.All, true)]
        [Row("me@machine.com.au", TopLevelDomains.Generic, true)] // two character domain - assumed to be country code
        [Row("me@machine.com.xx", TopLevelDomains.Generic, true)] // two character domain - assumed to be country code (actual country codes are not checked)
        [Row(" me@machine.com.au", TopLevelDomains.Generic, true)] // with leading space
        [Row("me@machine.com.au ", TopLevelDomains.Generic, true)] // with trailing space
        [Row(" me@machine.com.au ", TopLevelDomains.Generic, true)] // with leading and trailing space
        [Row("me@machine.com.aux", TopLevelDomains.Generic, false)] // three character domain - not in generic
        [Row("me@machine.com.x", TopLevelDomains.Generic, false)] // single character domain - not in generic
        [Row("example@testing.test", TopLevelDomains.Generic, false)] // reserved domain - not in generic
        [Row("example.example@test.testing.example", TopLevelDomains.Generic, false)] // reserved domain - not in generic
        public void TestValidTopLevelDomain(string emailAddress, TopLevelDomains domains, bool expectedResult)
        {
            Assert.That(EmailHelper.IsEmailAddressWellFormed(emailAddress, true, domains), Is.EqualTo(expectedResult));
        }
    }
}
