#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace Test.Ux.WatermarkInputPromptManager
{
    using System;
    using System.Drawing;
    using DevExpress.XtraEditors;
    using DevExpress.XtraEditors.Controls;
    using DevExpressContrib.Core.Ux;
    using NUnit.Framework;
    using NUnit.Framework.Extensions;
    using NUnit.Framework.SyntaxHelpers;

    [TestFixture]
    public class TestWatermarkInputPromptManager
    {

        [RowTest]
        [Row(typeof(ButtonEdit), true)]
        [Row(typeof(CalcEdit), true)]
        [Row(typeof(ComboBoxEdit), true)]
        [Row(typeof(DateEdit), true)]
        [Row(typeof(MemoEdit), true)]
        [Row(typeof(MRUEdit), true)]
        [Row(typeof(TextEdit), true)]
        [Row(typeof(BaseSpinEdit), false)]
        [Row(typeof(CheckedComboBoxEdit), false)]
        [Row(typeof(CheckEdit), false)]
        [Row(typeof(ColorEdit), false)]
        [Row(typeof(FontEdit), false)]
        [Row(typeof(HyperLinkEdit), false)]
        [Row(typeof(ImageComboBoxEdit), false)]
        [Row(typeof(ImageEdit), false)]
        [Row(typeof(LookUpEdit), false)]
        [Row(typeof(MarqueeProgressBarControl), false)]
        [Row(typeof(MemoExEdit), false)]
        [Row(typeof(PictureEdit), false)]
        [Row(typeof(PopupContainerEdit), false)]
        [Row(typeof(ProgressBarBaseControl), false)]
        [Row(typeof(ProgressBarControl), false)]
        [Row(typeof(RadioGroup), false)]
        [Row(typeof(RangeTrackBarControl), false)]
        [Row(typeof(SpinEdit), false)]
        [Row(typeof(TimeEdit), false)]
        [Row(typeof(TrackBarControl), false)]
        [Row(typeof(ZoomTrackBarControl), false)]
        // These are abstract and cannot be instantiated.
        //[Row(typeof(BlobBaseEdit), false)]
        //[Row(typeof(LookUpEditBase), false)]
        //[Row(typeof(PopupBaseAutoSearchEdit), false)]
        //[Row(typeof(PopupBaseEdit), false)]
        public void TestAdd(Type controlType, bool expectedResult)
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                BaseEdit control = (BaseEdit)Activator.CreateInstance(controlType);
                watermark.Add(control, control.GetType().ToString());
                if (expectedResult)
                    Assert.That(watermark.Count, Is.EqualTo(1));
                else
                    Assert.That(watermark.Count, Is.EqualTo(0));
            }
            // now test if it will throw the exception
            Assert.That(TestAddWillThrowExceptionYes(controlType), Is.EqualTo(!expectedResult));
        }

        [RowTest]
        [Row(TextEditStyles.Standard, true)]
        [Row(TextEditStyles.DisableTextEditor, false)]
        [Row(TextEditStyles.HideTextEditor, false)]
        public void TestAddInvalidComboBoxEdit(TextEditStyles style, bool expectedResult)
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                ComboBoxEdit control = new ComboBoxEdit();
                control.Properties.TextEditStyle = style;
                watermark.Add(control, control.GetType().ToString());
                if (expectedResult)
                    Assert.That(watermark.Count, Is.EqualTo(1));
                else
                    Assert.That(watermark.Count, Is.EqualTo(0));
            }
            // now test if it will throw the exception
            Assert.That(TestAddWillThrowExceptionYes(style), Is.EqualTo(!expectedResult));
        }

        [Test]
        public void TestAddWithDifferentColors()
        {
            TestAddProperties(Color.Empty, Color.Empty); // default Colors;
            TestAddProperties(Color.DarkViolet, Color.Purple);
            TestAddProperties(Color.Empty, Color.Purple);
            TestAddProperties(Color.DarkViolet, Color.Empty);
        }

        [Test]
        public void TestDefaults()
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                Assert.That(watermark.Count, Is.EqualTo(0));
                Assert.That(watermark.DefaultTextColor, Is.EqualTo(SystemColors.ControlText));
                Assert.That(watermark.DefaultWatermarkTextColor, Is.EqualTo(SystemColors.GrayText));
                Assert.That(watermark.ThrowExceptionWhenAddingInvalidControl, Is.False);
            }
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager(Color.Plum, Color.Red))
            {
                Assert.That(watermark.Count, Is.EqualTo(0));
                Assert.That(watermark.DefaultTextColor, Is.EqualTo(Color.Red));
                Assert.That(watermark.DefaultWatermarkTextColor, Is.EqualTo(Color.Plum));
                Assert.That(watermark.ThrowExceptionWhenAddingInvalidControl, Is.False);
            }
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDuplicateControl()
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                TextEdit edit1 = new TextEdit();
                watermark.Add(edit1, "edit1", Color.LightBlue, Color.CadetBlue);
                watermark.Add(edit1, "edit2", Color.LightSlateGray, Color.DarkGray);
            }
        }

        [Test]
        public void TestRemove()
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                TextEdit edit1 = new TextEdit();
                watermark.Add(new WatermarkInputPromptItem(edit1, "edit1"));
                TextEdit edit2 = new TextEdit();
                watermark.Add(new WatermarkInputPromptItem(edit2, "edit2"));

                Assert.That(watermark.Count, Is.EqualTo(2));
                Assert.That(watermark.Contains(edit1), Is.True);
                Assert.That(watermark.Contains(edit2), Is.True);

                watermark.Remove(edit2);
                Assert.That(watermark.Count, Is.EqualTo(1));
                Assert.That(watermark.Contains(edit1), Is.True);
                Assert.That(watermark.Contains(edit2), Is.False);
                Assert.That(watermark.GetPrompt(edit2), Is.EqualTo(String.Empty));
                Assert.That(watermark.GetWatermarkTextColor(edit2), Is.EqualTo(watermark.DefaultWatermarkTextColor));
                Assert.That(watermark.GetTextColor(edit2), Is.EqualTo(watermark.DefaultTextColor));
            }
        }

        private void TestAddProperties(Color watermarkColor, Color textColor)
        {
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager(watermarkColor, textColor))
            {
                TextEdit edit1 = new TextEdit();
                watermark.Add(edit1, "edit1", Color.LightBlue, Color.CadetBlue);
                TextEdit edit2 = new TextEdit();
                watermark.Add(edit2, "edit2", Color.LightSlateGray, Color.DarkGray);
                TextEdit edit3 = new TextEdit();
                watermark.Add(edit3, "edit3");

                Assert.That(watermark.Contains(edit1), Is.True);
                Assert.That(watermark.Contains(edit2), Is.True);
                Assert.That(watermark.Contains(edit3), Is.True);

                Assert.That(watermark.GetPrompt(edit1), Is.EqualTo("edit1"));
                Assert.That(watermark.GetWatermarkTextColor(edit1), Is.EqualTo(Color.LightBlue));
                Assert.That(watermark.GetTextColor(edit1), Is.EqualTo(Color.CadetBlue));

                Assert.That(watermark.GetPrompt(edit2), Is.EqualTo("edit2"));
                Assert.That(watermark.GetWatermarkTextColor(edit2), Is.EqualTo(Color.LightSlateGray));
                Assert.That(watermark.GetTextColor(edit2), Is.EqualTo(Color.DarkGray));

                Assert.That(watermark.GetPrompt(edit3), Is.EqualTo("edit3"));
                if (watermarkColor == Color.Empty)
                    Assert.That(watermark.GetWatermarkTextColor(edit3), Is.EqualTo(watermark.DefaultWatermarkTextColor));
                else
                    Assert.That(watermark.GetWatermarkTextColor(edit3), Is.EqualTo(watermarkColor));
                if (textColor == Color.Empty)
                    Assert.That(watermark.GetTextColor(edit3), Is.EqualTo(watermark.DefaultTextColor));
                else
                    Assert.That(watermark.GetTextColor(edit3), Is.EqualTo(textColor));
            }
        }

        private bool TestAddWillThrowExceptionYes(TextEditStyles style)
        {
            bool threwException = false;
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                watermark.ThrowExceptionWhenAddingInvalidControl = true;
                ComboBoxEdit control = new ComboBoxEdit();
                control.Properties.TextEditStyle = style;
                try
                {
                    watermark.Add(control, control.GetType().ToString());
                }
                catch (ArgumentException)
                {
                    threwException = true;
                }
            }
            return threwException;
        }

        private bool TestAddWillThrowExceptionYes(Type type)
        {
            bool threwException = false;
            using (WatermarkInputPromptManager watermark = new WatermarkInputPromptManager())
            {
                watermark.ThrowExceptionWhenAddingInvalidControl = true;
                BaseEdit control = (BaseEdit)Activator.CreateInstance(type);
                try
                {
                    watermark.Add(control, control.GetType().ToString());
                }
                catch (ArgumentException)
                {
                    threwException = true;
                }
            }
            return threwException;
        }
    }
}
