#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace Test.Xpo.UpdateSchema
{
    using System;
    using System.Reflection;
    using DevExpress.Xpo;
    using DevExpress.Xpo.DB;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema;
    using NUnit.Framework;
    using NUnit.Framework.Extensions;
    using NUnit.Framework.SyntaxHelpers;
    using Test.Xpo.UpdateSchema.Providers.Data;

    [TestFixture]
    public class TestProviders
    {

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestColumns(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(NameTestClassOne)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        Assert.That(updateSchemaProvider.ColumnExists("NameTestClassFour", "PublicField"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnExists("NameTestClassFour", "ProtectedField"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnExists("NameTestClassFour", "PublicProperty"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnExists("NameTestClassFour", "ProtectedProperty"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnExists("NameTestClassFour", "PersistantWithMapTo"), Is.True);
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //RegisterXpoProviders.Register();
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestHasForeignKeyRelationship(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(TestForeignKeyNoCollection)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("TestForeignKeyNoCollection", "Data"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("TestForeignKeyNoCollection", "OID"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("TestForeignKeyNoCollectionChild", "TestForeignKeyNoCollection"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("TestForeignKeyNoCollection", "TestForeignKeyChildren"), Is.False);

                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("Location", "OID"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("Location", "Departments"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("Department", "OID"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("Department", "Locations"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("LocationLocations_DepartmentDepartments", "Locations"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasForeignKeyRelationship("LocationLocations_DepartmentDepartments", "Departments"), Is.True);
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestHasIndex(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(TestIndexes)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "NoIndex"), Is.False);

                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "OID"), Is.True); // primary key should have index.
                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "HasUniqueIndex"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "IndexedNoName"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "HasIndexWithAdditionalFields"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnHasIndex("TestIndexes", "AdditionalFieldForIndex"), Is.True);
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString, SizeAttribute.Unlimited)]
        [Row(TestConnectionString.accessConnectionString, 0)]
        [Row(TestConnectionString.vistaDB3ConnectionString, 67106816)]
        [Row(TestConnectionString.asaConnectionString, 32767)]
        public void TestStrings(string connectionString, int unlimitedSize)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(StringTestClass)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        Assert.That(updateSchemaProvider.ColumnExists("StringTestClass", "PublicField"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnExists("StringTestClass", "PublicProperty"), Is.True);

                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicIntField"), Is.False);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicIntProperty"), Is.False);

                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicField"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicProperty"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicFieldMax"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicPropertyMax"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicFieldDefaultSize"), Is.True);
                        Assert.That(updateSchemaProvider.ColumnIsString("StringTestClass", "PublicPropertyDefaultSize"), Is.True);

                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicField"), Is.EqualTo(32));
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicProperty"), Is.EqualTo(32));
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicFieldMax") == unlimitedSize);
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicPropertyMax") == unlimitedSize);
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicFieldDefaultSize"), Is.EqualTo(SizeAttribute.DefaultStringMappingFieldSize));
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicPropertyDefaultSize"), Is.EqualTo(SizeAttribute.DefaultStringMappingFieldSize));

                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicIntField"), Is.EqualTo(0));
                        Assert.That(updateSchemaProvider.GetStringColumnSize("StringTestClass", "PublicIntProperty"), Is.EqualTo(0));
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestTableNames(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(NameTestClassOne)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        Assert.That(updateSchemaProvider.TableExists("NameTestClassOne"), Is.False);
                        foreach (string tableName in UpdateSchemaHelper.TableNames(assemblies))
                        {
                            bool tableExists = updateSchemaProvider.TableExists(tableName);
                            Assert.That(tableExists, Is.True);
                        }
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestTruncateStrings(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);
                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(StringTestTruncate)) };
                    UpdateSchema updateSchema = new UpdateSchema(connectionString);
                    bool result = updateSchema.Update(assemblies);
                    Assert.That(result, Is.True);

                    int freeSpace = 3;
                    int columnFieldSize;
                    IServicesHelper servicesHelper = new DefaultServicesHelper();
                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        columnFieldSize = updateSchemaProvider.GetStringColumnSize("StringTestTruncate", "TruncateMe");
                    }

                    using (IDataLayer dataLayer = XpoDefault.GetDataLayer(connectionString, AutoCreateOption.None))
                    {
                        using (Session session = new Session(dataLayer))
                        {
                            using (XPCollection<StringTestTruncate> test = new XPCollection<StringTestTruncate>(session))
                            {
                                string stringToAdd = "";
                                char character = 'a';
                                for (int i = 0; i < columnFieldSize - freeSpace; i++)
                                {
                                    using (StringTestTruncate stringTestTruncate = new StringTestTruncate(session))
                                    {
                                        stringToAdd += character;
                                        stringTestTruncate.TruncateMe = stringToAdd;
                                        test.Add(stringTestTruncate);
                                        stringTestTruncate.Save();
                                    }
                                    character = (char)(character + 1);
                                }
                            }
                            session.Connection.Close();
                        }
                        dataLayer.Connection.Close();
                    }

                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        for (int i = 1; i < columnFieldSize + 5; i++)
                        {
                            if (i >= columnFieldSize - freeSpace)
                                Assert.That(updateSchemaProvider.ColumnStringValuesWillBeTruncated("StringTestTruncate", "TruncateMe", i), Is.False);
                            else
                                Assert.That(updateSchemaProvider.ColumnStringValuesWillBeTruncated("StringTestTruncate", "TruncateMe", i), Is.True);
                        }
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }
    }
}
