#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace Test.Xpo.UpdateSchema
{
    using System;
    using System.Data;
    using System.Threading;
    using DevExpress.Xpo;
    using DevExpress.Xpo.DB;
    using DevExpressContrib.Core;
    using NUnit.Framework;
    using NUnit.Framework.SyntaxHelpers;

    [TestFixture]
    public class TestSchemaVersion
    {
        [Test]
        [ExpectedException(typeof(GuardAgainstValueException))]
        public void TestInvalidClassType()
        {
            using (TestSchemaVersionStringInvalidXpoType schemaVersion = new TestSchemaVersionStringInvalidXpoType())
            {
                Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
            }
        }

        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestInvalidPropertyNames()
        {
            IDataStore dataStore = new InMemoryDataStore(new DataSet(), AutoCreateOption.DatabaseAndSchema);
            using (IDataLayer dataLayer = new SimpleDataLayer(dataStore))
            {
                using (Session session = new Session(dataLayer))
                {
                    session.UpdateSchema(typeof(SchemaVersionXpoClassInt));
                    session.CreateObjectTypeRecords(typeof(SchemaVersionXpoClassInt));
                }
                using (TestSchemaVersionStringInvalidProperties schemaVersion = new TestSchemaVersionStringInvalidProperties(dataLayer))
                {
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.False);
                }
            }
        }

        [Test]
        public void TestMemoryDataSetUpdateString()
        {
            IDataStore dataStore = new InMemoryDataStore(new DataSet(), AutoCreateOption.DatabaseAndSchema);
            using (IDataLayer dataLayer = new SimpleDataLayer(dataStore))
            {
                using (Session session = new Session(dataLayer))
                {
                    session.UpdateSchema(typeof(SchemaVersionXpoClassString));
                    session.CreateObjectTypeRecords(typeof(SchemaVersionXpoClassString));
                }
                using (TestSchemaVersionString schemaVersion = new TestSchemaVersionString(dataLayer))
                {
                    const string helloWorld = "Hello World";
                    const string goodbyeWorld = "Goodbye Cruel World";
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(default(String)));
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(helloWorld));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.GetDateVersionUpdated(), Is.EqualTo(DateTime.MinValue));
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    DateTime currentDateTime = DateTime.SpecifyKind(DateTime.Now, schemaVersion.DateVersionUpdatedKind);
                    DateTime schemaVersionDateTime1 = schemaVersion.GetDateVersionUpdated();
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(helloWorld));
                    Assert.That(SchemaVersionDateEquals(schemaVersionDateTime1, currentDateTime), Is.True);
                    Thread.Sleep(2); //ensure that the milliseconds between the last run and this are different
                    schemaVersion.RequiredSchemaVersionMutable = goodbyeWorld;
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(goodbyeWorld));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    currentDateTime = DateTime.SpecifyKind(DateTime.Now, schemaVersion.DateVersionUpdatedKind);
                    DateTime schemaVersionDateTime2 = schemaVersion.GetDateVersionUpdated();
                    Assert.That(schemaVersionDateTime1, Is.Not.EqualTo(schemaVersionDateTime2));
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(goodbyeWorld));
                    Assert.That(SchemaVersionDateEquals(schemaVersionDateTime2, currentDateTime), Is.True);
                }
            }
        }

        [Test]
        public void TestMemoryDataSetUpdateWithOutUpdateDate()
        {
            IDataStore dataStore = new InMemoryDataStore(new DataSet(), AutoCreateOption.DatabaseAndSchema);
            using (IDataLayer dataLayer = new SimpleDataLayer(dataStore))
            {
                using (Session session = new Session(dataLayer))
                {
                    session.UpdateSchema(typeof(SchemaVersionXpoClassIntNoDate));
                    session.CreateObjectTypeRecords(typeof(SchemaVersionXpoClassIntNoDate));
                }
                using (TestSchemaVersionNoDateInt schemaVersion = new TestSchemaVersionNoDateInt(dataLayer))
                {
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(default(int)));
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(1));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.GetDateVersionUpdated(), Is.EqualTo(DateTime.MinValue));
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    Assert.That(schemaVersion.GetDateVersionUpdated(), Is.EqualTo(DateTime.MinValue));
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(1));
                    schemaVersion.RequiredSchemaVersionMutable = 2;
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(2));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    Assert.That(schemaVersion.GetDateVersionUpdated(), Is.EqualTo(DateTime.MinValue));
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(2));
                }
            }
        }

        [Test]
        public void TestMemoryDataSetUpdateWithUpdateDate()
        {
            IDataStore dataStore = new InMemoryDataStore(new DataSet(), AutoCreateOption.DatabaseAndSchema);
            using (IDataLayer dataLayer = new SimpleDataLayer(dataStore))
            {
                using (Session session = new Session(dataLayer))
                {
                    session.UpdateSchema(typeof(SchemaVersionXpoClassInt));
                    session.CreateObjectTypeRecords(typeof(SchemaVersionXpoClassInt));
                }
                using (TestSchemaVersionInt schemaVersion = new TestSchemaVersionInt(dataLayer))
                {
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(default(int)));
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(1));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.GetDateVersionUpdated(), Is.EqualTo(DateTime.MinValue));
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    DateTime currentDateTime = DateTime.SpecifyKind(DateTime.Now, schemaVersion.DateVersionUpdatedKind);
                    DateTime schemaVersionDateTime1 = schemaVersion.GetDateVersionUpdated();
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(1));
                    Assert.That(SchemaVersionDateEquals(schemaVersionDateTime1, currentDateTime), Is.True);
                    Thread.Sleep(100); //ensure that the milliseconds between the last run and this are different
                    schemaVersion.RequiredSchemaVersionMutable = 2;
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(2));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                    currentDateTime = DateTime.SpecifyKind(DateTime.Now, schemaVersion.DateVersionUpdatedKind);
                    DateTime schemaVersionDateTime2 = schemaVersion.GetDateVersionUpdated();
                    Assert.That(schemaVersionDateTime1, Is.Not.EqualTo(schemaVersionDateTime2));
                    Assert.That(schemaVersion.GetCurrentSchemaVersion(), Is.EqualTo(2));
                    Assert.That(SchemaVersionDateEquals(schemaVersionDateTime2, currentDateTime), Is.True);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(GuardAgainstValueException))]
        public void TestMoreThanOneRecord()
        {
            IDataStore dataStore = new InMemoryDataStore(new DataSet(), AutoCreateOption.DatabaseAndSchema);
            using (IDataLayer dataLayer = new SimpleDataLayer(dataStore))
            {
                using (Session session = new Session(dataLayer))
                {
                    session.UpdateSchema(typeof(SchemaVersionXpoClassString));
                    session.CreateObjectTypeRecords(typeof(SchemaVersionXpoClassString));
                    using (SchemaVersionXpoClassString item = new SchemaVersionXpoClassString(session))
                    {
                        item.SchemaVersion = "One";
                        item.Save();
                    }
                    using (SchemaVersionXpoClassString item2 = new SchemaVersionXpoClassString(session))
                    {
                        item2.SchemaVersion = "Two";
                        item2.Save();
                    }
                }
                using (TestSchemaVersionString schemaVersion = new TestSchemaVersionString(dataLayer))
                {
                    const string helloWorld = "Hello World";
                    Assert.That(schemaVersion.GetRequiredSchemaVersion(), Is.EqualTo(helloWorld));
                    Assert.That(schemaVersion.HasSchemaVersionBeenUpdated, Is.False);
                    Assert.That(schemaVersion.UpdateSchemaVersion(), Is.True);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(GuardAgainstValueException))]
        public void TestNoDataConnection()
        {
            using (TestSchemaVersionString schemaVersion = new TestSchemaVersionString())
            {
                schemaVersion.UpdateSchemaVersion();
            }
        }

        private object SchemaVersionDateEquals(DateTime dateTime1, DateTime dateTime2)
        {
            return (dateTime1.Year == dateTime2.Year)
                && (dateTime1.Month == dateTime2.Month)
                && (dateTime1.Day == dateTime2.Day)
                && (dateTime1.Hour == dateTime2.Hour)
                && (dateTime1.Minute <= dateTime2.Minute)
                && (dateTime1.Second <= dateTime2.Second);
        }
    }
}
