#region Copyright  2008-2009, DevExpressContrib (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

// if you don't want the reports to be generated during the testing then
// comment out the following define.
#define SaveReports

namespace Test.Xpo.UpdateSchema
{
    using System;
    using System.IO;
    using System.Reflection;
    using DevExpress.Xpo.DB.Exceptions;
    using DevExpress.Xpo.Exceptions;
    using DevExpressContrib.Core;
    using DevExpressContrib.Core.Interface;
    using DevExpressContrib.Xpo.UpdateSchema;
    using DevExpressContrib.Xpo.UpdateSchemaReports;
    using NUnit.Framework;
    using NUnit.Framework.Extensions;
    using NUnit.Framework.SyntaxHelpers;
    using Test.Xpo.UpdateSchema.Data;

    [TestFixture]
    public class TestUpdateSchema
    {

        private int reportCount = 0;
        // The tempPath is where the reports will be saved.
        private const string tempPath = @"C:\Temp\UpdateSchema";


        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestChangeStringSize(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                TestChangeStringSize(connectionString, "ChangeSize", 87);
                TestChangeStringSize(connectionString, "MaxString", 42);
                TestChangeStringSize(connectionString, "DefaultSize", 57);
                TestChangeStringSize(connectionString, "ChangeSize", 100);
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestDeleteColumn(string connectionString)
        {
            if (!connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring))
            {
                TestDeleteColumn(connectionString, "DeleteColumns", "DeleteMe", "DoNotDeleteMe");
            }
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            Directory.CreateDirectory(tempPath);
            //RegisterXpoProviders.Register();
        }

        [Test]
        [ExpectedException(typeof(UnableToOpenDatabaseException))]
        public void TestInvalidConnectionStringDataSource()
        {
            string connectionString = @"XpoProvider=MSSqlServer;data source=InvalidServer\InvalidInstance;"
                + "integrated security=SSPI;initial catalog=Invalid Catalog;Pooling=false";
            using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
            {
                Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(ModifyStringSize)) };
                bool result = updateSchema.Update(assemblies);
                GenerateReport(updateSchema, "InvalidConnectionStringDataSource", null);
                Assert.That(result, Is.False);
            }
        }

        [Test]
        [ExpectedException(typeof(CannotFindAppropriateConnectionProviderException))]
        public void TestReallyInvalidConnectionString()
        {
            string connectionString = "ConnectionString=Invalid";
            using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
            {
                Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(ModifyStringSize)) };
                bool result = updateSchema.Update(assemblies);
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestRenameColumn(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                TestRenameColumn(connectionString, "RenameColumns", "RenameMe", "BadColumnName", true);
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestUpdateWithNoInitialDatabase(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);
                IServicesHelper servicesHelper = new DefaultServicesHelper();

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);

                try
                {
                    Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(ModifyStringSize)) };
                    using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
                    {
                        bool result = updateSchema.Update(assemblies);
                        GenerateReport(updateSchema, "UpdateWithNoInitialDatabase", null);
                        Assert.That(result, Is.True);
                    }

                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        foreach (string tableName in UpdateSchemaHelper.TableNames(assemblies))
                        {
                            bool tableExists = updateSchemaProvider.TableExists(tableName);
                            Assert.That(tableExists, Is.True);
                        }
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        [RowTest]
        [Row(TestConnectionString.msSqlConnectionString)]
        [Row(TestConnectionString.accessConnectionString)]
        [Row(TestConnectionString.vistaDB3ConnectionString)]
        [Row(TestConnectionString.asaConnectionString)]
        public void TestUpdateWithNoItemsToUpdate(string connectionString)
        {
            if (!(connectionString.StartsWith(TestConnectionString.ignoreProviderConnectionstring)))
            {
                Type type = TestHelper.GetProviderType(connectionString);
                IServicesHelper servicesHelper = new DefaultServicesHelper();

                // just in case the database exists at the start of the cycle.
                TestHelper.DropDatabase(type, connectionString);

                Assembly[] assemblies = new Assembly[] { Assembly.GetAssembly(typeof(ModifyStringSize)) };
                // First, create the schema through XPO.
                TestHelper.XpoCreateSchema(assemblies, connectionString);
                try
                {
                    using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
                    {
                        bool result = updateSchema.Update(assemblies);
                        GenerateReport(updateSchema, "UpdateWithNoItemsToUpdate", null);
                        Assert.That(result, Is.True);
                    }

                    using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                    {
                        updateSchemaProvider.Open();
                        foreach (string tableName in UpdateSchemaHelper.TableNames(assemblies))
                        {
                            bool tableExists = updateSchemaProvider.TableExists(tableName);
                            Assert.That(tableExists, Is.True);
                        }
                    }
                }
                finally
                {
                    Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
                }
            }
        }

        private void GenerateReport(UpdateSchema updateSchema, string testName, string additional)
        {
#if (SaveReports)
            UpdateSchemaReports report = new UpdateSchemaReports(updateSchema.UpdateSchemaLogger.DataLayer);
            string xpoProviderName;
            if (!UpdateSchemaProviderHelper.TryGetXpoProvider(updateSchema.ConnectionString, out xpoProviderName))
                xpoProviderName = "Unknown";
            string fileName;
            if (String.IsNullOrEmpty(additional))
                fileName = String.Format("{0}.{1}.{2}.pdf", reportCount++, xpoProviderName, testName);
            else
                fileName = String.Format("{0}.{1}.{2}.{3}.pdf", reportCount++, xpoProviderName, testName, additional);
            fileName = Path.Combine(tempPath, fileName);
            if (File.Exists(fileName))
                File.Delete(fileName);
            report.GenerateLogReportToPdf(fileName);
            fileName = Path.ChangeExtension(fileName, "xml");
            if (File.Exists(fileName))
                File.Delete(fileName);
            updateSchema.SaveLog(fileName);
#endif // (SaveReports)
        }

        private void TestChangeStringSize(string connectionString, string columnName, int interimSize)
        {
            Type type = TestHelper.GetProviderType(connectionString);

            // just in case the database exists at the start of the cycle.
            TestHelper.DropDatabase(type, connectionString);
            IServicesHelper servicesHelper = new DefaultServicesHelper();

            Type[] types = new Type[] { typeof(ModifyStringSize) };
            string tableName = typeof(ModifyStringSize).Name;
            // First, create the schema through XPO.
            TestHelper.XpoCreateSchema(types, connectionString);
            try
            {
                int currentColumnSize;
                int originalColumnSize;
                using (IUpdateSchemaProvider updateSchemaProvider =
                    (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    // Now get the size of the field from the database.
                    originalColumnSize = updateSchemaProvider.GetStringColumnSize(tableName, columnName);
                    UpdateSchemaType updateSchemaType;
                    if (originalColumnSize > interimSize)
                        updateSchemaType = UpdateSchemaType.ShrinkColumnStringLength;
                    else
                        updateSchemaType = UpdateSchemaType.GrowColumnStringLength;
                    // Now we are going to change the size to of the string in the db,
                    // making the column a different size then it's SizeAttribute indicates.
                    updateSchemaProvider.ModifyStringColumnSize(UpdateSchemaOrder.BeforeXpoSchemaUpdate, new UpdateItemInfo(updateSchemaType, tableName, columnName, interimSize, servicesHelper));
                    updateSchemaProvider.ModifyStringColumnSize(UpdateSchemaOrder.AfterXpoSchemaUpdate, new UpdateItemInfo(updateSchemaType, tableName, columnName, interimSize, servicesHelper));
                    currentColumnSize = updateSchemaProvider.GetStringColumnSize(tableName, columnName);
                }
                Assert.That(currentColumnSize, Is.EqualTo(interimSize));

                // Now do the update via our UpdateSchema object. This will change the
                // column string size back to it's original size stored in the SizeAttribute.
                using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
                {
                    bool result = updateSchema.Update(types);
                    GenerateReport(updateSchema, "ChangeStringSize", tableName + "." + columnName);
                    Assert.That(result, Is.True);
                }
                using (IUpdateSchemaProvider updateSchemaProvider =
                    (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    currentColumnSize = updateSchemaProvider.GetStringColumnSize(tableName, columnName);
                }
                Assert.That(currentColumnSize, Is.EqualTo(originalColumnSize));
            }
            finally
            {
                Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
            }
        }

        private void TestDeleteColumn(string connectionString, string tableName, string deleteColumnName, string doNotDeleteColumnName)
        {
            Type type = TestHelper.GetProviderType(connectionString);
            IServicesHelper servicesHelper = new DefaultServicesHelper();

            // just in case the database exists at the start of the cycle.
            TestHelper.DropDatabase(type, connectionString);
            try
            {
                Type[] types = new Type[] { typeof(DeleteColumns) };
                // First, create the schema through XPO.
                TestHelper.XpoCreateSchema(types, connectionString);
                using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, doNotDeleteColumnName), Is.True);
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, deleteColumnName), Is.True);
                }

                // Now do the update via our UpdateSchema object. This will delete the
                // column.
                using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
                {
                    bool result = updateSchema.Update(types);
                    GenerateReport(updateSchema, "DeleteColumn", tableName + "." + deleteColumnName);
                    Assert.That(result, Is.True);
                }

                using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, doNotDeleteColumnName), Is.True);
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, deleteColumnName), Is.False);
                }
            }
            finally
            {
                Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
            }
        }

        private void TestRenameColumn(string connectionString, string tableName, string columnName, string oldColumnName, bool expectedResult)
        {
            Type type = TestHelper.GetProviderType(connectionString);
            IServicesHelper servicesHelper = new DefaultServicesHelper();

            // just in case the database exists at the start of the cycle.
            TestHelper.DropDatabase(type, connectionString);
            try
            {
                Type[] types = new Type[] { typeof(RenameColumns) };
                // First, create the schema through XPO.
                TestHelper.XpoCreateSchema(types, connectionString);

                using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    // Now get the ensure that the columns are there.
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, columnName), Is.True);
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, oldColumnName), Is.False);
                    // now rename the column to the old column name.
                    updateSchemaProvider.RenameColumn(UpdateSchemaOrder.BeforeXpoSchemaUpdate, new UpdateItemInfo(UpdateSchemaType.RenameColumn, tableName, oldColumnName, columnName, servicesHelper));
                    updateSchemaProvider.RenameColumn(UpdateSchemaOrder.AfterXpoSchemaUpdate, new UpdateItemInfo(UpdateSchemaType.RenameColumn, tableName, oldColumnName, columnName, servicesHelper));
                }

                using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, oldColumnName), Is.True);
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, columnName), Is.False);
                }

                // Now do the update via our UpdateSchema object. This will change the
                // column name back to it's original name from the old column which is
                // stored in the RenameColumnAttribute.
                using (UpdateSchema updateSchema = new UpdateSchema(connectionString))
                {
                    bool result = updateSchema.Update(types);
                    GenerateReport(updateSchema, "RenameColumn", tableName + "." + columnName);
                    Assert.That(result == expectedResult);
                }

                using (IUpdateSchemaProvider updateSchemaProvider = (IUpdateSchemaProvider)Activator.CreateInstance(type, new object[] { connectionString, servicesHelper }))
                {
                    updateSchemaProvider.Open();
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, columnName), Is.True);
                    Assert.That(updateSchemaProvider.ColumnExists(tableName, oldColumnName), Is.False);
                }
            }
            finally
            {
                Assert.That(TestHelper.DropDatabase(type, connectionString), Is.True);
            }
        }
    }
}
